C  ====================================================================
C
C      Library  IDA   :  Inelastic Data Analysis
C      Modul    i73   :     density of states pp.
C
C  ====================================================================

C     Contents :
C            SEGconv, DOSconv, DOS

C  --------------------------------------------------------------------
      SUBROUTINE SEGconv (nJlist, JList, Fehler)
C  --------------------------------------------------------------------
            ! JWu 7jul93. 
         ! Switch between Vercors group and rest of the world.

      IMPLICIT REAL *8 (a-h,o-p,r-z)
      IMPLICIT LOGICAL (q)

      INCLUDE 'i_dim.f'
      INCLUDE 'i_wrk.f'
      INCLUDE 'l_def.f'

      CHARACTER*(*)  Fehler
      INTEGER        JList(*)

      CHARACTER      aus*80
      CHARACTER*40   coX, unX
 
      DO lj = 1, nJList
         j = JList(lj)

         CALL OlfHeadDup (j, .true., jout, nK, Kout, Fehler)
         IF (Fehler.ne.'&ff') RETURN
         CALL OlfCnuG (j, 'x', coX, unX, Fehler)
         IF (Fehler.ne.'&ff') RETURN

         IF (coX.ne.'w' .and. coX.ne.'f') THEN
            CALL Compose2 (aus, ' x-coordinate is '//coX, ' - Continue ?')
            IF (.not.qAsk(aus)) THEN
               Fehler = ' '
               RETURN
               ENDIF
            ENDIF

         iSEG = iOlfG (j, '@sam-erg-gain', Fehler)
         IF (Fehler.ne.'&ff' .or. iabs(iSEG).ne.1) THEN 
            Fehler = '&ff'
            IF (lj.gt.1) THEN
               qSEG = qAskD(' Sign convention yet undefined: '//
     *                      ' does E>0 mean sample energy gain', intq(qSEG))
            ELSE
               qSEG = qAsk (' Sign convention yet undefined: '//
     *                      ' does E>0 mean sample energy gain ?')
               ENDIF
            iSEG = -1 + 2*intq(qSEG)
            qInv = .false.
         ELSE
            IF     (iSEG.eq.1) THEN
               Print *, ' E>0 means sample energy gain' 
            ELSEIF (iSEG.eq.-1) THEN
               Print *, ' E>0 means sample energy loss' 
            ELSE
               Fehler = 'PROGRAM ERROR/ sign undefined/ should not arrive here'
               RETURN 
               ENDIF
            IF (lj.gt.1) THEN
               qInv = qAskD(' Invert axis', intq(qInv))
            ELSE
               qInv = qAsk (' Invert axis ?')
               ENDIF
            IF (qInv) iSEGin = -iSEGin
            ENDIF

         CALL iOlfP (j, '@sam-erg-gain', iSEG, Fehler)
         IF (Fehler.ne.'&ff') RETURN

         ! info only :
         qSym = qOlfG (j, '?det-bal-sym', Fehler)
         IF (Fehler.ne.'&ff') THEN
            Fehler = '&ff'
            IF (lj.le.1) THEN
               qSymPut = qAsk ('Declare the spectra as det-bal symmetrized ?')
            ELSE
               qSymPut = qAskD('Declare the spectra as det-bal symmetrized ?',
     *                         intq(qSymPut))
               ENDIF
            CALL iOlfP (j, '?det-bal-sym', intq(qSymPut), Fehler)
         ELSEIF (qSym) THEN
            Print *, ' data are det-bal symmetrized'
         ELSE
            Print *, ' data aren''t det-bal symmetrized'
            ENDIF

         IF (qInv) THEN
            DO K = 1, nK
               CALL OlfGetXYD (j, K, n, X, Y, D, Fehler)
               IF (Fehler.ne.'&ff') RETURN
               DO i = 1, n
                  X(i) = X(-i)
                  ENDDO
               CALL OlfPutXYD (jout, K, n, X, Y, D, Fehler)
               IF (Fehler.ne.'&ff') RETURN
               ENDDO
            ENDIF

         CALL OlfClos (jout, nK, Fehler)
         IF (Fehler.ne.'&ff') RETURN

         ENDDO ! lj

      END ! SEGconv

C  --------------------------------------------------------------------
      SUBROUTINE DOSconv (nJlist, JList, qOv, Fehler)
C  --------------------------------------------------------------------
            ! JWu 20aug91, 28apr92. Formerly OprSpecial
            ! Rewritten with integer exponents 7/8jul94.
         ! Conversions between S,S~,g,G,Q,...
         ! Intermediate function is S(q,w)
            ! divide input by factor^ifacI
            ! then multiply by factor^ifacO

      IMPLICIT REAL *8 (a-h,o-p,r-z)
      IMPLICIT LOGICAL (q)

      INCLUDE 'i_dim.f'
      INCLUDE 'i_wrk.f'
      INCLUDE 'l_def.f'

      CHARACTER*(*) Fehler
      INTEGER       JList(*)
      REAL*8        yQ(MC), yA(MC), yB(MC)
      CHARACTER*80  aus, doc, ord, h1, h2
      CHARACTER*40  CoXin, UnXin, CoYin, UnYin, CoYout, UnYout
      DATA          AtNu / 1.d0 /

C  Menu :
      Print *, ' Convert input y(x,z) to'
      Print *, '    (1)  S(q,w)             (2)  S~(q,w)'
      Print *, '    (3)  g_1(w)             (4)  g_(1)(w)'
      Print *, '    (5)  G_1(w)             (6)  G_(1)(w)'
      Print *, '    (7)  c(w)               (8)  c(w)/T^3'
      Print *, '    (9)  u^2(w)            (10)  X''''(q,w)'
      Print *, '   (11)  q(2th,w)          (12)  exp(-2W)'
      Print *, '   (13)  1/n(w;T)          (14)  1/n~(w;T)'
c      Print *, '   (15)  b/n(w;T)          (16)  b/n~(w;T)'
c      Print *, '   (17)  3/a*b/n(w;T)      (18)  3/a*b/n~(w;T)'
      iOS = iAskDMu (' Enter option', iOS, 0, 16)
      IF (iOS.le.0) THEN
         Fehler = ' '
         RETURN
         ENDIF

C  Loop files :
      DO lj = 1, nJList
      j = JList(lj)

         CALL OlfHeadDup (j, qOv, jout, nK, Kout, Fehler)
         IF (Fehler.ne.'&ff') RETURN
         CALL OlfCnuG (j, 'x', CoXin, UnXin, Fehler)
         CALL OlfCnuG (j, 'y', CoYin, UnYin, Fehler)
         Print *
         Print *,  ' convert file '//cr2(j)//' :  '//CoYin
   
         iSEG = iOlfG (j, '@sam-erg-gain', Fehler)
         IF (Fehler.ne.'&ff') RETURN
         IF (iabs(iSEG).ne.1) THEN
            Print *, ' ivalid iSEG = ', iSEG
            Fehler = ' sign of energy unknown/ run _pm first'
            RETURN
            ENDIF
 
         IF (CoXin.ne.'w' .and. CoXin.ne.'w/2pi' .and. CoXin.ne.'f') THEN
            Print *, ' input x coordinate = '//CoXin
            IF (.not.qAsk(' Continue ?')) THEN
               Fehler = ' '
               RETURN
               ENDIF
            ENDIF
         CALL UnitConv (UnXin, 'meV', facX, Fehler)
         IF (Fehler.ne.'&ff') RETURN
   
C  Analyse output coordinate :
   
         ! Default settings :
         iSymO = 0
         mOrdO = 0
         idwfO = 0
         iBosO = 0
         iBetO = 0
         iQQQO = 0 ! just Q
         iCCCO = 0 ! all we need for c(w)
         iTTTO = 0 ! divide by T^3
         iUUUO = 0
         qNewY = .false.
         UnYout= UnYin
   
         IF     (iOS.eq.1) THEN
            CoYout  = 'S(q,w)'
         ELSEIF (iOS.eq.2) THEN
            iSymO = 1
            CoYout  = 'S~(q,w)'
         ELSEIF (iOS.eq.3) THEN
            CoYout  = 'g_1(w)'
            mOrdO = 1
            idwfO = 1
            iBosO = 1
         ELSEIF (iOS.eq.4) THEN
            CoYout  = 'g_(1)(w)'
            mOrdO = 1
            iBosO = 1
         ELSEIF (iOS.eq.5) THEN
            CoYout  = 'G_1(w)'
            mOrdO = 1
            idwfO = 1
         ELSEIF (iOS.eq.6) THEN
            CoYout  = 'G_(1)(w)'
            mOrdO = 1
         ELSEIF (iOS.eq.7) THEN
            CoYout  = 'c(w)'
            mOrdO = 1
            iCCCO = 1
         ELSEIF (iOS.eq.8) THEN
            CoYout  = 'c(w)/T^3'
            mOrdO = 1
            iCCCO = 1
            iTTTO = 1
         ELSEIF (iOS.eq.9) THEN
            CoYout  = 'u^2(w)'
            iUUUO = 1
         ELSEIF (iOS.eq.10) THEN         ! chi'' as suggested by a PRL referee
            CoYout = 'Chi''''' 
            UnYout = ' '
            iBosO  = 1
            iBetO  = -1
         ELSEIF (iOS.eq.11) THEN         ! supersede y by Q(x) :
            CoYout  = 'q(2th,w)'
            UnYout  = 'A-1'
            qNewY = .true.
            iQQQO = 1
         ELSEIF (iOS.eq.12) THEN         ! supersede y by DWF :
            CoYout  = 'exp(-2W)'
            UnYout  = ' '
            qNewY = .true.
            idwfO = 1
         ELSEIF (iOS.eq.13) THEN         ! supersede y 
            CoYout  = '1/n(w;T)'
            UnYout  = ' '
            qNewY = .true.
            iSymO = 0
            iBosO = 1
            iBetO =-1
         ELSEIF (iOS.eq.14) THEN         ! supersede y 
            CoYout  = '1/n~(w;T)'
            UnYout  = ' '
            qNewY = .true.
            iSymO = 1
            iBosO = 1
            iBetO =-1
         ELSEIF (iOS.eq.15) THEN         ! supersede y
            CoYout  = 'b/n(w;T)'
            UnYout  = ' '
            qNewY = .true.
            iSymO = 0
            iBosO = 1
            iBetO = 0
         ELSEIF (iOS.eq.16) THEN         ! supersede y
            CoYout  = 'b/n~(w;T)'
            UnYout  = ' '
            qNewY = .true.
            iSymO = 1
            iBosO = 1
            iBetO = 0
         ELSE
            Fehler = ' Option not yet implemented'
            RETURN
            ENDIF
   
C  More questions about the output :
         IF (iCCCO.ne.0) THEN !  for c(T) 
            AtNu = rAskD (' Number of atoms per molecule', AtNu)
            ENDIF
   
C  Analyse input y coordinate -> set exponents, mOrdI, ...
   
         ! Default setting of exponents :
         mOrdI = 0 ! no factor alpha/3
         iSymI = 0 ! not det-bal symmetrized
         idwfI = 0 ! no Debye Waller factor
         iBosI = 0 ! no Bose factor (n(beta)/beta)
         iBetI = 0 ! no beta factor

C  Auxiliary calculations :
         facI = 1.
         DO m = 1, mOrdI
            facI = facI * m
            ENDDO

C  No conversion of input y ?
         IF (qNewY) THEN
            idwfI = 0
            isymI = 0
            iBosI = 0
            aM = 1. ! needed for CalcAB (..yA..) even if yA isn't needed
   
C  Anaylse the input :
         ELSE
 112        CONTINUE 
            CALL FindN (CoYin, 1, niIn, iIn)
      
            IF (CoYin(1:1).eq.'S' .or. CoYin(1:1).eq.'I' ) THEN
               iSymI = iOlfG (j, '?det-bal-sym', Fehler)
               IF (Fehler.ne.'&ff') RETURN
            ELSEIF (CoYin(1:3).eq.'X''''') THEN ! corrected 25jan00
               iBosI = 1
               iBetI = -1
            ELSEIF (CoYin(1:2).eq.'g(') THEN
               idwfI = intq(qAsk(' Multiply input by dwf (not for Placzek) ?'))
               mOrdI = 1
               iBosI = 1
            ELSEIF (CoYin(1:4).eq.'g_#(') THEN
               mOrdI = 1 ! ignore iIn -> convert as if it were g_1
               idwfI = 1
               iBosI = 1
            ELSEIF (CoYin(1:6).eq.'g_(#)(') THEN
               mOrdI = 1 ! ignore iIn -> convert as if it were g_1
               iBosI = 1
            ELSEIF (CoYin(1:2).eq.'G(') THEN
               mOrdI = 1
               idwfI = intq(qAsk(' Multiply input by dwf (not for Placzek) ?'))
            ELSEIF (CoYin(1:4).eq.'G_#(') THEN
               mOrdI = iIn
               idwfI = 1
            ELSEIF (CoYin(1:6).eq.'G_(#)(') THEN
               mOrdI = iIn
            ELSE
               Print *, ' Input y coordinate = '//CoYin(1:len(CoYin))
               aus = ' Is of type S(q,w), g(w), G(w), ...'
               CALL Gong (1)
               CALL FrageC (aus, CoYin)
               IF (CoYin.eq.' ') THEN
                  Fehler = ' '
                  RETURN
                  ENDIF
               GOTO 112
               ENDIF
   
            ! - displacement :
            IF (idwfI.ne.idwfO) THEN
               IF (lj.eq.1) 
     *            qDM = qAskD (' Displacement from Debye-model', intq(qDM))
               IF (qDM) THEN
                  TD = rAskD ( ' Debye temperature', TD)
                  T0 = 4. ! normalization temperature
                  CALL NiceNum (TD, h2, i2)
                  CALL Append (doc, '; TD='//h2)
               ELSE
                  u2T = rAskD (' Displacement <ux^2>(T) ', u2T)
                  u20 = rAskD (' Of which offset <ux^2>(0)', u20)
                  CALL NiceNum (u2T, h2, i2)
                  CALL Append (doc, '; u^2='//h2)
                  CALL NiceNum (u20, h2, i2)
                  CALL Append (doc, ','//h2)
                  ENDIF
               ENDIF

            ENDIF 
   
         Print *,  ' -> '//CoYout
         CALL OlfCnuP (jout, 'y', CoYout, UnYout, Fehler)
         CALL Compose2 (doc, 'dosconv/ '//CoYin, ' -> '//CoYout)
         CALL OlfComAdd (jout, 'O', doc, Fehler)
         CALL iOlfP (jout, '?det-bal-sym', iSymO, Fehler)
   
C  DEBUG :
         Print '(a,2i3)', ' Conversion I->O : sym ', iSymI, iSymO
         Print '(a,2i3)', ' Conversion I->O : dwf ', idwfI, idwfO
         Print '(a,2i3)', ' Conversion I->O : ord ', mOrdI, mOrdO
         Print '(a,2i3)', ' Conversion I->O : bos ', iBosI, iBosO
         Print '(a,2i3)', ' Conversion I->O : bet ', iBetI, iBetO
   
C  Loop spectra :
         DO K = 1, nK
            CALL OlfGetXYD (j, K, nC, X, Y, D, Fehler)
            IF (mOrdI.ne.mOrdO .or. idwfI.ne.idwfO .or.
     *           iQQQO.eq.1 .or. iUUUO.eq.1) CALL CalcQQ (j, K, yQ, nC, Fehler)

            T  = rzOlfGG (j, K, 'T', 'K', Fehler)
            IF (Fehler.ne.'&ff') RETURN

            IF (Fehler.ne.'&ff') RETURN
            IF (T.lt.0.1) THEN ! limit was 10 
               Fehler = 'temperature too low'
               RETURN  ! else crash in dexp(yB/) 11mai93
               ENDIF
   
C  Atomic masses needed ?
            IF (mOrdI.ne.mOrdO) THEN
               aM = rOlfGG (j, 'at-mass', 'amu', Fehler)
               IF (Fehler(1:4).eq.'&pnf') THEN
                  Fehler = 'please define r-parameter "at-mass"'
                  ENDIF
               IF (Fehler.ne.'&ff') RETURN
               IF (aM.le.0.) THEN
                  Fehler = 'at-mass <= 0'
                  RETURN
                  ENDIF
               c2M   = aM * 931.48d9  ! amu * c^2 -> meV
               chbar = 1.97329d6      ! hbar * c [meV * A]
               DO i = 1, nC
                  yA(i) = chbar**2 * yQ(i)**2 / ( 2 * c2M * TkB)
                  ENDDO
               ENDIF

            TkB   = T  * .0861733 ! K -> meV
            DO i = 1, nC
               yB(i) = X(i)*facX / TkB
               ENDDO

C  Displacements from Debye temperature ?
            IF (idwfI.ne.idwfO) THEN
               ix0 = irPos (X, nC, 0.d0, 'n')
               yQ0 = yQ(ix0)
               IF (qDM) THEN
                  u2T = u2Debye (T,  TD, aM, 1.d-3)
                  u20 = u2Debye (T0, TD, aM, 1.d-3)
                  IF (K.eq.1) THEN
                     ! notify :
                     Print '(1x,2(a,f5.1),2(a,f7.5))',
     *                  ' TD = ', TD, ' T = ', T,
     *                  ' ==> u2(0) = ', u20, ', u2(T) = ', u2T
                     ENDIF
                  ENDIF
               ENDIF
   
C  Conversion yIn -> S(q,w) :
   
            DO i = 1, nC
   
               ! Prepare fact :
               fact = 1 ! out = in * fact
   
               IF (isymI.ne.isymO)
     *            fact = fact * dexp((iSymI-iSymO)*iSEG * yB(i)/2)
   
               IF (idwfI.ne.idwfO)
     *            fact = fact * 
     *                   dexp((idwfI-idwfO)*(-u2T*yQ(i)**2 + u20*yQ0**2))
               
               IF (mOrdI.ne.mOrdO)
     *            fact = fact * dpowi(yA(i)/3, mOrdI-mOrdO) / facI

               IF (iBosI.ne.iBosO) 
     *            fact = fact * dpowi(-iSEG*yB(i)*(dexp2(-iSEG*yB(i))-1),
     *                                iBosO-iBosI)

               IF (iBetI.ne.iBetO) 
     *            fact = fact * dpowi(-iSEG*yB(i), iBetO-iBetI)
   
               IF (iQQQO.eq.1)
     *            fact = fact * yQ(i)
   
               IF (iCCCO.eq.1)
     *            fact = fact * 8.3144 * AtNu * yB(i)**3 * (-iSEG) *
     *                   dquot0 (1.d0, 1-dexp2(iSEG*yB(i)))
   
               IF (iTTTO.eq.1)
     *            fact = fact / T**3
   
               IF (iUUUO.eq.1)
     *            fact = fact / yQ(i)**2 * (dexp2(-iSEG*yB(i)) + 1)
   
               ! Convert data (or set y) :
               IF (qNewY) THEN
                  Y(i) = fact
                  D(i) = 0
               ELSE 
                  Y(i) = fact * Y(i)
                  D(i) = fact * D(i)
                  ENDIF
               ENDDO
   
            CALL OlfCopZ (j, jout, K, K, Fehler)
            CALL OlfPutXYD (jout, K, nC, X, Y, D, Fehler)
            IF (Fehler.ne.'&ff') RETURN
   
            ENDDO ! K
   
         CALL OlfClos (jout, nK, Fehler)
   
         ENDDO ! lj

      END ! DOSconv

C  --------------------------------------------------------------------
      SUBROUTINE DOS (nJList, JList, Fehler)
C  --------------------------------------------------------------------
            ! JWu 14-?nov91. Revision 5-13feb92.
            ! Placzek-Expansion 30-31mar92.
            ! Arbitrary order 27-28apr92.
            ! Extrapolation from larger region 7feb95

         ! Iterative multiphonon subtraction for the
         ! density of states g(w).
         ! Before use, interpolate to constant w with X(1)=0.

            ! js        = uncorrected S(q,w)                        ! Y0
            ! jg        = 1-Phonon guess                            ! Y, Y(,1)
            ! JJ(2..mMcal) = m-Phonon contribution calculated from jg  ! Y(,m)
            ! JJ(1)     = new 1-Phonon guess = js - JJ(2) - ..      ! Y(,1)

      IMPLICIT REAL*8  (a-h,o-p,r-z)
      IMPLICIT LOGICAL (q)

      INCLUDE 'i_dim.f'
      INCLUDE 'i_wrk.f'
      INCLUDE 'l_def.f'

      PARAMETER        (MaxJ=10)

      CHARACTER*(*)  Fehler
      INTEGER        JList(*)

      REAL*8            yQ(MC), yA(MC), yB(MC), BoN(MC),
     *                  X0(MC), Y0(MC), D0(MC),
     *                  G0(-MC:MC), Gc(-MC:MC), Gm(MC),
     *                  Fak(0:MaxJ), Bin(0:MaxJ,0:MaxJ),
     *                  Gu2M(0:MaxJ), YY(MC,MaxJ)
      INTEGER           JJ(MaxJ), NConv(MaxJ), KKG(MK)
      LOGICAL           qLisK(MK), qLisKold(MK)

      CHARACTER*80      aus, aus2, h, cLisKg, cLisKd, ord, yGuess
      CHARACTER*40      Co, Un

      DATA              iExtra /1/, ig /1/, mMcal /5/, cofa /0.8d0/,
     *                  goalMatch /6.d0/, nloops /12/,
     *                  cLisKg /'1'/, cLisKd /'*'/,
     *                  qPlaczek /.false./, qu2Fix /.false./,
     *                  iSave /1/, qSaveExp /.false./

C  Set m! and (m over k) :
      Fak(0) = 1.
      DO i = 1, MaxJ
         Fak(i) = Fak(i-1) * i
         DO ii = 0, i
            Bin(i,ii) = Fak(i) / Fak(ii) / Fak(i-ii)
            ENDDO
         ENDDO

C  Global Setup :
      qPlaczek= qAskDi (' Multiphonon (0) or Placzek (1) expansion',
     *                     intq(qPlaczek))

      iSave   = iAskDMu (
     *   ' Save as g_(q,w) (1) or as G_(w) (2) (for m>1)', iSave, 1, 2)
      qSaveExp= qAskD (
     *   ' Save 1-phonon-term as (exp minus corr)', intq(qSaveExp))

C  Loop over input files :
      DO lj = 1, nJList
         jin = JList(lj)
     
         ! create new output files :
         DO m = 1, MaxJ
            CALL OlfHeadDup (jin, .false., JJ(m), nK, Kout, Fehler)
            IF (Fehler.ne.'&ff') RETURN
            ENDDO
   
C  Input files - S(q,w) :
         Print *, ' Uncorrected S(q,w) from file '//cl2(jin)
         CALL OlfCnuG (jin, 'y', Co, Un, Fehler)
         IF (Fehler.ne.'&ff') RETURN
         IF (Co(1:1).ne.'S') THEN
            Fehler =  'y-Coord is not S(..) but '//Co
            RETURN
            ENDIF

         CALL OlfCnuG (jin, 'x', Co, Un, Fehler)
         IF (Fehler.ne.'&ff') RETURN
         IF (Co.ne.'w') THEN
            Print *, ' input x coordinate = '//Co
            IF (.not.qAsk(' Continue ?')) THEN
               Fehler = ' '
               RETURN
               ENDIF
            ENDIF
         CALL UnitConv (Un, 'meV', facX, Fehler)
         IF (Fehler.ne.'&ff') RETURN

         aM  = rOlfGG (jin, 'at-mass', 'amu', Fehler) ! for g-scale, <u^2>,...
         IF (Fehler.ne.'&ff') RETURN
         CALL NiceNum (aM, h, ih)
         Print *, ' atomic mass = '//h(1:ih)

         TS  = rOlfGG (jin, 'T', 'K', Fehler)
         IF (Fehler.ne.'&ff') RETURN
   
         iSEG = iOlfG (jin, '@sam-erg-gain', Fehler)
         IF (Fehler.ne.'&ff') RETURN
         IF (iSEG.ne.-1) THEN
            Fehler = ' works only for neutron energy gain convention'
            RETURN
            ENDIF

         qSym = qOlfG (jin, '?det-bal-sym', Fehler)
         IF (Fehler.ne.'&ff') RETURN
   
C  Input files - g(w) guess ?
         aus = ' Guess for g(w) from file no. (0=from S, -1=escape)'
         jg = iAskDMu (aus, jg, -1, MemBlockInq('nF'))
         IF (jg.eq.-1) THEN
            jg = 0
            Fehler = ' '
            RETURN
            ENDIF 
         IF (jg.ge.1) THEN
            ! Check parameters :
            nKold = iOlfG (jg, '#spectra', Fehler)
            IF (Fehler.ne.'&ff') RETURN
            IF (nKold.gt.nK) THEN
               Fehler = ' guess contains too many spectra'
               RETURN
            ELSEIF (nKold.lt.nK) THEN
 132           CONTINUE
               CALL Compose2 (aus, ' Name the '//cl3(nKold),
     *            ' spectra for which a guess is given')
               CALL GetNList (aus, cLisKg, qLisKold, nK)
               IF (iqSum(qLisKold,nK).eq.0)     GOTO  92
               IF (iqSum(qLisKold,nK).ne.nKold) GOTO 132
               ENDIF
            CALL OlfCnuG (jg, 'y', Co, Un, Fehler)
            IF (Fehler.ne.'&ff') RETURN
            IF     (Co(1:1).eq.'g') THEN 
               yGuess = 'g'
            ELSEIF (Co(1:1).eq.'G') THEN
               yGuess = 'G'
            ELSE
               Fehler = ' y-coord of 1-phonon guess is neither g(..) nor G(..)'
     *                   //' but '//Co
               RETURN
               ENDIF
         ELSE ! jg=0
            nKold = nK
            ENDIF
   
C  Loop settings :
         qWorked = .false.
 10      CONTINUE
         
         CALL rAskPair (
     *      ' Offset <u2(0)> (always fixed) and initial/fixed <u2(T)>',
     *                   u20, u2T, u20, u2T)
         IF (u2T.ne.0) qu2Fix  = qAskD   (' Keep <u2> fixed', intq(qu2Fix))
   
         CALL rAskRge (' Low w extrapolation from average over region',
     *        wOnset, wAvge, wOnset, wAvge)
         wDebye = rAskDMu (' Fixed Debye energy (else 0)', wDebye, 0.d0, 1.d10)
         wCut = rAskDMu (' High w cut-off', wCut, wAvge, 1.d10)
   
         ig      = iAskDMu (' Grouping for convolution ', ig, 0, MC/4)
         IF (ig.eq.0) GOTO 92 ! emergency exit
   
         IF (mMsav.eq.0) mMsav = mMcal
 113     CONTINUE 
         CALL i2FrageD (' Highest order internally / on exit',
     *        mMcal, mMsav, mMcal, mMsav)
         IF (.not.(1.lt.mMsav .and. mMsav.le.mMcal .and. mMcal.le.MaxJ)) THEN
            CALL Gong (3)
            GOTO 113
            ENDIF
   
         goalMatch = rAskDMu (' Required precision (-lg)', 
     *                        goalMatch, 0.d0, 3.d1)
         aus     = ' Convergence factor (fraction of second but last g_1)'
         cofa    = rAskDMu (aus, cofa, 0.d0, 1.d0)
         cu      = cofa
   
         aus=' Maximum number of iterations (-1=quit, 0=correct settings)'
         nLin = iAskDMu (aus, nloops, -1, 100)
         IF (nLin.eq.-1) GOTO 92 ! Notausstieg
         IF (nLin.eq. 0) GOTO 10 ! settings
         nloops = nLin
   
         IF (nK.gt.1) THEN
            CALL GetNList (' Iterations for spectra', cLisKd, qLisK, nK)
            nKout = iqSum (qLisK,nK)
            IF (nKout.le.0) GOTO 92 ! Notausstieg
         ELSE
            qLisK(1) = .true.
            nKout    = 1
            ENDIF
   
         ! assignement of spectra j <-> jg :
         IF (nKold.lt.nK) THEN
            DO K = 1, nK
               IF (qLisKold(K)) THEN
                  Kin = K
                  GOTO 134
                  ENDIF
               ENDDO
 134        CONTINUE
            DO K = 1, nK
               IF (qLisKold(K)) Kin = K
               KKG(K) = Kin
               ENDDO
         ELSE
            DO K = 1, nK
               KKG(K) = K
               ENDDO
            ENDIF
   
C  Now that mMsav is given we may open the output files :
         ! half a doc line about the iteration's setting :
         CALL NiceNum (wOnset, h, ih)
         aus2 = 'w = '//h(1:ih)
         CALL NiceNum (wAvge, h, ih)
         CALL Append (aus2, '/'//h(1:ih))
         CALL NiceNum (wCut, h, ih)
         IF (qPlaczek) THEN
            CALL Append (aus2, '-'//h(1:ih)//',  m <='//cl3(mMcal))
         ELSE
            CALL Append (aus2, '-'//h(1:ih)//', (m)<='//cl3(mMcal))
            ENDIF
            IF (wDebye.gt.0) THEN 
               CALL NiceNum (wDebye, h, ih)
               CALL Append (aus, ' wD='//h(1:ih))
               ENDIF 
         IF (ig.gt.1) CALL Append (aus2, ' ig='//cl3(ig))
   
         ! loop over output files :
         DO m = 1, mMsav
   
            ! coordinate name :
            IF     (m.eq.1 .and. qSaveExp) THEN
               ord = ' '
            ELSEIF (qPlaczek) THEN
               CALL Compose2 (ord, '_('//cl2(m), ')')
            ELSE
               ord = '_'//cl2(m)
               ENDIF
            IF     (iSave.eq.1) THEN
               CALL Compose2 (Co, 'g'//ord, '(q,w)')
            ELSE
               CALL Compose2 (Co, 'G'//ord, '(w)')
               ENDIF
            CALL OlfCnuP (JJ(m), 'y', Co, ' ', Fehler)
  
            ! comment line :
            IF (m.eq.1 .and. qSaveExp) THEN
               aus = 'dos/ g(w) = full - g_2 - ..'
            ELSE
               CALL Compose2 (aus, cl2(m), '-phn guess')
               ENDIF
            CALL Append (aus, '/ '//aus2)
            CALL OlfComAdd (JJ(m), 'g'//cl2(m), aus, Fehler)
   
            ! linetype
c96/7            i-Par4(15) = min0 (m-1,4)
   
            CALL OlfClos (JJ(m), 0, Fehler)
            IF (Fehler.ne.'&ff') RETURN
   
            ENDDO
   
C  Loop spectra :
         Kout = 0
         DO K = 1, nK
         IF (qLisK(K)) THEN
            Kout = Kout+1
   
            CALL Compose2 (aus, ' iterations for spectrum '//cl3(K), ' :')
            IF (nKout.lt.nK) CALL Append (aus, ' output spectrum '//cl3(Kout))
            Print *, aus
   
C  Get uncorrected g(w) :
            ! open complete uncorrected S(q,w) and check scale :
               Print *, ' debug 1'
            CALL OlfGetXYD (jin, K, n, X0, Y0, D0, Fehler)
            CALL CalcQQ (jin, K, yQ, n, Fehler)
            T  = rzOlfGG (jin, K, 'T', 'K', Fehler)
            IF (Fehler.ne.'&ff') RETURN
            IF (.not.qEqEps(T,TS)) Fehler = ' problems with temperature'
            IF (Fehler.ne.'&ff') RETURN
            ! check x-scale :
               Print *, ' debug 2'
            tol = 1.d-1
            CALL CheckScale (n, X0, tol, qedX0, dX0)
            IF     (.not.qedX0) THEN
               Fehler = ' x-scale not equidistant in S(q,w), spectrum '//cl3(K)
            ELSEIF (dabs(X0(1)).ge.tol*dX0) THEN
               Fehler = ' X(1)<>0 in g(w), spectrum '//cl3(K)
               ENDIF
            IF (Fehler.ne.'&ff') RETURN
            ! Calculate Bose factor :
               Print *, ' debug 3'
c-tot            CALL CalcAB (n, T, X0, facX, yQ, aM, yA, yB)
            IF (T .le.0.) CALL Absturz ('CalcAB', 'T<=0')
            IF (aM.le.0.) CALL Absturz ('CalcAB', 'aM<=0')
     
            TkB   = T  * .0861733  ! K -> meV
            c2M   = aM * 931.48d9  ! amu * c^2 -> meV
            chbar = 1.97329d6      ! hbar * c [meV * A]
    
            DO i = 1, n
               yB(i) = X0(i)*facX / TkB
               yA(i) = chbar**2 * yQ(i)**2 / ( 2 * c2M * TkB)
               ENDDO

               Print *, ' debug 4'
            DO i = 1, n
               BoN(i) = yB(i) * (dexp(yB(i)) - 1.)  ! beta/n(w)
               IF (yA(i).le.0) THEN 
                  Fehler = ' alpha<=0 (probably Q=0)'
                  RETURN
                  ENDIF
               ENDDO
            ! transform S(q,w) into uncorrected g(w) (except for DWF(T)):
               Print *, ' debug 5'
            DO i = 1, n
               fact = 3 / yA(i) * BoN(i) * dexp1(-u20 * yQ(1)**2)
               IF (qSym) fact = fact * dexp(-yB(i)/2)
               Y0(i) = fact * Y0(i)
               D0(i) = fact * D0(i)
               ENDDO
   
            ! calculate G0(0) from wDebye :
            TkB   = T  * .0861733  ! K -> meV
            IF (wDebye.gt.0) GDebye = 9 * TkB**2 / wDebye**3
   
C  Get 1-phonon-guess :
               Print *, ' debug 6'
            IF (jg.gt.0) THEN
               Print *, ' debug 7'
               ! open 1-phonon guess :
               CALL OlfGetXYD (jg, KKG(K), ng, X, Y1, D, Fehler) ! old: zg
               IF (Fehler.ne.'&ff') RETURN
               ! check x-scale :
               CALL CheckScale (ng, X, tol, qedX, dX)
               IF     (.not.qedX) THEN
                  Fehler = ' x-scale not equidistant in g_1, spectrum '//cl3(K)
               ELSEIF (dabs(X(1)).ge.tol*dX) THEN
                  Fehler = ' X(1)<>0 in g_1, spectrum '//cl3(K)
               ELSEIF (ng.ne.n) THEN
                  Print *, ' WARNING/ different length of spectra '//cl3(K)
                  ENDIF
               IF (Fehler.ne.'&ff') RETURN
               IF     (yGuess.eq.'G') THEN
                  DO i = 1, ng
                     Y1(i) = BoN(i) * Y1(i)   ! g_1(w) <- G_1(w)
                     ENDDO
   
               ELSEIF (yGuess.eq.'S') THEN
                  Fehler = 'guess=S_ not yet implemented'
                  RETURN
                  ENDIF
               ENDIF
   
            ! wCut -> nCut :
               Print *, ' debug 8'
            nCut   = irPos (X0, n, wCut,   'r') - 1
            nOnset = irPos (X0, n, wOnset, 'r') 
            nAvge  = irPos (X0, n, wAvge,  'r') 
            IF     (nCut.le.3) THEN
               Fehler = ' Less than 4 channels in allowed x-range'
            ELSEIF (nCut.lt.2*nAvge) THEN
               Fehler = ' Extrapolating over more than half the w-range'
            ELSEIF (nOnset.le.0) THEN
               Fehler = ' low w cut-off too small, 0. not excluded'
            ELSEIF (nAvge-nOnset.lt.1) THEN 
               Fehler = ' Void region from which to extrapolate'
               ENDIF
            IF (Fehler.ne.'&ff') RETURN
            IF (nAvge-nOnset.gt.1 .and. .not.qPlaczek)
     *           Print *, ' WARNING/ in calculation of <u^2> no averaging'
   
            ! n -> NConv :
               Print *, ' debug 9'
            DO m = 1, mMcal
                NConv(m) = min0 (nCut*m, n-1)
                ENDDO
            DO m = 3, mMcal
               NConv(m) = NConv(2)  ! Sparmassnahme
               ENDDO
            nMax = NConv(mMcal)
   
            ! initialize by 0. :
            DO i = 1, n
               D (i) = 0. ! no errors for m-phonon-contributions
               DO m = 1, MaxJ
                  YY (i,m) = 0. ! for m-phonon contribution outside NConv(m)
                  ENDDO
               ENDDO
            DO i = -n, n
               G0 (i) = 0.
               ENDDO
   
C  Loop iterations :
               Print *, ' debug 10'
            niter = nloops
            iter  = 0
 101           CONTINUE
               iter = iter + 1
               Print *, ' iter : ', iter
   
               IF (.not. (iter.eq.1 .and. jg.gt.0)) ! else Y1 has been set above
     *         CALL rCopy (Y1, 1, n, 1, Y,  1, 1) ! guess < result
               CALL rCopy (Y, 1, n, 1, Y0,  1, 1) ! tractandum < raw g(w)
               CALL rCopy (D, 1, n, 1, D0,  1, 1)
   
               IF (.not.qPlaczek) THEN ! get <u^2> (simplified 7feb95)
                  IF     (qu2Fix .or. iter.eq.1) THEN
                     u2TK = u2T
                  ELSE
                     u2TK = cu*u2TK + (1-cu)*u2 ! damping with convergence factor
                     ENDIF
cdeb                  Print *, 'u2T u2TK cu u2', u2T, u2TK, cu, u2   ! <<<
                  ! correct raw g(w) by DWF :
                  DO i = 1, n
                     fact = dexp1((u2TK)*yQ(i)**2)
                     Y(i) = fact * Y(i)
                     D(i) = fact * D(i)
                     ENDDO
                  ENDIF 
   
               ! convergence factor : mix last and 2nd but last guess (12feb92)
               c = cofa
               IF (iter.eq.1) c = 0.
   
               ! guess g_1(w) -> full G0(w) :
               DO i = 2, nCut+1
                  G0( i-1) =   c  * G0( i-1) +
     *                      (1-c) * Y1(i) /  yB(i)  / (dexp( yB(i)) - 1.)
                  G0(-i+1) =   c  * G0(-i+1) +
     *                      (1-c) * Y1(i) /(-yB(i)) / (dexp(-yB(i)) - 1.)
                  ENDDO
   
               ! fill region around w=0 by extrapolation :
               IF (wDebye.le.0) THEN ! G0=const
                  Gleft  = 0
                  Gright = 0
                  DO i = nOnset, nAvge
                     Gleft  = Gleft  + G0(-i) / (nAvge-nOnset+1)
                     Gright = Gright + G0( i) / (nAvge-nOnset+1)
                     ENDDO 
                  DO i = -nOnset+1, nOnset-1
                     G0(i) = Gleft + (i+nOnset)*(Gright-Gleft)/(2*nOnset)
                     ENDDO
               ELSE ! G0 = linear
                  Gleft  = 0
                  Gright = 0
                  DO i = nOnset, nAvge
                     Gleft  = Gleft  + (G0(-i)-GDebye)/i/(nAvge-nOnset+1)
                     Gright = Gright + (G0( i)-GDebye)/i/(nAvge-nOnset+1)
                     ENDDO 
                  DO i = 0, nOnset-1
                     G0(-i) = GDebye + i * Gleft
                     G0( i) = GDebye + i * Gright
                     ENDDO 
                  ENDIF 
   
               ! copy (for iterative convolution: Gc = G0 * G0 * ..) :
               DO i = -n, n 
                  Gc(i) = G0(i)
                  ENDDO
   
C  Convolution -> YY (=G_m for multiphonon, =G_(m) for Placzek) :
   
               IF (qPlaczek) THEN
                  Gu2 =  u2TK * 3 * yQ(1)**2 / yA(1)
                  IF (Gu2.gt.1.d5) THEN
                     Print *, ' jg iter u2TK Q**2 A : ',
     *                  jg, iter, u2TK, yQ(1), yA(1)
                     Fehler = ' <u2> overflow/ G_0 > 1e5'
                     RETURN
                     ENDIF
                  Gu2M(0) = 1. ! to prevent FORTRAN error 0**0
                  DO mm = 1, mMcal
                     Gu2M(mm) = (-Gu2)**(mm)
                     ENDDO
                  DO mm = 2, mMcal
                     DO i = 1, NConv(mm)+1
                        YY(i,mm) = Bin(mm,1) * G0(i-1) * Gu2M(mm-1)
                        ENDDO
                     ENDDO
                  ENDIF
   
               DO m = 2, mMcal
                  ! convolution G0*Gc :
                  yc = 0.
                  DO i = 0, NConv(m)
                     IF (mod(i,ig).eq.0) THEN
                        yc = 0.
                        DO ii = max0 (-nCut, -NConv(m-1)+i),
     *                          min0 ( nCut,  NConv(m-1)+i), ig
                           yc = yc + G0(ii) * Gc(i-ii)
                           ENDDO
                        yc = yc * dX0 * ig
                        IF (dabs(yc).gt.1.d16) THEN
                           Print *, ' m, i, yc :', m, i, yc
                           Fehler = ' Convolution/ Arithmetic overflow imminent'
                           RETURN
                           ENDIF
                        ENDIF
                     Gm(1+i) = yc
                     ENDDO
                  DO i = 0, NConv(m)
                     Gc( i) = Gm(1+i)
                     Gc(-i) = Gm(1+i)
                     ENDDO
                  ! -> Gc = G_m
   
                  IF (qPlaczek) THEN
                     DO mm = m, mMcal
                        DO i = 1, NConv(m)+1
                           YY(i,mm) = YY(i,mm) +
     *                       Bin(mm,m) * Gc(i-1) * Gu2M(mm-m)
                           ENDDO
                        ENDDO
                  ELSE
                     DO i = 1, NConv(m)+1
                        YY(i,m) = Gc(i-1)
                        ENDDO
                     ENDIF
                  ENDDO ! m
   
C  Conversion G -> g, subtraction from g(0) :
                         ! b / n              for restoring g from G,
                         ! 1/m! * (a/3)^(m-1) prefactor in m-phonon expansion
               DO m = 2, mMcal
                  DO i = 1, NConv(m)+1
                     IF     (dabs(YY(i,m)).gt.1.d16) THEN
                        Print *, ' NConv, m, i, YY :',
     *                            NConv(m), m, i, YY(i,m)
                        Fehler = ' G->g/ Arithmetic overflow imminent'
                        RETURN
                        ENDIF
                        Y(i) = Y(i) -
     *                      BoN(i)*(yA(i)/3)**(m-1)/Fak(m) * YY(i,m)
                     ENDDO
                  ENDDO
               DO i = 1, NConv(mMcal)+1
                  Y(i) = dmax1 (Y(i), 0.d0)
                  ENDDO
   
C  Save G or G_1 ?
               IF (qSaveExp) THEN
                  DO i = 1, n
                     ii = i
                     IF (i.eq.1) ii = 2
                     YY(i,1) = Y(ii) / BoN(ii) ! g -> G
                     ENDDO
               ELSE
                  DO i = 1, NConv(1) + 1
                     YY(i,1) = G0(i-1)
                     ENDDO
                  ENDIF
   
C  Convert the YY for saving as g_ ?
               IF     (iSave.eq.1) THEN ! contribution S_m(q,w)
                  DO m = 1, mMcal
                     DO i = 1, n  ! NConv(m)+1 but n for (m=1 and qSaveExp=true)
                        YY(i,m) = BoN(i)*(yA(i)/3)**(m-1)/Fak(m)* YY(i,m)
                        ENDDO
                     ENDDO
                  ENDIF
   
C  Calculate new <u2> (30mar92) :
               u2 = 0.
               DO i = 1, nCut
                  ii = i
                  IF (ii.le.nOnset) ii = nOnset+1 ! extrapolation w->0
                  u2 = u2 + dX0 * Y(ii) / dtanh(yB(ii)/2) / X0(ii)
                  ENDDO
               u2 = u2 / 0.2393 / aM / 6   ! hbar^2/amu = meV A^2 / .2393
   
C  Estimate quality of match (6/13feb92) :
               nratio = 0
               ratio  = 0.
               box    = 0.
               squares= 0.
               igg = max0 (1, ig)
               DO i = igg, nMax, igg
                  IF (Y1(i).gt.0.) THEN
                     nratio = nratio + 1
                     ratio  = ratio + Y(i) / Y1(i)
                     ENDIF
                  box     = box     +  Y(i) * Y1(i)
                  squares = squares + (Y(i) - Y1(i))**2
                  ENDDO
               IF (nratio.le.0) THEN
                  Print *, ' old guess g_1(w) == 0.'
                  ratio = 0.
               ELSE
                  ratio = ratio / nratio
                  ENDIF
               IF (box.le.0.) THEN
                  Print *, ' new guess g_1(w) == 0. ?!'
                  squares = 0.
               ELSE
                  squares = squares / box
                  ENDIF
               squares = - dlg0 (squares)
   
C  Notify end of iteration :
               IF (iter.eq.1) Print *
               Print '(a,i3,a,f7.4,a,f4.1,a,f8.5)',
     *             '  end of iteration', iter,
     *             ', ratio =', ratio, ', -lg(match) =', squares, ', <u2>=', u2
   
               IF (squares.ge.goalMatch) THEN
                  ! '  found stationary solution'
                  GOTO 109
                  ENDIF
   
               IF (iter.ge.niter) THEN
                  aus = '  How many more iterations (-1=settings)'
                  nmore = iAskDMu (aus, niter/2, -1, 100)
                  IF     (nmore.eq.-1) THEN
                     GOTO 10
                  ELSEIF (nmore.le.0) THEN
                      GOTO 108 ! exit without solution
                  ELSE
                     niter = niter + nmore
                     ENDIF
                  ENDIF
   
               GOTO 101 ! loop : next iteration
C  End loop iterations.
   
 108        CONTINUE ! found no stationary solution
               ! tant pis
 109        CONTINUE ! save results for present K
   
   
            DO m = 1, mMsav
cdeb           IF (qAsk('Exit ?')) RETURN  ! <<<
           Print *, ' going to save in J=', JJ(m), ' K=', K, 'n=', n
               CALL OlfOpen (JJ(m), 1, Kdummy, Fehler)
                 IF (Fehler.ne.'&ff') RETURN
               DO i = 1, n ! statt YY(1,m)
                  Y4(i) = YY(i,m)
                  ENDDO 
               CALL OlfCopZ (jin, JJ(m), K, K, Fehler)
                 IF (Fehler.ne.'&ff') RETURN
               CALL OlfPutXYD (JJ(m), K, n, X0, Y4, D,  Fehler)
                 IF (Fehler.ne.'&ff') RETURN
               CALL OlfClos (JJ(m), nK, Fehler)
                 IF (Fehler.ne.'&ff') RETURN
               ENDDO
            Print *, ' end saving g_()'
   
            IF (niter.ge.2*nloops .and. Kout.lt.nKout) THEN
               nloops = iAskD ('  Increase number of iterations', nloops)
               IF (nloops.le.0) GOTO 11
               ENDIF
   
            ENDIF ! qLisK(K)
            ENDDO ! K
C  End loop spectra.
   
         qWorked = .true.
         jg = JJ(1)
         CALL qCopy (qLisKold, 1, nK, 1, qLisK, 1, 1)
         nKold = nKout
 11      CONTINUE
   
         IF (ig.gt.1)     Print *, ' NOTE/ grouping factor still > 1'
         IF (nKout.lt.nK) Print *, ' NOTE/ some spectra are still uncorrected'
         IF (qAsk (' More iterations with new settings ?')) GOTO 10
C  End loop settings.
   
 90      CONTINUE ! recover from error (Notausstieg)
         IF (qWorked) THEN
            ! close files :
            IF (Fehler.ne.'&ff') CALL FehlerGong (Fehler,1)
            Print *, ' m-phonon contributions ->'
            DO m = 2, mMsav
               CALL OlfClos (JJ(m), nK, Fehler)
               ENDDO
            Print *, ' 1-phonon contribution : new estimate ->'
            CALL OlfClos (JJ(1), nK, Fehler)
         ELSE
            Print *, 'did not work ??'
            ENDIF

         ENDDO ! lj

      write (aus, '(g14.7,l4)') goalMatch, qPlaczek ! exorcism of a compiler bug

C  End loop over files.
      RETURN

 92   CONTINUE ! Notausstieg
      Fehler = 'Ruecksprung leider unerwuenscht'
      RETURN 
c      GOTO 90

      END ! DOS
