C  ====================================================================
C
C      Library  IDA   :  Inelastic Data Analysis
C      Modul    i80   :     read raw data / neutron scattering
C
C  ====================================================================

C     Contents :
C        (1) NRSE-Saclay : [->store]
C               RRawNRSE, DCorrNRSE
C        (2) NSE/ IN11 :
C               RRawIN11
C        (3) Backscattering/ elastic scans :
C               RRawIN10e
C        (4) ILL inelastic data format :
C               Medir
C        (5) Backscattering/ full spectra (old SQW) :
C               RRawBS
C        (6) Time-of-flight (old INZ) :
C               RRawTOF
C        (7) HFBS data files
C               RRT_In_Hfbs

C  ====================================================================
C  i8 (2) NSE / IN11
C  ====================================================================

C  --------------------------------------------------------------------
      SUBROUTINE RRawIN11 (Fehler)
C  --------------------------------------------------------------------
            ! JWu 6may95
         ! Read full output from IN11 (march95 format)

      IMPLICIT REAL *8 (a-h,o-p,r-z)
      IMPLICIT LOGICAL (q)

      INCLUDE 'i_dim.f'
      INCLUDE 'l_def.f'
      PARAMETER    (MLin=1500, MBB=30)
      CHARACTER*40  RFNam, aux
      REAL*8        X(MC), R(7), Y1(MC), D1(MC), FIP(MBB,2), CBB(MBB)
      CHARACTER     Fehler*(*)
      CHARACTER*80  Lin(MLin)
     
      DATA          qFirst / .true. /     

      CALL OlfCreate (j1, K1, '&nodef', '&olddef', Fehler)
      IF (Fehler.ne.'&ff') RETURN
      CALL OlfCnuP (j1, 'x', 't', 'psec', Fehler)
      CALL OlfCnuP (j1, 'y', 'Echo/Aver', 'Cts', Fehler)
      CALL OlfCnuP (j1, 'z1', 'T', 'K', Fehler)
      IF (Fehler.ne.'&ff') RETURN

      wavel = rAskDMu ('Wavelength [A]', wavel, 1.d0, 3.d1)
      E0 = E_of_l(wavel)
      CALL rOlfP (j1, 'E0', 'meV', E0, Fehler)

C  Geometry :
      IF (qFirst) THEN
         DO iBB = 1, MBB
            CALL rAskPair (
     *           'Field integral prefactors (Oe cm) for coil # '//cl2(iBB), 
     *           FIP(iBB,1), FIP(iBB,2), 0.d0, 0.d0)
            CBB(iBB) = 0
            ENDDO
      ELSE
         Print *, 'Using field integral prefactors as before'
         ENDIF

      CBB(29) = 16.7 ! Polarizer
      CBB(30) = 16.7 ! Analyzer

C  Loop over input files :
      DO K = 1, MK

         CALL FrageC ('Raw data file [quit] ?', RFNam)
         IF (RFNam.eq.' ') GOTO 9
         CALL OpenFile (37, RFNam, 'i11', 'a', Fehler)
         IF (Fehler.ne.'&ff') RETURN

         CALL ReadFile (37, Lin, MLin, nLin, '&eof', Fehler)
         IF (Fehler.ne.'&ff') RETURN

         z = rAskMu ('Temperature', 0.d0, 1.d4)

C  Decode log :
         il = 0
         dwn = 0
         upp = 0
 21      CONTINUE
            cdet = 0
            cmon = 0
            npts = 0
 22         CONTINUE 
            iL = iL + 1
            IF (iL.gt.nLin) THEN
               Fehler = 'RR IN11/ did not find C-lines'
               RETURN 
               ENDIF
            IF (Lin(iL)(1:2).eq.'C ') THEN
               CALL FindR (Lin(il), 7, nR, R, .false.)
               IF (nR.lt.3) THEN
                  Fehler = 'RR IN11/ not 3 numbers in B-line '//cl4(il)
                  RETURN
                  ENDIF
               cdet = cdet + R(3)
               cmon = cmon + R(2)
               npts = npts + 1
               GOTO 22
               ENDIF
            IF (Lin(iL)(1:2).ne.'B ') GOTO 22
            IF (npts.ge.1) THEN
               IF     (dwn.eq.0) THEN
                  dwn = cdet / cmon
                  ddwn= dsqrt (cdet/cmon**2 + (cdet**2/cmon**3))
               ELSEIF (upp.eq.0) THEN
                  upp = cdet / cmon
                  dupp= dsqrt (cdet/cmon**2 + (cdet**2/cmon**3))
               ELSE
                  GOTO 29
                  ENDIF
               ENDIF
            GOTO 21
 29      CONTINUE 

         X (1) = 0
         Y1(1) = (upp - dwn) / (upp + dwn) ! Monitor k"urzt sich raus
         D1(1) = 4*(upp**2 * ddwn**2 + dwn**2 * dupp**2) / (upp+dwn)**4
         n = 1

         DO il = iL, nLin
            IF     (Lin(il)(1:2).eq.'B ') THEN
               CALL FindR (Lin(il), 7, nR, R, .false.)
               IF (nR.lt.4) THEN
                  Fehler = 'RR IN11/ not 4 numbers in B-line '//cl4(il)
                  RETURN
                  ENDIF
               iBB = idnint (R(1)) ! # of coil
               IF (qioutside(iBB,1,MBB-2)) THEN
                  Fehler = 'RR IN11/ bad coil # in B-line '//cl4(il)
                  RETURN
                  ENDIF
               CBB(iBB) = R(2)     ! set value
             
            ELSEIF (Lin(il)(1:7).eq.'EC aver') THEN
               CALL FindR (Lin(il), 7, nR, R, .false.)
               IF (nR.lt.4) THEN
                  Fehler = 'RR IN11/ not 4 numbers in EC_av-line '//cl4(il)
                  RETURN
                  ENDIF
               IF (R(1).gt.0) THEN
                  n = n+1 ! new data point
                  Y1(n) = R(3) / R(1)
                  D1(n) = dsqrt ((R(4)/R(1))**2 + (R(3)*R(2)/R(1)/R(1))**2)

c                  ! Log :
c                  Print '(i2,1x,6f8.2,2x,2f6.0)', 
c     *            n, CBB(2), CBB(17), CBB(9), CBB(30), CBB(18), CBB(5),
c     *            R(1), R(3)  
                  
                  IF     (CBB(4).gt.0 .and. CBB(7).eq.0) THEN
                     iKG = 1 ! small spin echo
                  ELSEIF (CBB(4).eq.0 .and. CBB(7).gt.0) THEN
                     iKG = 2 ! large spin echo
                  ELSE
                     Fehler = 'RR IN11/ small or large ?'
                     RETURN 
                     ENDIF
                     
                  ! Calculate spin echo time :
                  X(n) = 0
                  DO iBB = 1, MBB
                     X(n) = X(n) + FIP(iBB,iKG) * CBB(iBB)
                     ENDDO
                  X(n) = 1.863d-4 * X(n) * wavel**3
 
               ELSE 
                  Fehler = 'RR IN11/ no counts in EC_av-line '//cl4(il)
                  RETURN
                  ENDIF

               ENDIF
            ENDDO ! loop over log file lines
               
C  Save Results :
         K1 = K1 + 1
         CALL OlfPutSpe (j1, K1, 1, z, n, X, Y1, D1, Fehler)
         IF (Fehler.ne.'&ff') RETURN

         qFirst = .false.
         ENDDO
C  End loop over input files.

 9    CONTINUE
      CALL OlfClos (j1, K1, Fehler)
      IF (Fehler.ne.'&ff') RETURN

      END ! RRawIN11

C  ====================================================================
C  i8 (3) Backscattering/ elastic scans
C  ====================================================================

C  --------------------------------------------------------------
      SUBROUTINE RRawIN10e (Fehler)
C  --------------------------------------------------------------
         ! read elastic scans from IN10 dump file

      IMPLICIT REAL*8   (a-h,o-p,r-z)
      IMPLICIT LOGICAL  (q)

      INCLUDE 'i_dim.f'
      INCLUDE 'l_def.f'
      PARAMETER    (MLin=MC)
      CHARACTER*40  RFNam, aux
      DIMENSION     X (MC), Y (MC), D (MC), RR(11), ZZ(8)
      CHARACTER     Fehler*(*)
      CHARACTER*80  ein, Lin(MLin)
      DATA          qFirst / .true. /

      IF (qFirst) THEN
         Print *, ' Setup for IN10 elastic scan:'
         wavel = rAsk ('Wavelength (A) ?')
         nK = iAskMu ('Number of detectors used ?', 1, 8)
         DO K = 1, nK
            ZZ(K) = rAsk ('Enter detector angle')
            ENDDO
         qFirst = .false.
      ELSE
         Print *, 'Re-using previous setup (for other setup, restart IDA)'
         ENDIF
      
 1    CONTINUE 

      nRF = iAskDMu ('Number of raw data file (0=qui)', nRF+1, 0, 9999)
      IF (nRF.eq.0) RETURN

      RFNam   = cv4(nRF)
      CALL OpenFile (37, RFNam, 'col', 'a', Fehler)
      IF (Fehler.ne.'&ff') RETURN

      CALL ReadFile (37, Lin, MLin, nLin, '&eof', Fehler)
      IF (Fehler.ne.'&ff') RETURN

C  Output file :
      CALL OlfCreate (jout, Kout, 'r'//cv4(nRF), '&olddef', Fehler)
      IF (Fehler.ne.'&ff') RETURN
      CALL OlfCnuP (jout, 'x', 'T', 'K', Fehler)
      CALL OlfCnuP (jout, 'y', 'Cts', ' ', Fehler)
      CALL OlfCnuP (jout, 'z1', '2th', ' ', Fehler)
      IF (Fehler.ne.'&ff') RETURN

      DO K = 1, nK
         n = 0
         DO i = 5, nLin ! FOUR lines HEADER assumed <<<<<<<< GEAENDERT !!!!
            n = n + 1 
            ein = Lin(i)
            CALL FindR (ein, 11, nR, RR, .false.)
            X(n) = RR(2)
            Y(n) = RR(K+3) / RR(3)
            D(n) = dsqrt0(RR(K+3)) / RR(3)
            ENDDO

         CALL OlfPutSpe (jout, K, 1, ZZ(K), n, X, Y, D, Fehler)
         IF (Fehler.ne.'&ff') RETURN
         ENDDO

      CALL OlfClos (jout, nK, Fehler)
      IF (Fehler.ne.'&ff') RETURN

      GOTO 1

      END ! RRawIN10e

C  ====================================================================
C  i8 (4) Inelastic data access/ 1995 ILL format
C  ====================================================================

C  --------------------------------------------------------------------
      LOGICAL FUNCTION Medir (Instru, iCycle, irun, iBlock, exp, date,
     *                        medpar,pp2,p1,p2,iAdd)
C  --------------------------------------------------------------------
               ! From INZ; copy for IN10sgi may95
         !  Read input as produced from the ILL program SPECTRA
         !  VMS compatibility still maintained, but obsolete 15dec95

      IMPLICIT REAL*8  (a-h,o-p,r-z)
      IMPLICIT LOGICAL (q)
      PARAMETER (MCin=1024, MM=156, ML=384, MN=128)

C  Variables for use in Medir :
      REAL*4    p1(128), p2(128)
      INTEGER   medpar(156), iD(MCin), iM(MCin), iAdd(MCin)
      BYTE      pp2(512)
      CHARACTER exp*10, date*18

      CHARACTER*80  Fehler, LongTit, DirRaw, FileRaw, line
      Character*(*) Instru  
      CHARACTER     cl3*3, cv3*3, cv4*4, cv6*6

      Fehler = '&ff'
      Medir = .false.

C  Open the input files :
      IF (irun.lt.1 .or. irun.gt.99999) THEN
         Print *, ' ILL_In/ Bad number of run : ',irun
         RETURN
         ENDIF

      IF     (iCycle.eq.-1) THEN
         CALL ExeML ('\p dir-raw-n-lst', DirRaw)
      ELSEIF (iCycle.eq. 0) THEN
         CALL ExeML ('\p dir-raw-n-new', DirRaw)
      ELSEIF (iCycle.eq. 1) THEN
         CALL ExeML ('\p dir-raw-n-dac', DirRaw)
      ELSE
         CALL ExeML ('\p dir-raw-n-old', DirRaw)
         ENDIF 
      IF (DirRaw(1:4).eq.'&err') THEN
         Print *, 'This cycle is not accessible with the present setup'
         RETURN
         ENDIF
      FileRaw = DirRaw

      IF     (Instru.eq.'IN10') THEN
         CALL ReplaceT (FileRaw, '&inst', 'in10')
      ELSEIF (Instru.eq.'IN16') THEN
         CALL ReplaceT (FileRaw, '&inst', 'in16')
      ELSEIF (Instru.eq.'IN13') THEN
         CALL ReplaceT (FileRaw, '&inst', 'in13')
      ELSE
         Print *, 'Invalid instrument : ', Instru
         RETURN
         ENDIF

      IF (iCycle.lt.1000) THEN
         CALL ReplaceT (FileRaw, '&cycle', cv3(iCycle))
      ELSE
         CALL ReplaceT (FileRaw, '&cycle', cv4(iCycle))
         ENDIF
   
      CALL Append (FileRaw, cv6(irun))
     
      CALL OpenFile (11, FileRaw, '&noext', 'a', Fehler)
      IF (Fehler.ne.'&ff') THEN
         Print *, 'ILL_In/ Could not open file ', FileRaw
         RETURN
         ENDIF

C  Read header blocks :
      Read(11, '(/i8)') jrun
    
      Read(11, '(/////16(10(i8)/))', end=11) (medpar(i), i=1,156)
 11   CONTINUE 

      nK = medpar(1) 
 
      Read (11,'(a)') line
      IF (line(1:3).ne.'AAA') THEN
         Print *, line
         CALL Absturz ('ILL_In', 'Block 4/ expected AAAAAA')
         ENDIF 
      Read(11, '(/a80,6(/))') LongTit

      Read (11,'(a)') line
      IF (line(1:3).ne.'FFF') THEN 
         Print *, line
         CALL Absturz ('ILL_In', 'Par1/ expected FFFFFF')
         ENDIF 
      Read(11, '(/40(5(2x,d14.8)/))', end=12) (p1(i), i=1,128)
 12   CONTINUE

      Read (11,'(a)') line
      IF (line(1:3).ne.'FFF') THEN 
         Print *, line
         CALL Absturz ('ILL_In', 'Par2/ expected FFFFFF')
         ENDIF 
      Read(11, '(/40(5(2x,d14.8)/))', end=13) (p2(i), i=1,128)
 13   CONTINUE 

      IF (Instru.eq.'IN6')
     *     Read (11,'(53(/),x)') ! a new, nonsensical integer field on IN6

C  Read data blocks :
      DO K=1, iBlock
         Read (11,'(a)') line
         IF (line(1:3).ne.'SSS') THEN
            Print *, 'spectrum '//cl3(K)
            Print *, line
            CALL Absturz ('ILL_In', 'expected SSSSSSS')
            ENDIF 
         Read (11,'(//i8)') iBlo
         IF (iBlo.gt.MCin) CALL Absturz (
     *       'ILL_In', '#channels > MCin in spectrum '//cl3(K)) 
         Read (11, '(10i8)')  (iAdd(i), i=1,iBlo)
         ENDDO

      Close(11)

C  Checks :
      IF    (jrun.ne.irun) THEN
         Print *, '#run : given/read ', irun, jrun
         CALL Absturz ('ILL_In',' Bad number of run')
c      ELSEIF (nC.ne.511) THEN
c         CALL Absturz ('ILL_In', '#channels<>511)')
         ENDIF

      Medir = .true. ! successfully executed

      END ! Medir

C  ====================================================================
C  i8 (5) Backscattering (old SQW)
C  ====================================================================

C   General information :
C        Completely rewritten March-May 1991 by M.Wendel, J.Wuttke
C
C   Table of major modifications :
C
C           27. 2.96  IN16 version (from O.Randl's version of SQW)
C            3. 5.95  Implementaion for SG-Workstation; read in ASCII-File
C           26. 9.91  Debye integral
C           23. 9.91  Log file, outer loop, defaults
C           14. 8.91  Dialogue revised, some repairs
C           21. 5.91  Absorption correction added

C   Structure (location of modules given if <> sqw0) :
C        Main
C           GetData        ! sum numors, set common x-scale
C              ReadILL(2)  ! read 1 numor, determine energies
C                 Medir(SpeLib)
C           GetSpectrum    ! take spectra from array, group them

C   Implementation :
C        Link sqw0-2, i1, l0-l6, (ComData:SpeLib/Li), NAGLIB2/L

C  --------------------------------------------------------------------
      SUBROUTINE RRawBS (Instru, Fehler)
C  --------------------------------------------------------------------

      IMPLICIT REAL*8  (a-h,o-p,r-z)
      IMPLICIT LOGICAL (q)

      INCLUDE 'i_dim.f'
      INCLUDE 'l_def.f'

      PARAMETER (MCin=1024, MKin=35)
      REAL*8     SDY(MCin), SDX(MCin), SDYE(MCin), SDAngleA(MKin),
     *           EshiftA(MKin), SDYA(MCin,MKin), SDYEA(MCin,MKin),
     *           Ef, SDAngle

      CHARACTER*(*) Fehler, Instru
      CHARACTER*40  Title, TitOut

      DATA          TitOut / ' ' /

C  --------------------------------------------------------------------
C        Preset
C  --------------------------------------------------------------------

      IF     (Instru.eq.'IN10' .or. Instru.eq.'IN16') THEN 
         iErgPow = -6 ! ueV
         iz      =  3 ! q0
      ELSEIF (Instru.eq.'IN13') THEN
         iErgPow = -3 ! meV
         iz      =  3 ! q0
      ELSE
         Fehler = 'RawBS/ no BS instrument'
         RETURN
         ENDIF

C  --------------------------------------------------------------------
C        Read raw data
C  --------------------------------------------------------------------

 1    CONTINUE ! new run
      CALL GetData(SDX1, SDXstep, SDYA, SDYEA, SDAngleA, EshiftA, SDTemp,
     *             nSD, nKSD, Ef, Title, Instru)
      IF (nKSD.le.0) RETURN
      nK = nKSD
      IF (nSD.le.0) THEN
         Fehler = 'RRawBS/ read empty spectra ???'
         RETURN
         ENDIF

      IF (SDTemp.le.0.) THEN
            ! for IN10, the sample temperature is not recorded,
            ! GetData returns Temp=0.
         SDTemp = rAskMu ('Sample temperature [K] ?',0.d0,8000.d0)
         ENDIF

      CALL OlfCreate (jout, Kout, '&nodef', '&olddef', Fehler)
      IF (Fehler.ne.'&ff') RETURN

      IF     (iErgPow.eq.-3) THEN
         CALL OlfCnuP (jout, 'x', 'w', 'meV', Fehler)
         CALL OlfCnuP (jout, 'y', 'S(q,w)', 'meV-1', Fehler)
      ELSEIF (iErgPow.eq.-6) THEN
         CALL OlfCnuP (jout, 'x', 'w', 'ueV', Fehler)
         CALL OlfCnuP (jout, 'y', 'S(q,w)', 'ueV-1', Fehler)
      ELSE
         Fehler = 'iErgPow o.o.r.'
         ENDIF
      IF (Fehler.ne.'&ff') RETURN

      IF     (iz.eq.1) THEN
         CALL OlfCnuP (jout, 'z1', '2th', ' ', Fehler)
      ELSEIF (iz.eq.3) THEN
         CALL OlfCnuP (jout, 'z1', 'q', 'A-1', Fehler) ! in guter Naeherung
      ELSE
         Fehler = 'iz o.o.r.'
         ENDIF
      IF (Fehler.ne.'&ff') RETURN

      CALL iOlfP (jout, '?det-bal-sym',   0, Fehler)
      CALL iOlfP (jout, '@sam-erg-gain', -1, Fehler)
      
      CALL rOlfP (jout, 'E0', 'meV', Ef,     Fehler)
      CALL rOlfP (jout, 'T',  'K',   SDTemp, Fehler)

      CALL OlfComAdd (jout, ' ',
     *     'raw data '//Instru(1:lenU(Instru))//': '//Title, Fehler)

C  --------------------------------------------------------------------
C        Loop over grouped spectra
C  --------------------------------------------------------------------

      DO K = 1, nK

C  - Get sample data :
         DO i = 1,nSD
            SDY(i)  = SDYA(i,K)
            SDYE(i) = SDYEA(i,K)
            ENDDO
         SDAngle = SDAngleA(K)

C  - Write out :
         DO i = 1,nSD
            SDX(i) = SDX1+(i-1)*SDXstep
            ENDDO
         IF (iErgPow.eq.-6) THEN ! meV -> ueV
            DO i = 1, nSD
               SDX (i) = SDX (i) * 1000
               SDY (i) = SDY (i) * 1000
               SDYE(i) = SDYE(i) * 1000
               ENDDO
            ENDIF

         IF     (iz.eq.1) THEN 
            z = SDAngle
         ELSEIF (iz.eq.3) THEN
            z = 2 * dsqrt (Ef / 2.0723) * dsind (SDAngle/2) ! Q = kf-ki
            ENDIF
             
         CALL OlfPutSpe (jout, K, 1, z, nSD, SDX, SDY, SDYE, Fehler)
         IF (Fehler.ne.'&ff') RETURN

         ENDDO ! K

      CALL OlfClos (jout, nK, Fehler)
      Print *

      GOTO 1 

      END ! SQW_main

C  --------------------------------------------------------------------
      SUBROUTINE GetData (X1,Xstep,YA,YEA,AngleA,EshiftA,Temp,
     *                    n,nK,Ef,Title,Instru)
C  --------------------------------------------------------------------
         ! reads all data of all angles in a 2-dim array
         ! sets energy scale

      IMPLICIT REAL*8  (a-h,o-p,r-z)
      IMPLICIT LOGICAL (q)
      PARAMETER (MCin=1024, MKin=35)
      REAL*8        YM(MCin),YMa(MCin),Xo(MCin),Xu(MCin),Xao(MCin),Xau(MCin),
     *              YA(MCin,MKin),YEA(MCin,MKin),YAa(MCin,MKin),
     *              AngleA(MKin),AngleAa(MKin),EshiftA(MKin),EshiftAa(MKin),
     *              Coder(30),Codera(30), Temp,TempA, Ef,EfA
      CHARACTER*(*) Instru,Title
      CHARACTER exp*10,date*18
c     EQUIVALENCE (YAE,YAa) !! to save storage capacity if necessary

C     external character functions :
      CHARACTER     ch1*1, cr2*2, cl2*2, cr3*3, cr4*4, cl6*6

C  Read first numor :
      CALL ReadILL (X1,Xstep,YA,YM,AngleA,Coder,EshiftA,Temp,
     *                   n,nK,Ef,Title,Instru)
      IF (nK.eq.0) RETURN ! no sample data are given
      IF (n.le.0) THEN
         CALL Gong (9)
         Print *, 'GetData/ got n<=0 from ReadILL'
         RETURN
         ENDIF 

C  Add further numors as long as wanted :
      DO WHILE (1.ne.0)

C  - Read in :
200      CONTINUE
         Print *, 'Add further numors ?'
         CALL ReadILL (X1a,Xstepa,YAa,YMa,AngleAa,Codera,EshiftAa,
     *                  Tempa,na,nKa,Efa,Titlea,Instru)
         IF (nKa.eq.0) GOTO 300

C  - Check for consistency :
         IF (nka.ne.nk) THEN
            CALL Gong (3)
            Print *, 'WARNING/ This numor ignored.'
            Print *, 'Number of detectors inconsistent.'
            GOTO 200
            ENDIF
         CALL Append (Title, Titlea)
C  - - Check consistency of all angles and energy shifts :
         qAngleIncons = .false.
         qEshiftIncons = .false.
         DO k = 1,nK
            IF ((dabs(AngleA(k)-AngleAa(k))).gt.(0.1)) THEN
               qAngleIncons = .true.
               ENDIF
            IF ((dabs(EshiftA(k)-EshiftAa(k))).gt.(0.1*Xstep)) THEN
               qEshiftIncons = .true.
               ENDIF
            ENDDO
         IF (qAngleIncons) THEN
            Print *, 'WARNING/ Angle inconsistent.'
            ENDIF
         IF (qEshiftIncons) THEN
            Print *, 'WARNING/ Energy shift (analyser offset) inconsistent.'
            ENDIF
C  - - Check consistence of temperature :
         IF ((dabs(Temp-Tempa)).gt.(1+0.01*Temp)) THEN
            Print *, 'WARNING/ Temperature inconsistent.'
            ENDIF
C  - - Check consistence of neutron energy :
         IF ((dabs(Ef-Efa)).gt.(0.01*Ef)) THEN
            Print *, 'WARNING/ Neutron energy inconsistent.'
            ENDIF
C  - - Check consistence of coder :
         DO l = 1,30
            IF ((dabs(Coder(l)-Codera(l))).gt.(0.01*Coder(l))) THEN
               Print *, 'WARNING/ Coder '//cr2(l)//' inconsistent.'
               ENDIF
            ENDDO

C  - Add the new numor (the non-trivial part of this subroutine) :

         ! calculate number of channels added at the beginning :
         IF (X1a-Xstepa/2.lt.x1-Xstep/2) THEN
            dXu = (X1-Xstep/2)-(X1a-Xstepa/2)
            nu = idint(dXu/Xstep)
         ELSE
            nu = 0
            ENDIF

         ! calculate number of channels added at the end :
         IF (X1a+Xstepa*(na+0.5).gt.X1+Xstep*(n+0.5)) THEN
            dXo = (X1a+Xstepa*(na+0.5))-(X1+Xstep*(n+0.5))
            no = idint(dXo/Xstep)
         ELSE
            no = 0
            ENDIF

        ! warns if channel number of extended energy scale to large

         IF (n+nu+no.gt.MCin) THEN
            Print *, 'WARNING/ number of channels exceeds '//cl6(MCin)
            Print '(2(a,g10.4))', ' old : X1 = ', X1,  ', dX = ', Xstep
            Print '(2(a,g10.4))', ' new : X1 = ', X1a, ', dX = ', Xstepa
            Print '(3(a,i4))', ' #old =', n,
     *         ', #new(<1) = ', nu, ', #new(>N) = ', no
            Print *, 'last numor not added'
            GOTO 210
            ENDIF

        ! shifts the data nu channels

         IF (nu.gt.0) THEN
            DO i = n,1,-1
               YM(i+nu) = YM(i)
               DO k = 1,nK
                  YA(i+nu,k) = YA(i,k)
                  YEA(i+nu,k) = YEA(i,k)
                  ENDDO
               ENDDO
            DO i = 1,nu
               YM(i) = 0.
               DO k = 1,nK
                  YA(i,k) = 0.
                  YEA(i,k) = 0.
                  ENDDO
               ENDDO
            ENDIF ! shift of data

         ! calculates new channel number and new X1

         n = n+nu+no
         X1 = X1-nu*Xstep

         ! add other channels

         DO i = 1,n                ! calculates boundaries of each
            Xo(i) = X1+(i-0.5)*Xstep ! channel of old energy scale
            Xu(i) = Xo(i)-Xstep
            ENDDO
         DO j = 1,na+1                ! calculates boundaries of each
            Xao(j) = X1a+(j-0.5)*Xstepa ! channel of new energy scale
            Xau(j) = Xao(j)-Xstepa
            ENDDO

         j = 1                           ! j: channel counter of new scale
         DO i = 1,n                      ! i:   "         "    " old   "
            DO WHILE (Xao(j).lt.Xu(i))   ! while no overlap of channels i
               j = j+1                   ! of old scale and j of new scale
               IF (j.gt.n) GOTO 210
               ENDDO
            DO WHILE (Xau(j).lt.Xo(i))   !while overlap of channels i and j
               part = (dmin1(Xao(j),Xo(i))-dmax1(Xau(j),Xu(i)))/Xstepa
                                         ! part: relative extend of overlap
cdeb         print '(a,2i4,5g10.2)', 'i j p YM.. YA(8)..', i, j, part,
cdeb     *      YM(i), YMa(j), YA(i,8), YAa(j,8)
               YM(i) = YM(i)+part*YMa(j) ! add monitor data
               DO k = 1,nK               ! k: angle counter
                  YA(i,k) = YA(i,k)+part*YAa(j,k) ! add count rates
                  ENDDO
               j = j+1
               IF (j.gt.na) GOTO 220   ! if all new channels added
               ENDDO
            IF (j.gt.1) j = j-1
220         CONTINUE
            ENDDO

C  - The new numor is added, end of loop :
210      CONTINUE
         ENDDO
300   CONTINUE ! exit from loop : no further numors are given


C  Normalize detector to monitor counts :

      DO k = 1,nK
         DO i = n,1,-1
            ymr = YM(i)
            ydr = YA(i,k)
            YA(i,k) = dquot0(ydr,ymr)
            YEA(i,k) = YA(i,k)*dsqrt(dquot0(1.d0,ydr)+dquot0(1.d0,ymr))
            ENDDO ! i
         ENDDO   ! K

      END ! GetData

C  --------------------------------------------------------------------
      SUBROUTINE ReadILL (X1,Xstep,YA,YM,AngleA,Coder,EshiftA,Temp,
     *                    n,nK,Ef,Title,Instru)
C  --------------------------------------------------------------------

      ! reads all data of all angles in a 2-dim array
      ! calculates error of all values
      ! calculates X1 and Xstep

      IMPLICIT REAL*8  (a-h,o-p,r-z)
      IMPLICIT LOGICAL (q)
      PARAMETER (MCin=1024, MKin=35)

C  Global variables :
      DIMENSION YA(MCin,MKin), YM(MCin)
      DIMENSION AngleA(MKin), Coder(30), EshiftA(MKin)
      CHARACTER Instru*4, Title*40

C  Local variables :
      REAL*8    X(MCin)
      CHARACTER udb*1, udb0*1

C  Variables for use in Medir :
      REAL*4    p1(128), p2(128)
      INTEGER   medpar(156), iD(MCin), iM(MCin), iAdd(MCin), iE(MCin)
      BYTE      pp2(512)
      CHARACTER exp*10, date*18, TitleIn*40

      EQUIVALENCE (pp2(1),TitleIn)

C  External character functions :
      CHARACTER cr3*3, cr6*6, cl8*8

C  External function :
      LOGICAL   Medir

C  Defaults for dialogue :
      DATA      udb /'b'/, udb0 /' '/

C  Ask for numor identification :

10    CONTINUE
      numor = iAskDu('Numor (or RETURN) ? ',0)
      IF (numor.le.0) THEN
         nK = 0
         RETURN
         ENDIF
      iCycle = iAskDMu ('Cycle (1=bus, 0=present, -1=last)',iCycle,0,9999)
      Title = cl8(Numor)

C  read header blocks :
      iBlock = 1
      IF (.not.Medir(Instru,iCycle,numor,iBlock,exp,date,medpar,pp2,
     *               p1,p2,iD)) THEN
         CALL Gong (2)
         GOTO 10
         ENDIF

C  read number of blocks and number of channels :
      ! default : no double blocks (corr. JWu 5may93)
      Koffset = 0
      qboth   = .false.
      udb0 = ' '
      IF (Instru.eq.'IN10') THEN      ! IN10
         qboth = .false.
         nK = int(p1(20))
         nKMon = nK+1
         IF (p1(22).eq.0) THEN         ! if IN10 Doppler
            qMono = .false.
            n = medpar(2)-1
         ELSEIF (p1(22).eq.12 .or. p1(22).eq.13) THEN   ! if IN10 Monochromator
                    ! Que'est-ce que c'est que p1(22) ? Ajoute le cas =13 ?
            qMono = .true.
            n = medpar(2)
         ELSE
            Print *, ' ReadILL/ IN10 A or B ?? p1(22) = ', p1(22)
            nK = 0
            RETURN
            ENDIF
         ENDIF                          ! end IN10

      IF (Instru.eq.'IN16') THEN      ! IN16
         qboth = .false.
         nK = int(p1(8))
         nKMon = nK+1
         qMono = .false.
         n = int(p1(7)-1.)
         ENDIF                          ! end IN16

      IF (Instru.eq.'IN13') THEN      ! IN13
         qMono = .true.
         n = int(p1(7))
         nK = int(p1(8))-2
         nKMon = int(p1(8)) - 1 ! (apr00) monitor now in block 36
         nBlock = medpar(1)
         Print *, 'ReadILL/ IN13/ ', n, nK, nBlock
         IF (nBlock.eq.2*nK+4) THEN
            qDouble = .true.              ! if up- and down-scan
         ELSE IF (nBlock.eq.nK+2) THEN
            qDouble = .false.
         ELSE
            Print *, 'FATAL ERROR :'
            Print *, 'Problems with data format.'
            Print *, 'Number of blocks fits neither to up - scan'//
     *              'nor to up and down - scan.'
            nK = 0
            RETURN
            ENDIF
         IF (qDouble) THEN   ! sets variables for up and down scan
132         CONTINUE
            CALL FrageCD
     *         ('Take up (u), down (d) or both (b) scans', udb, udb)
            CALL Minuskeln(udb)
            IF (udb.eq.'u') THEN
            ELSE IF (udb.eq.'d') THEN
               Koffset = nK+2
            ELSE IF (udb.eq.'b') THEN
               qboth = .true.
            ELSE
               CALL Gong(3)
               GOTO 132
               ENDIF
            udb0 = udb ! for doc
            ENDIF ! qDouble
         ENDIF ! IN13

C  read sample temperature, ... 

      IF (Instru.eq.'IN10') THEN
         Temp = 0.
      ELSE IF (Instru.eq.'IN13') THEN
         Temp = dble(p1(9))
      ELSE IF (Instru.eq.'IN16') THEN
         Temp = dble(p1(10))
         ENDIF

C  read coders, angles

      IF ((Instru.eq.'IN10').or. (Instru.eq.'IN13')) THEN

         DO i = 1,30                   !  IN10 and IN13
            Coder(i) = dble(p1(50+i))
            ENDDO
         DO k = 1,nK                   !  IN10 and IN13
            AngleA(k) = dble(p2(k))
            ENDDO

      ELSE IF (Instru.eq.'IN16') THEN

         DO i = 1,30
            Coder(i) = dble(p1(30+i))
            ENDDO

         IF (nK.gt.20) THEN
        
            DO k = 1, 20
               AngleA(k) = dble(p2(k))+(p1(65)-1.)*5.0d0 ! multidetector
               ENDDO

            DO k = 21,nK
               AngleA(k) = dble(p2(k)) !  single detectors
               ENDDO

          ELSEIF (nK.eq.20) then

             DO k = 1,20 
                AngleA(k) = dble(p2(k))+(p1(65)-1.)*5.0d0 ! multidetector
                ENDDO
 
          ELSE
             DO k = 1,nK  
                AngleA(k) = dble(p2(k+20)) ! single detectors
                ENDDO
          
             ENDIF ! K > or = or < 20
  
         ENDIF ! Instru

C  determine lattice constant

      IF (Instru.eq.'IN10') THEN
         dlattice = dble(p1(84))
         ENDIF
      IF (Instru.eq.'IN16') THEN
         dlattice = dble(p1(80))
         ENDIF
      IF (Instru.eq.'IN13') THEN
         deltaTemp = dble(p1(91))-28.  ! p1(91):analyser temperature
          ! temperature difference to reference temperature 28 C
         dlattice = dble(p1(82)) *
     *             ( 1. + dble(p1(89))*deltaTemp +
     *                    dble(p1(90))*(deltaTemp**2)/2 )
          ! p1(82) : lattice constant of analyzer-crystal at 28 C
          ! p1(89),p1(90) material constants for thermal expansion
         ENDIF

C  calculate energy Ef, energy shift EshiftA(k)

      Ef = 81.8055/((2.*dlattice)**2.)
        ! dlattice : lattice constant of analyzer crystal
        ! lattice-const-->wavelength-->energy of detected neutrons
      DO k = 1,nK
         EshiftA(k) = Ef*(dtand(dble(p2(50+k))/2.)**2.)
          ! p2(50+k) : analyser offsets (angle)
         ENDDO

C  calculate X1, Xstep

      IF (Instru.eq.'IN10') THEN     ! if IN10
         IF (.not.qMono) THEN         ! if IN10 Doppler
            EM = 81.8055 / (2*dble(p1(82)))**2
             ! p1(82) : lattice constant of monochromator crystal
             ! lattice-const = .5 wavelength-->energy of incident neutr.
            ED = 19.656769d8 * dble(p1(2)) * 4.135701327d-12 /
     *                                          (2*dble(p1(82)))
             ! 19.65..d8 : length of Doppler-drive [A] (d8:cm-->A)
             ! p1(2) : Doppler-frequency [s^-1]
             ! 4.1357..(-12) : h [meV*s]
             ! calculates maximal energy difference by doppler effect
             !INFO: Xstep and X1 calculated after analysing monitor rates
         ELSE IF (qMono) THEN   ! if IN10 Monochromator
            iBlock = nK+2
            IF (.not.Medir(Instru,iCycle,numor,iBlock,exp,date,medpar,
     *         pp2,p1,p2,iD)) THEN
               Print *, 'FATAL ERROR :'
               Print *, 'Medir cannot read the energy block.'
               nK = 0
               RETURN
               ENDIF
            fact = dble(p1(23))      ! converts energy scale to microeV
             ! Exclude channels with X=0. :

c  das ist noch gar nicht schoen;
c  hier muessen schon nL und nR eingefuehrt werden.
            j = 1
            DO WHILE (iD(j).eq.0)   ! first energy value not equal 0
               j = j+1
               ENDDO
            m = n
            DO WHILE (iD(m).eq.0)   ! last energy value not equal 0
               m = m-1
               ENDDO
c  auch schlecht :
            Xone = dble(iD(j))   / (fact*1000)   ! first written energy value
            Xtwo = dble(iD(j+1)) / (fact*1000)   ! second   "       "     "
            Xstep = Xtwo - Xone
            X1 = Xone - (j-1) * Xstep
            DO i = j+1,m  ! control if energy steps are equidistant
               X(i)   = dble (iD(i)  ) / (fact*1000)
               X(i-1) = dble (iD(i-1)) / (fact*1000)
                  ! Energies are stored in nano(!)eV as integer values.
                  ! 1000 converts neV to ueV
                  ! fact is usually 1000 and converts ueV to meV
               IF (dabs(X(i)-X(i-1)-Xstep).gt.(0.001*Xstep)) THEN
c                  Print *, 'FATAL ERROR :'
                  CALL Gong (5)
                  Print *, 'Energy scale is not equidistant: i = ',i
c                  STOP
                  Print *, 'Notreparatur :'
                  X1    = rAsk ('E(#1) [ueV]') / 1000
                  Xstep = rAsk ('dE    [ueV]') / 1000
                  GOTO 209 ! exit loop
                  ENDIF
               ENDDO
 209        CONTINUE
         ELSE     ! if no inelastic scan
            Print *, 'ERROR :'
            Print *, 'this numor is no inelastic scan : p1(22) = ',p1(22)
            GOTO 10
            ENDIF                      ! IN10 Monochromator
         ENDIF                        ! IN10

      IF (Instru.eq.'IN16') THEN 

            EM = 81.8055 / (2*dble(p1(70)))**2
             ! p1(70) : lattice constant of monochromator crystal
             ! lattice-const = .5 wavelength-->energy of incident neutr.

            ED = 15.939917d8 * dble(p1(3)) * 4.135701327d-12 /
     *                                          (2*dble(p1(70)))
             ! 15.93..d8 max.speed (A/s) for Doppler-freq. = 1Hz
             ! p1(3) : Doppler-frequency [s^-1]
             ! 4.1357..(-12) : h [meV*s]
             ! p1(70) lattice parameter monochromator (A)
             ! calculates maximal energy difference by doppler effect
             !INFO: Xstep and X1 calculated after analysing monitor rates
         ENDIF ! IN16

      IF (Instru.eq.'IN13') THEN 
c         X0 = dble(p1(2)-p1(3))/1000
c          ! p1(2) = centre of energy range (microeV)
c          ! p1(3) = half width of energy range
c         Xstep = dble(p1(12))/1000
c         X1 = X0+Xstep/2
         ! apr00: energies are in block 38
         iBlock = nKMon + 2
         IF (.not.Medir(Instru,iCycle,numor,iBlock,exp,date,medpar,pp2,
     *                  p1,p2,iE)) THEN
            Print *, 'problem with Medir reading energies '
            GOTO 10
            ENDIF
         X1    = iE(1) / 1d5
         Xstep = (iE(2)-iE(1)) / 1d5
         Xn    = iE(n) / 1d5
         IF (Xn.ne.X1 + (n-1)*Xstep) THEN 
            Print *, 'energy scale not equidistant ????'
            RETURN
            ENDIF
         ENDIF

C  read monitor counts

      iBlock = nKMon+Koffset
      IF (.not.Medir(Instru,iCycle,numor,iBlock,exp,date,medpar,pp2,
     *               p1,p2,iM)) THEN
         Print *, 'problems with Medir reading monitor-data '
         GOTO 10
         ENDIF

C  add up and down monitor rates (IN13) if wanted

      IF (qboth) THEN
         iBlock = nKMon+(nK+2)
         IF (.not.Medir(Instru,iCycle,numor,iBlock,exp,date,medpar,pp2,
     *                 p1,p2,iAdd)) THEN
            Print *, 'problems with Medir reading monitor-data of'
            Print *, 'second scan of up and down scan'
            GOTO 10
            ENDIF
         DO i = 1,n
            iM(i) = iM(i)+iAdd(i)
            ENDDO
         ENDIF

C  determine significant channels nL,...,nR (revised JWu 14aug91) :
      nL  = 1
      nR  = n
      rel = .1 ! arbitrary cutoff condition

 320  CONTINUE
      ! mean monitor counts :
      YmMean = 0
      DO i = nL,nR
         YmMean = YmMean+dble(iM(i))
         ENDDO
      nFull  = nR - nL + 1
      YmMean = YmMean / nFull

      ! throw away empty or poor channels on the left :
      DO WHILE (iM(nL).le.(rel*YmMean))
         nL = nL+1
         ENDDO
      ! the same on the right :
      DO WHILE (iM(nR).le.(rel*YmMean))
         nR = nR-1
         ENDDO

      Print *, 'ReadILL/ nR = ', n, ' -> ', nR, '; nL = ', 1, ' -> ', nL
      n = nR - nL + 1                   ! number of significant channels
      IF (n.lt.nFull) GOTO 320          ! repeat the operation with new YmMean

C  adjust energy scale :
      IF (((Instru.eq.'IN10').and.(.not.qMono)) .or.
     *     (Instru.eq.'IN16')) THEN                   ! doppler scan
         Xstep = 2*ED/n
          ! energy difference between each channel
         X1 = -Xstep*(n-1)/2.+Ef-EM ! corrected 29aug94 (error found by ORa)
          ! minimal energy (compared to incoming neutron energy)
      ELSE
         X1 = X1+(nL-1)*Xstep                         ! monochromator scan
         ENDIF

C  save monitor data as YM(i) :
      DO i = 1,n
         YM(i) = dble(iM(i+(nL-1)))
         ENDDO

C  read all spectra and save as YA(i,k)

      DO k = 1,nK
         iBlock = k+Koffset            ! Koffset for IN13 up and down scan
         IF (.not.Medir(Instru,iCycle,numor,iBlock,exp,date,medpar,pp2,
     *                 p1,p2,iD)) THEN
            Print *, 'problems with Medir reading spectrum',k
            GOTO 10
            ENDIF
         IF (qboth) THEN              ! if adding up and down scan IN13
            iBlock = k+nK+2
            IF (.not.Medir(Instru,iCycle,numor,iBlock,exp,date,medpar,
     *                   pp2,p1,p2,iAdd)) THEN
               Print *, 'problems with Medir reading second spectrum',k
               GOTO 10
               ENDIF
            DO i = 1,n
               iD(i) = iD(i)+iAdd(i)
               ENDDO
            ENDIF
         DO i = n,1,-1                 ! +(nL-1) : begins with first
            YA(i,k) = dble(iD(i+(nL-1))) ! significant channel
            ENDDO ! i
         ENDDO   ! K

      END ! ReadILL

C  ====================================================================
C  i8 (6) Time-of-flight (old INX/INZ)
C  ====================================================================

C   Structure :
C      RRawTOF
C        RRT_InSub
C          RRT_InSum
C            RRT_Input
C              RRT_In_ILL
C              RRT_In_Mib
C              RRT_In_Sil
C              RRT_In_Fcs
C              RRT_EPeak_Fcs
C        RRT_Check
C        RRT_EPeak
C        RRT_Energ
C        RRT_DetEff
C        RRT_Norm
C        RRT_Sort
C        RRT_Add

C  History :
C     A.Meyer feb98 RRT_In_Fcs
C     JWu 5dec95 2dim arrays eliminated
C     JWu 15aug95 as subroutine RRawTOF
C     A.Meyer + JWu feb95 : RRT_In_Sil, spec sum+del under output 
C     JWu jan93 : installation Saclay, entruempelt, REAL*8
C     JWu jun92 : frame overlap, output, input, entruempelt
C     JWu sep91 : DetEff for IN6
C     JWu jul91 : RRT_In_Mib version 2, DetEff for Mibemol, Text in EppCon
C     J.Wuttke aug91 : version INZ without menus
C     M.Bee '91 : Mibemol version 1
C     Program INX by Fr. Rieutord 1990

C  --------------------------------------------------------------------
      SUBROUTINE RRawTOF (Inst, Fehler)
C  --------------------------------------------------------------------

      IMPLICIT REAL*8  (a-h,o-p,r-z)
      IMPLICIT LOGICAL (q)
      INCLUDE      'l_def.f'
      INCLUDE      'i_dim.f'
      INCLUDE      'i_wrk.f'
      PARAMETER   (MNuSu=100)

      REAL*8       AngleS(MK), AngleV(MK), FPath(MK), DetE1(MK), DetE2(MK),
     *             XepS(MK), Xepfcs(MK), YepS(MK), XepV(MK), YepV(MK),
     *             DWF, DWFmin
      CHARACTER*1  BadS(MK), BadV(MK)
      LOGICAL      qLisK(MK), qKnoAnal(MK)

      CHARACTER*(*) Inst, Fehler
      CHARACTER*40  FilOut, Title
      CHARACTER*80  ListSD, ListSC, ListVD, ListVC, LKnoAnal, LisKdel

      DIMENSION     JNumorSD(MNuSu), JNumorSC(MNuSu), 
     *              JNumorVD(MNuSu), JNumorVC(MNuSu)

C  --------------------------------------------------------------------
C  Default values :
C  --------------------------------------------------------------------

      DATA     nLoop /0/, ListSC, ListVC /2*'-'/, qVan /.true./
     *         qOutE /.true./, q0fromS /.false./, iFmod /1/,
     *         EFmax /1.d3/, EFminRel /-.99d0/, qMon /.true./,
     *         qDetEff /.true./, iDel /1/, qAngEqAdd / .true. /

C  --------------------------------------------------------------------
C  Checks and Initializations :
C  --------------------------------------------------------------------

      IF (.not. (Inst.eq.'IN4' .or. Inst.eq.'IN5' .or. Inst.eq.'IN6' .or.
     *           Inst.eq.'MIB' .or. Inst.eq.'SIL' .or. Inst.eq.'FCS' .or.
     *           Inst.eq.'NEAT')) THEN
         Fehler = 
     *     'Instruments are IN4,IN5,IN6,MIB,SIL,FCS,NEAT - do not know '//Inst
         GOTO 99
         ENDIF

C  Open Log-File :
      CALL OpenFile (35, 'inz', 'log', 'e', Fehler)
      IF (Fehler.ne.'&ff') GOTO 99

C  --------------------------------------------------------------------
C  Begin dialogue :
C  --------------------------------------------------------------------

c      IF (Inst(1:2).eq.'IN') 
      iCycle = iAskD (
     *     'Data from instrument(-1), current cycle(0), archive(>0)', iCycle)
      iNOff = iAskD ('Run number offset', iNOff)

 110  CONTINUE ! outer loop
      nLoop = nLoop + 1

      ListSD = ' '
      CALL GetJList ('Sample run numbers', ListSD, 
     *               MNuSu, nNumorSD, JNumorSD, 0, 999999)
      IF (nNumorSD.le.0) GOTO 99
      DO i = 1, nNumorSD
         JNumorSD(i) = JNumorSD(i) + iNOff
         ENDDO

      IF (nLoop.gt.1) THEN
         qTheSame = qAskD('All the rest as before',intq(qTheSame))
         IF (qTheSame) GOTO 200
      ELSE
         qTheSame = .false.
         ENDIF

      CALL GetJList ('Background run numbers', ListSC,
     *               MNuSu, nNumorSC, JNumorSC, 0, 999999)
      DO i = 1, nNumorSC
         JNumorSC(i) = JNumorSC(i) + iNOff
         ENDDO

      CALL GetJList ('Vanadium run numbers', ListVD,
     *               MNuSu, nNumorVD, JNumorVD, 0, 999999)
      DO i = 1, nNumorVD
         JNumorVD(i) = JNumorVD(i) + iNOff
         ENDDO

      IF (nNumorVD.ne.0) THEN
         CALL GetJList ('Vanadium background run numbers', ListVC,
     *                   MNuSu, nNumorVC, JNumorVC, 0, 999999)
         DO i = 1, nNumorVC
            JNumorVC(i) = JNumorVC(i) + iNOff
            ENDDO
         qMon = .true.
      ELSE
         qMon = qAskD ('Normalization to monitor', qMon)
         ENDIF

      Print *
      Print *, 'Sample treatment :'

      qOutE   = .true. ! qAskD ('Conversion to energy', intq(qOutE))

      IF (nNumorVD.ne.0) THEN
         q0fromS= qAskDi (
     *      'Elastic channel from vanadium(0) or sample(1)', intq(q0fromS))
      ELSE
         q0fromS = .true.
         ENDIF

      IF (qOutE) THEN
         iSEG    = -1 ! neutron energy gain is positive
         qDetEff = qAskD ('Detector efficiency correction', intq(qDetEff))
      ELSE
         iSEG    = 0 ! not used
         qDetEff = .false.
         ENDIF

      IF (nNumorVD.ne.0) THEN
         Print *
         Print *, 'Vanadium treatment :'
         qVan = qAskD ('Take DWF for vanadium', intq(qVan))
         IF (.not.qVan) THEN
            u2xV = rAskDMu
     * ('Mean square displacement <(u_x)^2> [A^2]', u2xV, 0.d0, 10.d0)
            ENDIF

         ENDIF

      ! for use in EppCon (handle frame-overlap) :
      Print *
      Print *, 'Handling of frame overlap :'
      Print *, '   (0) do nothing'
      Print *, '   (1) set energies to limit the frame,'
      Print *, '   (2) explicitly enter channel numbers,'
C      Print *, '   (3) determine from global minimum of scattering'

      iFmod = iAskDMu ('Choose option', iFmod, 0, 3)
      IF    (iFmod.eq.1) THEN
         EFmax    = rAskDMu (
     * 'Maximal neutron energy gain in frame (meV)', EFmax, 0.d0, 1.d6)
         EFminRel = - rAskDLu (
     * 'Maximal energy loss in frame (unit=E0)', -EFminRel, 0.d0, 1.d0)
      ELSEIF (iFmod.eq.0) THEN
         NooF = 0
         NooD = 0
      ELSEIF (iFmod.eq.2) THEN
         NooF = iAskDMu (
     * 'Number of fast neutron channels to move around', NooF, 0, MCin)
         NooD = iAskDMu (
     * 'Number of intermediate channels to delete', NooD, 0, MCin)
      ELSE ! case iFmod=3
         ! nothing to ask
         ENDIF

      qAngEqAdd = qAskD (
     *   'Sum spectra at equal angle (before background subtraction)', 
     *   intq(qAngEqAdd))

      IF (iFmod.eq.1) THEN
         IF (EWmax.le.0. .or. EWmax.gt.EFmax) EWmax = EFmax ! default
      ELSE
         EFmax = 1.d19 ! no limitation
         IF (EWmax.le.0. .or. EWmax.gt.EFmax) EWmax = 1.d3 ! default
         ENDIF

      EWmax = EFmax ! rAskDMu ('Maximal energy on output', EWmax, 0.d0, EFmax)

      CALL GetNList ('Exclude spectra from analysis', LKnoAnal, qKnoAnal, MK)

 200  CONTINUE

C  --------------------------------------------------------------------
C  Get vanadium data / prepare normalization :
C  --------------------------------------------------------------------

      Write (35,'(/3a)') 'going to read data from instrument ', Inst,
     *     ', cycle '//cl6(iCycle)

      IF (qTheSame) THEN ! this option new 28aug91
         Print *
         Print *, 'vanadium as in last run'
         Write (35, '(/a)') 'Vanadium as above'

      ELSEIF (nNumorVD.le.0) THEN
         WRITE (35,'(/a)') 'No vanadium given - no normalization.'

      ELSE
         Print *
         Print *, 'getting vanadium runs :'

         Write (35,'(/a)') 'going to read vanadium run(s):'
         CALL RRT_InSub (qMon, Inst, iCycle, qKnoAnal,
     *               nNumorVD, JNumorVD, nNumorVC, JNumorVC, j,
     *               VTemp, EelastV, WaveLV, CwidthV, PeriodV, qAngEqAdd,
     *               nKV, AngleV, FPath, DetE1, DetE2, BadV, Fehler) 
         IF (Fehler.ne.'&ff') GOTO 99

         Print *, 'processing the vanadium :'

         IF (Inst.eq.'FCS') THEN
            CALL RRT_EPeak (j, nKV, AngleV, Xepfcs, YepV, BadV, Fehler)
            IF (Fehler.ne.'&ff') GOTO 99

            CALL RRT_EPeak_FCS (j, nKV, AngleV, FPath, Xepfcs,
     *        WaveLV, EelastV, CwidthV, PeriodV, Fehler)

            CALL RRT_EPeak (j, nKV, AngleV, XepV, YepV, BadV, Fehler)
            IF (Fehler.ne.'&ff') GOTO 99

         ELSE 
            CALL RRT_EPeak (j, nKV, AngleV, XepV, YepV, BadV, Fehler)
            IF (Fehler.ne.'&ff') GOTO 99
            ENDIF
         Write (35,'(/a)') 'analyse vanadium data :'

C  Correct for Debye-Waller factor of vanadium :
         IF (qVan) THEN
            u2xV = u2Debye (VTemp, 359.d0, 50.94d0, 2.5d-5) ! WuL6
            Write (*, '(a,f6.3,a)') ' <u_x^2> = ', u2xV*1000, '*10-3 A^2'
            ENDIF
         Write (35, '(a,f6.3,a)') 'Van : <u_x^2> = ', u2xV*1000, '*10-3 A^2'
         pi = 3.14159d0
         DWFmin = 1.d0
         DO K = 1, nKV
            elaQ     = 4*pi/WaveLV * dsind(AngleV(K)/2)
            DWF      = dexp (-u2xV * elaQ**2)
            YepV(K)  = YepV(K)  / DWF
            DWFmin   = dmin1(DWF,DWFmin)
            ENDDO
         Print *, ' >> DWF at highest angle : ', DWFmin

         CALL MemFileDel (j, Fehler)
         IF (Fehler.ne.'&ff') GOTO 99

         ENDIF

      IF (.not.qMon) WRITE (35,'(/a)') 'No normalization to monitor.'

C  --------------------------------------------------------------------
C  Get sample data / normalize and correct them :
C  --------------------------------------------------------------------

      Print *
      Print *, 'getting sample runs :'

      Write (35,*) 'going to read sample run(s):'
      CALL RRT_InSub (qMon, Inst, iCycle, qKnoAnal,
     *               nNumorSD, JNumorSD, nNumorSC, JNumorSC, j,
     *               STemp, Eelast, WaveL, Cwidth, Period, qAngEqAdd,
     *               nK, AngleS, FPath, DetE1, DetE2, BadS, Fehler) 
      IF (Fehler.ne.'&ff') GOTO 99

      IF (nNumorVD.gt.0) THEN
         ! check whether parameters from sample and vanadium are consistent :
         CALL RRT_Check ('sample vs vanadium',
     *                STemp, Eelast, WaveL, Cwidth, Period,
     *                nK, AngleS, FPath, DetE1, DetE2, 
     *                VTemp, EelastV, WaveLV, CwidthV, PeriodV,
     *                nKV, AngleV, FPath, DetE1, DetE2, 
     *                Fehler)
         IF (Fehler.ne.'&ff') GOTO 99
         ENDIF

      Write (35, '(/a)')        'TOF parameters (sample) :'
      Write (35, '(a28,g12.5)') '  wavelength (A) ', WaveL
      Write (35, '(a28,g12.5)') '  energy (meV) ', Eelast
      Write (35, '(a28,g12.5)') '  channel width (usec) ', Cwidth
      Write (35, '(a28,g12.5)') '  period (chs) ', Period

      Print *
      Print *, 'processing the sample :'

      Write (35,'(/a)') 'analyse sample data :'

      IF (Inst.eq.'FCS') THEN
         IF (nNumorVD.ne.0) THEN
            DO K = 1, nK
               XepS(K) = Xepfcs(K)
               ENDDO
            ELSE
            CALL RRT_EPeak (j, nK, AngleS, XepS, YepS, BadS, Fehler)
            ENDIF

         CALL RRT_EPeak_FCS (j, nK, AngleS, FPath, XepS,
     *       WaveL, Eelast, Cwidth, Period, Fehler)

         CALL RRT_EPeak (j, nK, AngleS, XepS, YepS, BadS, Fehler)
         IF (Fehler.ne.'&ff') GOTO 99

      ELSE
         CALL RRT_EPeak (j, nK, AngleS, XepS, YepS, BadS, Fehler)
         IF (Fehler.ne.'&ff') GOTO 99
         ENDIF

      IF (q0fromS) THEN
         Write (35, '(/a)') '  elastic peak from sample'
      ELSE
         Write (35, '(/a)') '  elastic peak from vanadium'
         ENDIF

      Write (35,*)
      WRITE (35,*) 'Parameters used for normalization :'
      qBad0 = .false.
      qBada = .false.
      qBadx = .false.
      qBadm = .false.
      qBade = .false.
      qBadv = .false.
      qBadg = .false.
      qBadn = .false.
      WRITE (35,*)
     *  '          |     EP Intensity    |    EP Position | Trouble |'
      WRITE (35,*)
     *  ' Spec|Ang.| Sample   |   Vana   | Sample | Vana  | S  |  V |'
      DO K=1, nK
         WRITE (35,435) K, AngleS(K),
     *                  YepS(K), YepV(K), XepS(K), XepV(K), BadS(K), BadV(K)
         IF     (BadV(K).eq.'0' .or. BadS(K).eq.'0') THEN
            qBad0 = .true.
         ELSEIF (BadV(K).eq.'x' .or. BadS(K).eq.'x') THEN
            qBadx = .true.
         ELSEIF (BadV(K).eq.'a' .or. BadS(K).eq.'a') THEN
            qBada = .true.
         ELSEIF (BadV(K).eq.'-' .or. BadS(K).eq.'-') THEN
            qBadm = .true.
         ELSEIF (BadV(K).eq.'e' .or. BadS(K).eq.'e') THEN
            qBade = .true.
         ELSEIF (BadV(K).eq.'n' .or. BadS(K).eq.'n') THEN
            qBadn = .true.
         ELSEIF (BadV(K).eq.'v' .or. BadS(K).eq.'v') THEN
            qBadv = .true.
         ELSEIF (BadV(K).eq.'g' .or. BadS(K).eq.'g') THEN
            qBadg = .true.
            ENDIF
         ENDDO
435   FORMAT (2X,i3,X,F5.1,X,G10.4,X,G10.4,X,F6.2,2x,G10.4,x,a3,2x,a3)
      Write (35, *)
      ! explain error marks :
      IF (qBadx) Write(35, *) '   trouble x : spectrum excluded a priori'
      IF (qBad0) Write(35, *) '   trouble 0 : empty data block'
      IF (qBadm) Write(35, *) '   trouble - : negative count rates'
      IF (qBada) Write(35, *) 
     *        '   trouble a : invalid angle (monitor or special block)'
      IF (qBadv) Write(35, *) '   trouble v : nearly vanishing peak'
      IF (qBadg) Write(35, *) '   trouble g : giant peak'
      IF (qBade) Write(35, *) '   trouble e : excentric peak'
      IF (qBadn) Write(35, *) '   trouble n : normalization impossible'
      qBad = qBad0 .or. qBadm .or. qBada .or. qBadv .or. qBadg .or.
     *       qBade .or. qBadn .or. qBadx

      Print *
      Print *, 'conversion to energy :'
      Write (35,'(/a)') ' conversion to energy :'
      IF     (iSEG.eq.+1) THEN
         Write (35, '(a)') ' E>0 means sample energy gain'
      ELSEIF (iSEG.eq.-1) THEN
         Write (35, '(a)') ' E>0 means neutron energy gain'
         ENDIF
      IF (q0fromS) THEN
         CALL RRT_Energ (qOutE, iSEG, XepS, BadS, AngleS, FPath,
     *                iFmod, NooW, NooD,
     *                EFmax, EWmax, EFminRel,
     *                WaveL, Eelast, Cwidth, Period,
     *                j, nK, Fehler)
      ELSE
         CALL RRT_Energ (qOutE, iSEG, XepV, BadV, AngleV, FPath,
     *                iFmod, NooW, NooD,
     *                EFmax, EWmax, EFminRel,
     *                WaveL, Eelast, Cwidth, Period,
     *                j, nK, Fehler)
         ENDIF

      IF (qDetEff) THEN
         WRITE (35,*) 'detector efficiency correction for ', Inst
         Print *, 'detector efficiency correction'
         CALL RRT_DetEff (j, nK, Eelast, DetE1, DetE2, Fehler)
      ELSE
         WRITE (35,*) 'no detector efficiency correction'
         ENDIF
      IF (Fehler.ne.'&ff') GOTO 99

C  Normalisation to the vanadium
       ! The normalisation to vanadium should be done after the grouping
       ! otherwise one should take an average over Angle instead of a sum

      IF (nNumorVD.eq.0) THEN
         Print *, 'no vanadium - no normalization'
         DO K=1, nK
            BadV(K) = ' '
            ENDDO
      ELSE
         Print *, 'normalization to vanadium'
         CALL RRT_Norm (j, nK, YepV, BadV, BadS, Fehler)
         IF (Fehler.ne.'&ff') GOTO 99
         CALL OlfComAdd (j, ' ', 'normalized to '//ListVD, Fehler)
         ENDIF

C  --------------------------------------------------------------------
C     Output :
C  --------------------------------------------------------------------

C  default output filename :
      CALL FrageCD ('File name', FilOut, 'r'//cl6(JNumorSD(1)))
      CALL FrageCD ('And title', Title, Title)
      CALL tOlfP (j, 'fil', FilOut, Fehler)
      CALL tOlfP (j, 'tit', Title,  Fehler)
      IF (Fehler.ne.'&ff') GOTO 99

      IF (qTheSame) THEN
         Print *, ' delete some spectra according to option ', iDel
      ELSE
         Print *
         Print *, 'Output :'
         Print *, 'Delete spectra :'
         Print *, '   (0) none                    (1) if some data are bad'
         Print *, '   (2) if sample data are bad  (3) if vanadium data are bad'
         Print *, '   (4) enter list yourself  '
         iDel = iAskDMu ('Option ', iDel, 0, 4)
         ENDIF
      Print *

C  For bad data blocks, set AngleS=0 :
      IF     (iDel.eq.1) THEN
         DO K = 1, nK
            IF (BadS(K).ne.' ' .or. BadV(K).ne.' ') AngleS(K) = 0.
            ENDDO
      ELSEIF (iDel.eq.2) THEN
         DO K = 1, nK
            IF (BadS(K).ne.' ') AngleS(K) = 0.
            ENDDO
      ELSEIF (iDel.eq.3) THEN
         DO K = 1, nK
            IF (BadV(K).ne.' ') AngleS(K) = 0.
            ENDDO
      ELSEIF (iDel.eq.4) THEN
         CALL GetNList ('Delete spectra nos.', LisKdel, qLisK, nK)
         DO K = 1, nK
            IF (qLisK(K)) AngleS(K) = 0.
            ENDDO
         ENDIF

      Write (35,'(/2a)') 'Output: ', FilOut
      Write (35,'(2a/)') 'Title:  ', Title

      CALL RRT_Sort (j, nK, AngleS, STemp, qOutE, Fehler)
      IF (Fehler.ne.'&ff') RETURN

C  Outermost loop: next run ?
      Print *
      Print *, 'Next data set:'
      GOTO 110

C  Errors :
 99   CONTINUE
      Close(35)

      END ! RRawTOF

C  --------------------------------------------------------------------
      SUBROUTINE RRT_InSub (qMon, Inst, iCycle, qKnoAnal,
     *                nNumor, JNumor, nNumor1, JNumor1, j,
     *                Temp, Eelast, WaveL, Cwidth, Period, qAngEqAdd,
     *                nK, Angle, FPath, DetE1, DetE2, Status, Fehler)
C  --------------------------------------------------------------------
         ! read in, subtract empty can

      IMPLICIT REAL*8  (a-h,o-p,r-z)
      IMPLICIT LOGICAL (q)
      INCLUDE      'l_def.f'
      INCLUDE      'i_dim.f'
      INCLUDE      'i_wrk.f'

      DIMENSION     JNumor(*), JNumor1(*)
      DIMENSION     Angle(*), FPath(*), DetE1(*), DetE2(*),
     *              Angle1(MK), FPath1(MK), DetE11(MK), DetE21(MK)
      CHARACTER*1   Status(*)
      CHARACTER*(*) Inst, Fehler
      LOGICAL       qKnoAnal(*)

      DATA          LisK /'-'/

C  Read first file:
      CALL RRT_InSum (qMon, Inst, iCycle, nNumor, JNumor, j, 
     *                Temp, Eelast, WaveL, Cwidth, Period, qAngEqAdd,
     *                nK, Angle, FPath, DetE1, DetE2, Fehler)
      IF (Fehler.ne.'&ff') RETURN

      DO K = 1, nK
         IF (Angle(K).le.0) THEN
            Status(K) = 'a'
         ELSEIF (qKnoAnal(K)) THEN
            Status(K) = 'x'
         ELSE
            Status(K) = ' '
            ENDIF
         ENDDO

C  Read second file ?
      IF (nNumor1.le.0) RETURN

      Write (35, '(a)') 'subtract background:'
      CALL RRT_InSum (qMon, Inst, iCycle, nNumor1, JNumor1, j1, 
     *                Temp1, Eelast1, WaveL1, Cwidth1, Period1, qAngEqAdd,
     *                nK1, Angle1, FPath1, DetE11, DetE21, Fehler)
      IF (Fehler.ne.'&ff') RETURN

      CALL RRT_Check ('Subtract background/ ',
     *                Temp, Eelast, WaveL, Cwidth, Period,
     *                nK, Angle, FPath, DetE1, DetE2, 
     *                Temp1, Eelast1, WaveL1, Cwidth1, Period1,
     *                nK1, Angle1, FPath1, DetE11, DetE21, Fehler)
      IF (Fehler.ne.'&ff') RETURN

      DO K = 1, nK
         CALL OlfGetXYD (j,  K, n, X, Y, D, Fehler)
         IF (Fehler.ne.'&ff') RETURN
         CALL OlfGetXYD (j1, K, n1, X1, Y1, D1, Fehler)
         IF (Fehler.ne.'&ff') RETURN
         IF (n.ne.n1) THEN 
            Fehler = 'RRT_InSub/ Inconsistent length of spectra '//cl4(K)
            RETURN
            ENDIF
         DO i = 1, n
            Y(i) = Y(i) - Y1(i)
            D(i) = dsqrt( D(i)**2 + D1(i)**2 )
            ENDDO
         CALL OlfPutXYD (j, K, n, X, Y, D, Fehler)
         IF (Fehler.ne.'&ff') RETURN
         ENDDO

      CALL MemFileDel (j1, Fehler)
      IF (Fehler.ne.'&ff') RETURN
      
      END ! RRT_InSub

C  --------------------------------------------------------------------
      SUBROUTINE RRT_InSum (qMon, Inst, iCycle, nNumor, JNumor, j,
     *                      Temp, Eelast, WaveL, Cwidth, Period, qAngEqAdd,
     *                      nK, Angle, FPath, DetE1, DetE2, Fehler)
C  --------------------------------------------------------------------
         ! sum numors
         ! normalize to monitor, set error

      IMPLICIT REAL*8  (a-h,o-p,r-z)
      IMPLICIT LOGICAL (q)
      INCLUDE      'l_def.f'
      INCLUDE      'i_dim.f'
      INCLUDE      'i_wrk.f'

      DIMENSION     JNumor(*)
      DIMENSION     Angle(*), FPath(*), DetE1(*), DetE2(*),
     *              Angle1(MK), FPath1(MK), DetE11(MK), DetE21(MK)
      CHARACTER*(*) Inst, Fehler

         Write (35,*) nNumor, ' run(s) to be read:'
         Write (35,'(20i5)') (JNumor(iNu),iNu=1, nNumor)

C  First File:
      CALL RRT_Input (Inst, iCycle, JNumor(1), j, 
     *                rMon, Temp, Eelast, WaveL, Cwidth, Period,
     *                nK, Angle, FPath, DetE1, DetE2, Fehler)
      IF (Fehler.ne.'&ff') RETURN

C  Add more files ?
      DO iNu = 2, nNumor

         CALL RRT_Input (Inst, iCycle, JNumor(iNu), j1,
     *                   rMon1, Temp1, Eelast1, WaveL1, Cwidth1, Period1,
     *                   nK1, Angle1, FPath1, DetE11, DetE21, Fehler)
         IF (Fehler.ne.'&ff') RETURN

         CALL RRT_Check ('Sum numors/ ',
     *                   Temp, Eelast, WaveL, Cwidth, Period,
     *                   nK, Angle, FPath, DetE1, DetE2, 
     *                   Temp1, Eelast1, WaveL1, Cwidth1, Period1,
     *                   nK1, Angle1, FPath1, DetE11, DetE21, Fehler)
         IF (Fehler.ne.'&ff') RETURN

         rMon = rMon + rMon1

         DO K = 1, nK
            CALL OlfGetXYD (j,  K, n, X, Y, D, Fehler)
            CALL OlfGetXYD (j1, K, n1, X1, Y1, D1, Fehler)
            IF (Fehler.ne.'&ff') RETURN
            IF (n.ne.n1) THEN 
               Fehler = 'RRT_InSum/ Inconsistent length of spectra '//cl4(K)
               RETURN
               ENDIF
            DO i = 1, n
               Y(i) = Y(i) + Y1(i)
               ENDDO
            CALL OlfPutXYD (j, K, n, X, Y, D, Fehler)
            IF (Fehler.ne.'&ff') RETURN
            ENDDO

         CALL MemFileDel (j1, Fehler)
         IF (Fehler.ne.'&ff') RETURN

         ENDDO ! sum numors

C  Save summed monitor:
      CALL rOlfP (j, 'cts[mon]', ' ', rMon, Fehler)
      IF (Fehler.ne.'&ff') RETURN

C  Normalize to monitor and set error :
      IF (qMon .and. rMon.le.0.d0) THEN
         Print *, 'WARNING/ monitor = 0'
         rMon = rAsk ('Enter monitor')
         ENDIF
      DO K = 1, nK
         CALL OlfGetXYD (j, K, n, X, Y, D, Fehler)
         IF (Fehler.ne.'&ff') RETURN
         IF (qMon) THEN
            DO i = 1, n
               D(i) = dsqrt(Y(i)) / rMon
               Y(i) = Y(i) / rMon
               ENDDO
         ELSE
            DO i = 1, n
               D(i) = dsqrt(Y(i))
               ENDDO
            ENDIF
         CALL OlfPutXYD (j, K, n, X, Y, D, Fehler)
         IF (Fehler.ne.'&ff') RETURN
         ENDDO

      IF (qAngEqAdd) CALL RRT_Add (j, nK, Angle, Fehler)

      END ! RRT_InSum

C  --------------------------------------------------------------------
      SUBROUTINE RRT_Input (Inst, iCycle, iNumor, j, 
     *                      rMon, Temp, Eelast, WaveL, Cwidth, Period,
     *                      nK, Angle, FPath, DetE1, DetE2, Fehler)
C  --------------------------------------------------------------------
         ! call institute specific input routines

      IMPLICIT REAL*8  (a-h,o-p,r-z)
      IMPLICIT LOGICAL (q)
      INCLUDE      'i_dim.f'
      INCLUDE      'i_wrk.f'

      DIMENSION     Angle(*), FPath(*), DetE1(*), DetE2(*)
      CHARACTER*(*) Inst, Fehler
      CHARACTER*80  DirRaw, LongTit, LongDat

C  Open on-line file :
      CALL OlfCreate (j, Kout, '&noask', '&noask', Fehler)
      IF (Fehler.ne.'&ff') RETURN

C  Get pathname from setup file :
      IF     (iCycle.eq.-1) THEN
         CALL ExeML ('\p dir-raw-n-dac', DirRaw)
      ELSEIF (iCycle.eq. 0) THEN
         CALL ExeML ('\p dir-raw-n-new', DirRaw)
      ELSE
         CALL ExeML ('\p dir-raw-n-old', DirRaw)
         ENDIF 

C  Call specific input routine:
      IF     (Inst(1:2).eq.'IN') THEN
         CALL RRT_In_ILL (Inst, DirRaw, iCycle, iNumor, j, rMon,
     *               Temp, WaveL, Cwidth, Period, LongTit, LongDat,
     *               nK, Angle, FPath, DetE1, DetE2, Fehler)
         IF (Fehler.ne.'&ff') RETURN
         Eelast  = 81.805 / WaveL**2             ! in meV
      ELSEIF (Inst.eq.'MIB') THEN
         CALL RRT_In_Mib (DirRaw, iCycle, iNumor, j, 
     *               rMon, Temp, WaveL, Cwidth, Period, LongTit,
     *               nK, Angle, FPath, DetE1, DetE2, Fehler)
         IF (Fehler.ne.'&ff') RETURN
         Eelast  = 81.805 / WaveL**2             ! in meV
      ELSEIF (Inst.eq.'FCS') THEN
        CALL RRT_In_Fcs (DirRaw, iNumor, j, 
     *               rMon, Temp, WaveL, Cwidth, Period, LongTit,
     *               nK, Angle, FPath, DetE1, DetE2, Fehler)
         IF (Fehler.ne.'&ff') RETURN
         Eelast  = 81.805 / WaveL**2             ! in meV
      ELSEIF (Inst.eq.'NEAT') THEN
        CALL RRT_In_NEAT (DirRaw, iNumor, j, 
     *               rMon, Temp, WaveL, Cwidth, Period, LongTit,
     *               nK, Angle, FPath, DetE1, DetE2, Fehler)
         IF (Fehler.ne.'&ff') RETURN
         Eelast  = 81.805 / WaveL**2             ! in meV
      ELSE
         Fehler = 'RRT_Input/ Unknown instrument'
         RETURN 
         ENDIF

      Write (35, *) 'read numor ', iNumor, ' - Monitor = ', rMon

      CALL OlfComAdd (j, ' ', Inst(1:lenU(Inst))//': '//LongDat, Fehler)
      CALL OlfComAdd (j, ' ', '"'//LongTit(1:lenU(LongTit))//'"', Fehler)

      CALL iOlfP (j, '?det-bal-sym',   0, Fehler)
      CALL iOlfP (j, '@sam-erg-gain', -1, Fehler)
      CALL iOlfP (j, 'plot-sy#',  0, Fehler)
                   
      CALL rOlfP (j, 'E0', 'meV', Eelast, Fehler)
      CALL rOlfP (j, 'T',  'K',   Temp,   Fehler)

      END ! RRT_Input

C  --------------------------------------------------------------------
      SUBROUTINE RRT_In_ILL (Inst, DirRaw, iCycle, iNumor, j, rMon, 
     *                  Temp, WaveL, Cwidth, Period, LongTit, LongDat,
     *                  nK, Angle, FPath, DetE1, DetE2, Fehler)
C  --------------------------------------------------------------------
            !  Von K.Bauszus eingetippt, 1993.
            !  Modifications after ILL-upstart mrz95
            !  Idol by Helga Schwab, removed 6mai98

      IMPLICIT REAL*8  (a-h,o-p,r-z)
      IMPLICIT LOGICAL (q)

      INCLUDE      'l_def.f'
      INCLUDE      'i_dim.f'
      INCLUDE      'i_wrk.f'
      PARAMETER    (MM=156, ML=384, MN=128, MD=512)
      REAL*8        Angle(*), FPath(*), DetE1(*), DetE2(*),
     *              Par0(MM), Par1(ML), Par2(MN)
      INTEGER       iDet(MD)

      CHARACTER*80  FileRaw, line
      CHARACTER*(*) Inst, Fehler, DirRaw, LongTit, LongDat

C  Direct access or not ?
      IF (DirRaw.eq.'&error') THEN 
         Fehler = 'Directory not accessible (&error)'
         RETURN 
         ENDIF
   
      FileRaw = DirRaw
      
      IF (iCycle.lt.1000) THEN
         CALL ReplaceT (FileRaw, '&cycle', cv3(iCycle))
      ELSE
         CALL ReplaceT (FileRaw, '&cycle', cv4(iCycle))
         ENDIF
      
      IF     (Inst.eq.'IN6') THEN
         CALL ReplaceT (FileRaw, '&inst', 'in6')
      ELSEIF (Inst.eq.'IN5') THEN
         CALL ReplaceT (FileRaw, '&inst', 'in5')
      ELSEIF (Inst.eq.'IN4') THEN
         CALL ReplaceT (FileRaw, '&inst', 'in4')
      ELSE
         Fehler = 'Invalid instrument : '//Inst
         RETURN
         ENDIF

C  Construct file name :
      CALL Append (FileRaw, cv6(iNumor))

C  Open the input files :
      CALL OpenFile (11, FileRaw, '&noext', 'a', Fehler)
      IF (Fehler.ne.'&ff') RETURN

C  Read header blocks :
C  - Block 1: Numor
      Read (11, '(/a80)', err=91) line 
      CALL Fi1I (line, jrun)
      IF    (jrun.ne.iNumor) THEN
         Print *, 'header line "', line(1:lenU(line)), '"'
         Print *, '#run : given/read ', iNumor, jrun
         Fehler = 'inconsistent number of run or invalid data format'
         Close(11)
         RETURN
         ENDIF
      
C  - Block 2: Short title
C  - Block 3: Some integer parameters (Par0)
      Read (11, '(/////16(10(f8.0)/))', err=92, end=11) (Par0(i), i=1,156)
 11   CONTINUE 
      
      nK = Par0(1) 

C  - Block 4: Long title
      Read(11, '(//a80/a80/////)') LongTit, LongDat
      Print *, LongTit
      Print *, LongDat
      
C  - Block 5: Real parameters (Par1), mainly scattering angles
      Read (11,'(a)') line
      IF (line(1:3).ne.'FFF') THEN 
         Print *, line
         Fehler = 'RRT_In_ILL/ Par2/ expected FFFFFF'
         RETURN
         ENDIF
      Read (11, '(i8)', err=93) nPar1
      Read (11, '(80(5(2x,d14.8)/))', err=93, end=12) (Par1(i), i=1,nPar1)
 12   CONTINUE 

C  - Block 6: Other real parameters (Par2) 
      Read (11,'(a)') line
      IF (line(1:3).ne.'FFF') THEN 
         Print *, line
         Fehler = 'RRT_In_ILL/ Par2/ expected FFFFFF'
         RETURN
         ENDIF 
      Read (11, '(/80(5(2x,d14.8)/))', err=94, end=13) (Par2(i), i=1,128)
 13   CONTINUE 
      
      IF (Inst.eq.'IN5') THEN
         Read (11,'(27(/),x)') ! a new, nonsensical field (128*real)
         ENDIF ! (vgl. Mail von Andreas Meyer 26aug97)

C  A new field, intended to contain a code for the detector type :
      Read (11,'(a)') line ! dieser Test 1okt96
      IF (line(1:3).ne.'III') THEN 
         Print *, line
         Fehler = 'RRT_In_ILL/ Det-Codes/ expected III'
         RETURN
         ENDIF 
      Read (11, '(i8)', err=95) niDet
      Read (11, '(53(10(i8)/))', err=96, end=14) (iDet(i), i=1,niDet)
 14   CONTINUE

C  Read data blocks :
      IF (nK.gt.MK) THEN
         CALL Compose2 (Fehler, 'RRT_In_ILL/ found '//cl6(nK),
     *      ' data blocks while MK='//cl6(MK))
         RETURN
         ENDIF
      
      DO K = 1, nK
         Read (11,'(a)') line
         IF (line(1:3).ne.'SSS') THEN
            Print *, 'spectrum '//cl3(K)
            Print *, line
            Fehler = 'RRT_In_ILL/ expected SSSSSSS'
            RETURN 
            ENDIF 
         Read (11,'(//i8)', err=97) iBlo
         IF (iBlo.ne.512) THEN 
            Fehler = 'RRT_In_ILL/ #channels <> 512 in spectrum '//cl3(K)
            RETURN 
            ENDIF 
         Read (11, '(10f8.0)', err=98)  (Y(i), i=1,512)
         IF (K.eq.1) THEN
            rMon = 0
            DO i = 1, 511
               rMon = rMon + Y(i)
               ENDDO
            ENDIF
         CALL OlfPutSpe (j, K, 1, 0.d0, 511, X, Y, D, Fehler)
            ! ch. 512 is used for no. of block
         IF (Fehler.ne.'&ff') RETURN
         ENDDO
      Close (11)
 
C  Checks:
      nCnom = jidnnt(Par2(2))  ! nominelle Zahl der verwendeten Kanaele 
      IF (nCnom.ne.512) THEN
         Fehler = 'RRT_In_ILL/ Decode Par2/ unexpected #chs. = '//cl6(nCnom)
         RETURN
         ENDIF

C  Par2 enth"alt SetUp-Daten :
         Temp   = Par2(11)          ! Probentemp
         Period = Par2(13)          ! Periode der Messung
         Cwidth = Par2(18)          ! Kanalbreite
         WaveL  = Par2(21)          ! Wellenlaenge
         Eelast = 81.805 / WaveL**2             ! in meV

C  K dependent parameters :
         DO K = 1, nK
            Angle (K) = Par1(K+31) ! die ersten 30 Zahlen sind keine Winkel
            FPath (K) = Par2(27)   ! Abstand Probe Detektor      

            DetE1(K) = 1001 ! preset = nonsense
            DetE2(K) = 1002 ! preset = nonsense

            IF     (Inst.eq.'IN4') THEN
               DetE1(K) = - .0887
               DetE2(K) = -5.597
            ELSEIF (Inst.eq.'IN5') THEN
               IF     (iDet(K).eq.0) THEN ! monitor or special block
                  Angle(K) = -1
               ELSEIF (iDet(K).eq.1) THEN ! IN5 type detectors (8bars,D = .9cm)
                  DetE1(K) = - .0887
                  DetE2(K) = -4.07
               ELSEIF (iDet(K).eq.2) THEN ! IN6 type detectors
                  DetE1(K) = -.0565
                  DetE2(K) = -3.284
               ELSEIF (iDet(K).eq.3) THEN ! IN6 type detectors
                  DetE1(K) = -.0565
                  DetE2(K) = -3.284
               ELSEIF (iDet(K).eq.4) THEN ! IN6 type detectors
                  DetE1(K) = -.0565
                  DetE2(K) = -3.284
               ELSEIF (iDet(K).eq.5) THEN ! Multidetector
                  DetE1(K) = -.0565
                  DetE2(K) = -3.284
                  FPath(K) =  Par2(27) - .30
                  ENDIF
            ELSEIF (Inst.eq.'IN6') THEN
               DetE1(K) = -.0565
               DetE2(K) = -3.284
               IF (K.le.6) Angle(K) = 0 ! monitor blocks
               ENDIF
         ENDDO

C  Errors :
      RETURN
 91   CONTINUE
      CALL Compose2 (Fehler, 
     *               'Error while reading 1st lines of "'//FileRaw, '"')
      Close(11)
      RETURN
 92   CONTINUE 
      Fehler = 'Read from data file / error 92'
      RETURN 
 93   CONTINUE 
      Fehler = 'Read from data file / error 93'
      RETURN 
 94   CONTINUE 
      Fehler = 'Read from data file / error 94'
      RETURN 
 95   CONTINUE
      Fehler = 'Could not read new integer block (with detector code)'
      RETURN
 96   CONTINUE 
      Fehler = 'Read from data file / error 96'
      RETURN 
 97   CONTINUE 
      Fehler = 'Read from data file / error 97'
      RETURN 
 98   CONTINUE 
      Fehler = 'Read from data file / error 98'
      RETURN 

      END ! RRT_In_ILL

C  --------------------------------------------------------------------
      SUBROUTINE RRT_In_NEAT (DirRaw, iNumor, j, rMon, 
     *                  Temp, WaveL, Cwidth, Period, LongTit,
     *                  nK, Angle, FPath, DetE1, DetE2, Fehler)
C  --------------------------------------------------------------------

      IMPLICIT REAL*8  (a-h,o-p,r-z)
      IMPLICIT LOGICAL (q)

      INCLUDE      'l_def.f'
      INCLUDE      'i_dim.f'
      INCLUDE      'i_wrk.f'
      PARAMETER    (MM=156, ML=384, MN=128, MD=512)
      DIMENSION     Angle(*), FPath(*), DetE1(*), DetE2(*),
     *              Par0(MM), Par2(MN), iDet(MD)

      CHARACTER*80  FileRaw, line
      CHARACTER*(*) Fehler, DirRaw, LongTit

      INTEGER       j,iang,nbheaderblock
      INTEGER       nbchan_SD,nbspec_SD,int_ty_SD,nbblock_SD
      INTEGER*2     buffer1(256)
      INTEGER*4     buffer3(4096)
      REAL*8        buffer2(128)
      CHARACTER*20  LongDat

C  Direct access or not ?
      IF (DirRaw.eq.'&error') THEN 
         Fehler = 'Directory not accessible (&error)'
         RETURN 
         ENDIF
   
C  Construct file name and open input file :
      FileRaw = DirRaw
      CALL Append (FileRaw, cv4(iNumor))
      CALL Append (FileRaw, 'neat.out')
      CALL OpenFile (11, FileRaw, '&noext', 'a', Fehler)
      IF (Fehler.ne.'&ff') THEN
         Fehler = '&ff'
         FileRaw = DirRaw
         CALL Append (FileRaw, cv4(iNumor))
         CALL Append (FileRaw, 'NEAT.OUT')
         CALL OpenFile (11, FileRaw, '&noext', 'a', Fehler)
         ENDIF 
      IF (Fehler.ne.'&ff') RETURN

C  Read header blocks :

C.......Read Header-Block No 1

      Read (11, '(a80)', err=91) line 
      IF (line(1:3).ne.'RRR') THEN
         Print *, line
         Fehler = 'missed beginning of block 1'
         RETURN 
         ENDIF 
      DO i = 1, 16
         READ (11, '(a)', Err=91) line ! (buffer1(i),i=1,256)
         ENDDO 
      nbheaderblock = 10        ! buffer1(256) ! nach Konversion i2 -> i4 !
cdeb      Print *, 'read block 1'

C.......Read Header-Block No 2

      Read (11, '(a80)') line
      IF (line(1:3).ne.'RRR') THEN
         Print *, line
         Fehler = 'missed beginning of block 2'
         RETURN 
         ENDIF 
      Read (11, '(a80)') LongTit
      Print '(2a)', ' Run : ', LongTit(1:20) ! , 'started at :', LongDat(1:20)
cdeb      Print *, 'read block 2'

C.......Read Header-Block No 3

      Read (11, '(a80)') line
      IF (line(1:3).ne.'RRR') THEN
         Fehler = 'missed beginning of block 3'
         RETURN
         ENDIF 
      Read (11, '(15(8(d10.2)/),8d10.2)') (buffer2(i), i=1,128)

        nbchan_SD=buffer2(56)
        nbspec_SD=buffer2(70)
        int_ty_SD=buffer2(14)
        nbblock_SD=nbchan_SD/256*(1+int_ty_SD)

      Period = buffer2(53)     
      Cwidth = buffer2(61) * .125   ! Kanalbreite (usec) < time step 125nsec
      WaveL  = buffer2(27)          ! Wellenlaenge
      FPathSD= buffer2(21) / 1000   ! Abstand Probe - Detektor (mm -> m)

      Period = Period / Cwidth ! (#chs)

      nC = nbchan_SD
      nK = nbspec_SD

      Print *, '#spec, #block, #chan :', nbspec_SD, nbblock_SD, nbchan_SD
      IF (nC.gt.MC) THEN
         Fehler = 'channels/spectrum > MC'
         RETURN
         ENDIF 
      IF (nK.gt.MK) THEN
         CALL Compose2 (Fehler, 'RRT_In_NEAT/ found '//cl6(nK),
     *      ' data blocks while MK='//cl6(MK))
         RETURN
         ENDIF
cdeb      Print *, 'read block 3'

C.......Read single detector angles
        iang=0
        DO i=1,16
           iang=iang+1
c          IF (iang .GT. nbspec_SD) GOTO 50
           Angle(i)=buffer2(112+i)
           ENDDO
        Read (11, '(a80)') line
        IF (line(1:3).ne.'RRR') THEN
           Print *, '::', line(1:76)
           Fehler = 'missed beginning of block 4'
           RETURN
           ENDIF 
        READ (11, '(15(8(d10.2)/),8d10.2)', err=94) (buffer2(i),i=1,128)
        DO i=17,144
           iang=iang+1
c          IF (iang .GT. nbspec_SD) GOTO 50
           Angle(i)=buffer2(i-16)
           ENDDO
        Read (11, '(a80)') line
        IF (line(1:3).ne.'RRR') THEN
           Print *, '::', line(1:76)
           Fehler = 'missed beginning of block 5'
           RETURN
           ENDIF 
        READ (11, '(15(8(d10.2)/),8d10.2)', err=94) (buffer2(i),i=1,128)
        DO i=145,272
           iang=iang+1
c           IF (iang .GT. nbspec_SD) GOTO 50
           Angle(i)=buffer2(i-144)
           ENDDO
        Read (11, '(a80)') line
        IF (line(1:3).ne.'RRR') THEN
           Fehler = 'missed beginning of block 6'
           RETURN
           ENDIF 
        READ (11, '(15(8(d10.2)/),8d10.2)', err=94) (buffer2(i),i=1,128)
        DO i=273,400
           iang=iang+1
           IF (iang .GT. nbspec_SD) GOTO 50
           Angle(i)=buffer2(i-272)
           ENDDO
50    CONTINUE
cdeb      Print *, 'read angles, iang = ', iang

C.......Read single detector spectra

      Read (11, '(a80)') line
      IF (line(1:3).ne.'RRR') THEN
           Print *, '::', line(1:76)
         Fehler = 'missed beginning of data blocks'
         RETURN
         ENDIF 
      DO KK=1, nbspec_SD
         IF (nbchan_SD.ne.512) THEN
            Fehler = 'data format requires fixed nC'
            RETURN 
            ENDIF 
         READ (11, '(51(10i8/),2i8)', end=51) (buffer3(i),i=1,512)
 51      CONTINUE
         DO i=1,nbchan_SD
            Y(i)=buffer3(i)
            ENDDO
         IF (KK.eq.1) THEN
            rMon = 0
            DO i = 1, nC
               rMon = rMon + Y(i)
               ENDDO
            ENDIF 
         CALL OlfPutSpe (j, KK, 1, 0.d0, nbchan_SD, X, Y, D, Fehler)
         IF (Fehler.ne.'&ff') RETURN
         ENDDO
cdeb      Print *, 'read single detector spectra'

C  skip multidetector data ...

C  end of input file
      Close (11)

      TempIn = rAskD (' Temperature', TempIn)         ! Probentemp
      Temp   = TempIn
      Eelast = 81.805 / WaveL**2             ! in meV

C  K dependent parameters :
      Print *, ' flight path SD (m) = ', FPathSD
      DO K = 1, nK
         FPath (K) = FPathSD  ! Abstand Probe Detektor (m) 
         DetE1(K) = -.0565
         DetE2(K) = -3.284
         ENDDO

C  Errors :
      RETURN
 91   CONTINUE
      CALL Compose2 (Fehler, 
     *               'Error while reading 1st lines of "'//FileRaw, '"')
      Close(11)
      RETURN
 94   CONTINUE
      Fehler = 'Error while reading integer block'
      RETURN

      END ! RRT_In_NEAT

C  --------------------------------------------------------------------
      SUBROUTINE RRT_In_Mib (DirRaw, iCycle, irun, j,  
     *                  rMon, Temp, WaveL, Cwidth, Period, LongTit,
     *                  nK, Angle, FPath, DetE1, DetE2, Fehler)
C  --------------------------------------------------------------------
         !  Lecture des donnees du spectrometre temps de vol MIBEMOL de Saclay
         !  Version 1 by M.Bee, 25jan91
         !  Adapted to new raw data format by J.Wuttke, 25jul91

         !  Reads two input files d++++.ust and d++++.asc,
         !  where ++++ is a four-digit integer.

      IMPLICIT REAL*8  (a-h,o-p,r-z)
      IMPLICIT LOGICAL (q)
      INCLUDE      'i_dim.f'
      INCLUDE      'i_wrk.f'
      DIMENSION     Angle(*), FPath(*), DetE1(*), DetE2(*)

      CHARACTER*(*)  Fehler, LongTit, DirRaw
      CHARACTER*80   inline, aus, FileUST, FileASC, nam
      CHARACTER      cl5*5, cl4*4, cl3*3, cl2*2, cv4*4

      IF (Fehler.ne.'&ff') THEN
         Print *, 'RRT_In_Mib/ Fehler on entry'
         RETURN
         ENDIF

C  Open the input files :
      IF (DirRaw.eq.' ') THEN
         nam = 'd'
      ELSE
         CALL Compose2 (nam, DirRaw, 'd')
         ENDIF
      CALL ReplaceT (nam, '&cycle', cv4(iCycle))

      IF (irun.ge.1 .and. irun.lt.9999) THEN
         isub = 0 ! accumulated data
      ELSEIF (irun.ge.100000 .and. irun.le.999999) THEN
         isub = jmod (irun, 100)
         irun = (irun - isub) / 100
      ELSE
         Fehler = 'RRT_In_Mib/ bad number of run'
         RETURN
         ENDIF

      CALL Compose3 (FileUST, nam, cl4(irun), '.ust')
      IF (isub.le.0) THEN
         CALL Compose3 (FileASC, nam, cl4(irun), '.asc')
      ELSE
         CALL Compose3 (FileASC, nam, cl4(irun), '.pro'//cl2(isub))
         ENDIF
      CALL OpenFile (11, FileUST, '&noext', 'a', Fehler)
      IF (Fehler.ne.'&ff') RETURN 
      CALL OpenFile (12, FileASC, '&noext', 'a', Fehler)
      IF (Fehler.ne.'&ff') RETURN 

C  Read *.ust :
      IF (irun.ge.4000) THEN ! new format encountered 2/98
         Read (11, '(x)') ! -/- 
         IF (irun.ge.6000) THEN ! warum muessen die immer was aendern ?
            Read (11, '(19x,i4)') jrun
         ELSE
            Read (11, '(20x,i4)') jrun
            ENDIF 
         Print *, ' reading ust for d'//cl4(jrun)
         Read (11, '(x)') ! date
         Read (11, '(x)') ! -/-
         Read (11, '(x)') ! subr max
         Read (11, '(x)') ! subr is
         Read (11, '(x)') ! n-count control
         Read (11, '(x)') ! T control
         Read (11, '(a)') inline
            CALL Fi1R (inline, Temp)
            print *, ' read T = ', Temp
         Read (11, '(x)') ! delta T
         Read (11, '(x)') ! waiting time
         Read (11, '(x)') ! grad
         Read (11, '(a)') inline
            CALL Fi1R (inline, delay)
            print *, ' read delay = ', delay
         Read (11, '(a)') inline
            CALL Fi1R (inline, ch_width)
            print *, ' read width = ', ch_width
         Read (11, '(x)') ! time preset
         Read (11, '(x)') ! Exp pars
         Read (11, '(x)') ! --------
         Read (11, '(19x,a17)') LongTit
            print *, ' read long title : ', LongTit(1:40)
         Read (11, '(x)') ! experimentateurs
         Read (11, '(23x,d8.0)') WaveL
            print *, ' read wavelength = ', WaveL
         Read (11, '(x)') ! geom
         Read (11, '(x)') ! angle
         Read (11, '(23x,d3.0)') fre_chops
         Read (11, '(23x,d1.0)') ratio_ch4
         Read (11, '(23x,d1.0)') ratio2
         channels = 512
         Read (11, '(30x,i2)')   nKentry
            print *, ' read nK = ', nKentry
         nK = nKentry
         Read (11, '(x)')
      ELSE
         ! d10.0 reads any real number which contains a decimal point
      Read (11, '(x)')                              ! three empty lines
      Read (11, '(x)')
      Read (11, '(x)')
      Read (11, '(37x,i5)')       jrun ! normally, it's i4.
      Read (11, '(x)')
      Read (11, '(x)')                              ! start time
      Read (11, '(6x,24x,x)')                       ! # subruns
      Read (11, '(6x,24x,a)')     LongTit           ! sample name
c      Print *, ' reading .ust titled ', LongTit(1:lenU(LongTit))
      Read (11, '(6x,24x,x)')                       ! transmission
      Read (11, '(6x,24x,d10.0)') WaveL             ! lambda
      Read (11, '(6x,24x,x)')                       ! container
      Read (11, '(6x,24x,d10.0)') Temp              ! sample temperature
      Read (11, '(6x,24x,x)')                       ! sample angle
      Read (11, '(6x,24x,d10.0)') ch_width          ! time channel width
      Read (11, '(6x,24x,d10.0)') delay   !(10-7sec)! time offset T_0
      Read (11, '(6x,24x,x)')                       ! timer preset
      Read (11, '(6x,24x,x)')                       ! monitor preset
      Read (11, '(6x,24x,d10.0)') channels          ! # tof channels
c      Print *, ' #channels is ', channels
      Read (11, '(6x,24x,i10)')   nKentry           ! # det groups
      nK = nKentry ! Ablesen aus Winkeltabelle waere auch nicht besser
c      Print *, ' #spectra is ', nK
      Read (11, '(6x,24x,d10.0)') fre_chops         ! frequency choppers
         ! 1/2 RPM choppers 1,2,5,6 (2 trous)
      Read (11, '(6x,24x,d10.0)') ratio_ch4         ! ratio ch4 / others
      Read (11, '(6x,24x,x)')                       ! user name
      Read (11, '(x)')
      ENDIF

C  Read the last block which contains the scattering angles :
      DO il = 1, MK/10+1
         Read (11, '(10f6.2)', end=211) (Angle(ii), ii=il*10-9, min0(nK,il*10))
         ENDDO 
c      GOTO 212
 211  CONTINUE
c      Fehler = 'unexpected end-of-list in angle table'
c      RETURN
c 212  CONTINUE
      Close (11)

C  There is always a meaningless first entry in Angle :
      nK = nK - 1
      DO K = 1, nK
         Angle (K) = Angle (K+1)
         ENDDO

C  There may be empty angles at the end :
      DO WHILE (Angle(nK).le.0.d0) 
         Print *, 'WARNING: unexpected zero angle at nK = ', nK
         IF (nK.lt.0) STOP
         nK = nK - 1
         ENDDO 

C  Tell :
      CALL Say2 (' Found '//cl3(nK), ' spectra as set in table')

C  Checks :
      IF    (jrun.ne.irun) THEN
         Print *, '#run : given/read ', irun, jrun
         Fehler = 'RRT_In_Mib/.ust/ Bad number of run'
         RETURN 
      ELSEIF (nint(channels).ne.512) THEN
         Print *, 'channels = ', channels
         Fehler = 'RRT_In_Mib/.ust/ #channels<>512)'
         RETURN 
      ELSEIF (qroutside(fre_chops,3.d2,3.d4)) THEN
         IF (fre_chops.ge.3.d4) THEN
            Fehler = 'RRT_In_Mib/.ust/ Chopper frequency must be in Hz'
            RETURN 
         ELSE 
            Print *, 'Chopper frequency times 60 ...'
            fre_chops = fre_chops * 60
            ENDIF
      ELSEIF (qroutside(ratio_ch4,1.d0,1.d1)) THEN
         CALL Gong (4)
         Print *, ' definition of "frequency chopper 4" has changed'
         Print *, ' it is now the ratio of frequencies (ch4/others)'
         ratio_ch4 = fre_chops / ratio_ch4
         IF (qroutside(ratio_ch4,1.d0,1.d1)) THEN
            Fehler = 'RRT_In_Mib/.ust/ value ratio_ch4 outside 1..10'
            RETURN 
            ENDIF 
         Print *, ' assuming it is meant ', ratio_ch4
         ENDIF

C  Get ToF parameters :
      Cwidth  = ch_width / 10                 ! in usec
      Period  = 60.d6 * ratio_ch4 / (2*fre_chops) / Cwidth
                                              ! (usec/rot)/(usec/ch)

C  Read *.asc :
      DO K = 1,nK
         Read(12,'(8f10.0)')  (Y(i),i=1,512)
            ! En realite le format d'ecriture est 8i10
         IF (K.eq.1) rMon = Y (509)
         CALL OlfPutSpe (j, K, 1, 0.d0, 508, X, Y, D, Fehler)
            ! ch. 509-512 are used for monitors, time, ?
         IF (Fehler.ne.'&ff') RETURN
         ENDDO
      Close (12)

C  Instrument parameters : detector efficiency, flight path
      DO K = 1, nK
         DetE1(K) =  0. 
         DetE2(K) = -2.822
         ! dist_4_s   = 2.81d0 ! chopper4 - sample   ! not used at all
         FPath(K) = 3.58d0     ! sample - detectors
         ENDDO

      END ! RRT_In_Mib

C  --------------------------------------------------------------------
      SUBROUTINE RRT_In_Fcs (DirRaw, irun, j,  
     *                  rMon, Temp, WaveL, Cwidth, Period, LongTit,
     *                  nK, Angle, FPath, DetE1, DetE2, Fehler)
C  --------------------------------------------------------------------
         !  by Andreas Meyer, Feb. 1998
         !  NIST fermi choper tof

      IMPLICIT REAL*8  (a-h,o-p,r-z)
      IMPLICIT LOGICAL (q)
      INCLUDE      'i_dim.f'
      INCLUDE      'i_wrk.f'
      DIMENSION     Angle(*), FPath(*), DetE1(*), DetE2(*)

      CHARACTER*(*) LongTit, Fehler
      CHARACTER*80  inline, aus
      CHARACTER*40  Fileida
      CHARACTER     cl5*5
      CHARACTER*1   dettyp
      INTEGER       detno,detgrp
      REAL          detangle, detdist


      Fehler = '&ff' ! 

C  The following variables are no longer read in :
      nC         = 1024       ! # of tof channels
      nK         = 63         ! # of spectra

C  Open the input files :

      IF (irun.lt.1 .or. irun.gt.99999) THEN
         CALL Absturz ('RRT_In_Fcs', 'Bad number of run : '//cl5(irun))
         ENDIF

      CALL Compose2 (Fileida, 'f'//cl5(irun), '.ida')
 
      CALL OpenFile (11, Fileida, '&noext', 'a', Fehler)

      IF (Fehler.ne.'&ff')
     *   CALL Absturz ('RRT_In_Fcs', 'Could not open file '//Fileida)


C  Read Usertable--File Header :
         ! d10.0 reads any real number which contains a decimal point
         ! f10.0 reads any integer number and transfer into a real one

      Read (11, '(x)')                          ! three lines not read in
      Read (11, '(x)')
      Read (11, '(x)')
      Read (11, '(14x,a)')     LongTit          ! sample name and comment
      Read (11, '(x)')
      Read (11, '(20x,d10.0)') rMon             ! Monitor
      Read (11, '(x)')
      Read (11, '(20x,d10.0)') WaveL            ! wavelength in AA
      Read (11, '(20x,d10.0)') Cwidth           ! channel width in musec
      Read (11, '(20x,d10.0)') Period           ! Period in Hz
      Read (11, '(20x,d10.0)') Temp             ! sample temperature (K)
      Read (11, '(x)')
      Read (11, '(x)')

      Period = 1.0 / Period * 1000000.0  ! Period in musec

         Print *, ' sample: ',LongTit
         Print *, ' temperature    = ',Temp
         Print *, ' monitor        = ',rMon
         Print *, ' wavelength     = ',WaveL
         Print *

C  Read *.ida :
      IF (MK.lt.nK) THEN
         CALL Absturz ('RRT_In_Fcs', 'Recompile with MK >= 63')
         ENDIF

C  Detector angles and efficiency
      DO K = 1,nK
         Angle(K) = 0.0
      ENDDO

      DO K = 1, nK
         Read(11, '(1x,i3,1x,f8.4,1x,f8.1,2x,a1,3x,i2)')
     *       detno,detangle,detdist,dettyp,detgrp
         IF (detgrp.lt.37) THEN
           Angle(K) = detangle - 0.6363
           Read (11, '(x)') 
         ELSEIF (detgrp.eq.37) THEN
           Angle(K) = detangle
         ELSEIF (detgrp.gt.37 .and. detgrp.lt.47) THEN
           Angle(K) = detangle - 0.6363
           Read (11, '(x)') 
         ELSE
           Angle(K) = detangle
         ENDIF
         FPath(K) = detdist * 0.001   !  in meter
C  Detector effeciency
         DetE1(K) = -0.0806           !  cylindric d=25.4mm, 4bar 
         DetE2(K) = -5.306            !  deff=20mm 

      ENDDO   

C  Data
      IF (MC.lt.nC) THEN
         CALL Absturz ('RRT_In_Fcs', 'Recompile with MC >= 1024')
      ENDIF

      Read (11,'(129(/),x)') ! Monitor in first block

      Nmax = min0(int(Period /Cwidth), nC)
      DO K = 1,nK
         Read(11,'(x)')
         DO l = 0,127
           Read(11,'(7(d11.0,1x),d11.0)')  (Y(i+l*8), i=1,8)
         ENDDO
         CALL OlfPutSpe (j, K, 1, 0.d0, Nmax, X, Y, D, Fehler)
         IF (Fehler.ne.'&ff') RETURN
         ENDDO

      Close (11)

      END ! RRT_In_Fcs

C  --------------------------------------------------------------------
      SUBROUTINE RRT_EPeak_FCS (j, nK, Angle, FPath, Xep, 
     *           WaveL, Eelast, Cwidth, Period, Fehler)
C  --------------------------------------------------------------------
         !  by A. Meyer, Feb. 1998 
         !  special treatment for NIST fermi chopper tof
         !  shift elastic peak to correct position

      IMPLICIT REAL*8  (a-h,o-p,r-z)
      IMPLICIT LOGICAL (q)
      INCLUDE          'l_def.f'
      INCLUDE          'i_dim.f'
      INCLUDE          'i_wrk.f'
      CHARACTER*(*)    Fehler
      DIMENSION        Angle(*), FPath(*), Xep(*)

      nC = 1024
      
      DO K = 1, nK
         Nelist = int(Xep(K))

         CALL OlfGetXYD (j, K, n, X, Y, D, Fehler)
         IF (Fehler.ne.'&ff') RETURN

         Telast = FPath(K) / 3956.d-6 * WaveL   ! in usec 

         Nmax = min0(int(Period /Cwidth), nC)
         Nfend= min0(Nmax - int(Telast / Cwidth) + Nelist, Nmax-1)  

C  Wrap around :
         DO i = 1, nC 
            Y2(i) = Y(i)
            D2(i) = D(i)
            ENDDO
         DO i = 1,Nfend 
            Y(i + Nmax - Nfend) = Y2(i)
            D(i + Nmax - Nfend) = D2(i)
            ENDDO
         DO i = Nfend+1, Nmax
            Y(i - Nfend) = Y2(i)
            D(i - Nfend) = D2(i)
            ENDDO

         CALL OlfPutXYD (j, K, Nmax, X, Y, D, Fehler)
         IF (Fehler.ne.'&ff') RETURN

         ENDDO ! K

      END ! RRT_EPeak_FCS

C  --------------------------------------------------------------------
      SUBROUTINE RRT_Check (what,
     *                Temp, Eelast, WaveL, Cwidth, Period,
     *                nK, Angle, FPath, DetE1, DetE2, 
     *                Temp1, Eelast1, WaveL1, Cwidth1, Period1,
     *                nK1, Angle1, FPath1, DetE11, DetE21, 
     *                Fehler)
C  --------------------------------------------------------------------

      IMPLICIT LOGICAL (q)
      IMPLICIT REAL*8  (a-h,o-p,r-z)

      INCLUDE      'l_def.f'
      CHARACTER*(*)     Fehler, what
      REAL*8            Angle(*), FPath(*), DetE1(*), DetE2(*),
     *                  Angle1(*), FPath1(*), DetE11(*), DetE21(*)

      IF (nK1.ne.nK) THEN
         CALL Compose2 (Fehler,
     *                  what//'/ no. of data blocks '//cl4(nK), '<>'//cl4(nK1))
         RETURN
         ENDIF
      DO K = 1, nK
         IF (dabs(Angle1(K)-Angle(K)).ge.1.d0) THEN
            Fehler = what//'/ angles too different'
            RETURN
            ENDIF
         ENDDO

      IF (.not.qEqTol(Eelast1, Eelast, 5.d-2)) THEN
         Fehler = what//'/ wavelengths too different'
         RETURN
         ENDIF

      END ! RRT_Check

C  --------------------------------------------------------------------
      SUBROUTINE RRT_EPeak (j, nK, Angle, Xep, Yep, Bad, Fehler)
C  --------------------------------------------------------------------
             ! JWu 30jan93 : rewritten, using M.Bee's RRT_In_Mib

         ! calculates elastic peak area (mainly for use with vanadium),
         ! and peak position (mainly for use with sample).

      IMPLICIT REAL*8  (a-h,o-p,r-z)
      INCLUDE      'l_def.f'
      INCLUDE      'i_dim.f'
      INCLUDE      'i_wrk.f'
      CHARACTER*(*) Fehler
      REAL*8        Xep(*), Yep(*), Angle(*)
      CHARACTER*1   Bad(*)
      CHARACTER     aus*80

C  First pass : rough estimate of elastic channel :
      Write (35, '(a)') '  first pass'
      nK1    = 0
      xmep1  = 0.
      ymep1  = 0.
      DO K = 1, nK
         CALL OlfGetXYD (j,  K, n, X, Y, D, Fehler)
         IF (Fehler.ne.'&ff') RETURN

         IF (Bad(K).eq.' ') THEN
            IF (Angle(K).le.0.d0) Fehler = 
     *   'EPeak/ Invalid angle (should have been detected earlier), K ='
     *   //cl4(K)
            ! find maximum :
            ima = 1
            yma = 0.
            DO i = 1, n
               IF (Y(i).gt.yma) THEN
                  ima = i
                  yma = Y(i)
                  ENDIF
               ENDDO
            IF (yma.eq.0.) THEN
               Write (*, *) '  > no counts in data block ', K
               Write (35,*) '  > no counts in data block ', K
               Bad(K) = '0' ! empty
            ELSEIF (yma.le.0.) THEN
               Write (*, *) '  > negative counts in data block ', K
               Write (35,*) '  > negative counts in data block ', K
               Bad(K) = '-' ! negativ
            ELSE
               Xep(K) = dflotj (ima)
               Yep(K) = Y(ima)
               xmep1 = xmep1 + ima * Y(ima)
               ymep1 = ymep1 +       Y(ima)
               nK1  = nK1  + 1
               ENDIF
            ENDIF
         ENDDO
      xmep1 = xmep1 / ymep1 ! weighted average
      ymep1 = ymep1 / nK1

C  Second pass : check maximum, sum over region around it :
      ! initial tolerance (10*final) :
      xtol  = iExeMLP('tof-epeak-xtol-i')
      ytol  = iExeMLP('tof-epeak-ytol-i')
      xtolf = iExeMLP('tof-epeak-xtol-f')
      ytolf = iExeMLP('tof-epeak-ytol-f')
      IF (xtol.lt.1 .or. ytol.lt.1 .or. xtolf.lt.1 .or.ytolf.lt.1) THEN
         Fehler = 'Please set tolerances in ida.su'
         RETURN
         ENDIF
      Write (35, '(a)') '  second pass'
 21   CONTINUE
      nK2    = nK1
      nK1    = 0
      xmep2  = 0.
      ymep2  = 0.
      DO K = 1, nK
         IF (Bad(K).eq.' ') THEN
            ! check height of max :
            IF (Yep(K).lt.ymep1/ytol) THEN
               Write (aus, '(a,i3,a,g8.3,a,g8.3)')
     *            '  > peak too small in spectrum ', K, ': ',
     *            Yep(K), 'vs mean ', ymep1
               Write (* , '(a)') aus
               Write (35, '(a)') aus
               Bad(K) = 'v'  !  vanishing
               GOTO 29
               ENDIF

            IF (Yep(K).gt.ymep1*ytol) THEN
               Write (aus, '(a,i3,a,g8.3,a,g8.3)')
     *            '  > elefantic peak in spectrum ', K, ': ',
     *            Yep(K), 'vs mean ', ymep1
               Write (* , '(a)') aus
               Write (35, '(a)') aus
               Bad(K) = 'g'  !  giant
               GOTO 29
               ENDIF

            ! compare individual to average peak position :
            IF (dabs(Xep(K)-xmep1).gt.xtol) THEN
               Write (aus, '(a,i3,a,g8.3,a,g8.3)')
     *            '  > excentric peak in spectrum ', K, ': ', 
     *            Xep(K), 'vs mean ', xmep1
               Write (* , '(a)') aus
               Write (35, '(a)') aus
               Bad(K) = 'e' !  excentric
               GOTO 29
               ENDIF

            ! determine resolution width :
            CALL OlfGetXYD (j,  K, n, X, Y, D, Fehler)
            IF (Fehler.ne.'&ff') RETURN
            yhalf = Yep(K) / 2
            imax = idnint (Xep(K))
            DO ich=imax-1,1,-1
            IF (Y(ich).lt.yhalf .or. ich.eq.1) then
               ileft=ich
               goto 1                     ! Sortie mi-hauteur a gauche
               ENDIF
            ENDDO
 1          CONTINUE
            DO ich=imax+1, n
               IF (Y(ich).lt.yhalf .or. ich.eq.n) then
               iright=ich
               goto 2                      ! Sortie mi-hauteur a droite
               ENDIF
            ENDDO
 2          CONTINUE
            nfwhm = iright-ileft+1      ! Nombre de points a mi-hauteur
            nfwB  = min0 (  nfwhm, imax-1, n-imax)   ! pour barycentre
            nfwC  = min0 (2*nfwhm, imax-1, n-imax)   ! pour integrale
            ilB = iinside (imax-nfwB    , 1, imax)   ! pour barycentre
            irB = iinside (imax+nfwB    , imax, n)
            ilC = iinside (imax-nfwC    , 1, imax)   ! pour integrale
            irC = iinside (imax+nfwC    , imax, n)
            IF (irB.lt.ilB .or. irC.lt.ilC) THEN
               Print *, ' problems with peak: K imax ileft iright : ',
     *                 K, imax, ileft, iright
               STOP
               ENDIF

            ! Barycenter :
            x_peak   = 0.
            s_peak   = 0.
            DO ich=ilB, irB                ! Boucle sur canaux du pic
            x_peak   = x_peak   + Y(ich) * ich
            s_peak   = s_peak   + Y(ich)
            ENDDO
            IF (s_peak.le.0.d0) THEN ! encountered by AToelle jun95
               Write (*,  '(a)') '  > negative peak in spectrum ', K
               Write (35, '(a)') '  > negative peak in spectrum ', K
               Bad(K) = '-' ! negativ
               ENDIF
            Xep(K) = x_peak/s_peak
            xmep2  = xmep2 + Xep(K)

            ! sum over peak :
            SX0 = 0.
            DO I = ilC, irC
               SX0 = SX0+Y(I)
               ENDDO
            Yep(K) = SX0
            ymep2  = ymep2+ Yep(K)
            nK1    = nK1  + 1
 29         CONTINUE
            ENDIF
         ENDDO

      IF (nK1.le.0) THEN 
         Write (* , '(a)') '  > no spectrum has good shape'
         Write (35, '(a)') '  > no spectrum in good shape/'//
     *                     ' mean peak position undefined'
         RETURN 
         ENDIF 

      xmep1 = xmep2 / nK1
      ymep1 = ymep2 / nK1

      IF (nK1.lt.nK2) GOTO 21 ! averages changed -> repeat

      ! decrease tolerance step by step :
      xtol = xtol/1.2
      ytol = ytol/1.2
      IF (xtol.lt.xtolf .and. ytol.lt.ytolf) RETURN

      xtol = dmax1 (xtol, xtolf)
      ytol = dmax1 (ytol, ytolf)

      END ! Analysis

C  --------------------------------------------------------------------
      SUBROUTINE RRT_Energ (qOutE, iSEG, Xep, Bad, Angle, FPath,
     *                   iFmod, NooW, NooD,
     *                   EFmax, EWmax, EFminRel,
     *                   WaveL, Eelast, Cwidth, Period,
     *                   j, nK, Fehler)
C  --------------------------------------------------------------------
                 ! rewritten JWu 22/23jun92, X=tof option 30apr93
         ! shift elastic peak   (from Xep = XepS or XepV)
         ! shift frame overlap  (par. EFmax, EWmax, EFminRel)
         ! convert to energy    (if qOutE)

      IMPLICIT REAL*8  (a-h,o-p,r-z)
      IMPLICIT LOGICAL (q)
      INCLUDE      'l_def.f'
      INCLUDE      'i_dim.f'
      INCLUDE      'i_wrk.f'

      CHARACTER*(*) Fehler
      REAL*8        Xep(*), Angle(*), FPath(*), EFmin, Eelast, EFmax, EWmax
      CHARACTER*1   Bad(*)

      DO K = 1, nK
         CALL OlfGetXYD (j, K, n, X, Y, D, Fehler)
         IF (Fehler.ne.'&ff') RETURN

         Telast = FPath(K) / 3956.d-6 * WaveL ! in usec ! TBM

C  Prepare determination of the frame :
      IF     (iFmod.eq.1) THEN
         ! Given frame (neutrons belonging to one pulse)
         ! and window (channels on output) :
         EFmin  = EFminRel * Eelast
         TFmax  = Telast/(dsqrt(EFmin/Eelast+1))
         TFmin  = Telast/(dsqrt(EFmax/Eelast+1))
         TWmin  = Telast/(dsqrt(EWmax/Eelast+1))

            IF (K.eq.1111) THEN
               Print *
               Print *, ' Tela ', Telast
               Print *, ' Cwid ', Cwidth
               Print *, ' Peri ', Period
               Print *
               Print *, ' EFmi ', EFmin
               Print *, ' EFma ', EFmax
               Print *, ' EWma ', EWmax
               Print *
               Print *, ' TFma ', TFmax
               Print *, ' TFmi ', TFmin
               Print *, ' TWmi ', TWmin
               Print *
               ENDIF

      ELSEIF (iFmod.eq.3) THEN
         Fehler = 'RRT_Energ/ Option 3 ausser Betrieb'
         RETURN
c          ! determine absolute minimum :
c          jmia = 0
c          ymia = 1.d20
c          DO i = 1, n
c            yj = 0.
c            ! sum of all detectors, for one channel :
c            DO K = 1,nK
c               IF (Bad(K).eq.' ') yj = yj + Y(i)
c               ENDDO
c            IF (yj.lt.ymia) THEN
c               ymia = yj
c               imia = i
c               ENDIF
c            ENDDO
c         Print *, 'absolute minimum at channel '//cl6(jmia)
c         NooD = 0
c         NooF = max0 (1, imia-1)

         ENDIF ! iFmod

C  channel number of elastic peak :
         REPP = Xep(K)

C  Set the frame :
         IF    (iFmod.eq.1) THEN
            IF (K.eq.1111) THEN
               Print *, ' K    ', K
               Print *, ' REPP ', REPP
               Print *, ' ooF1 ', int(REPP-(Telast-TFmin)/Cwidth)+1
               Print *, ' ooW1 ', int(REPP-(Telast-TWmin)/Cwidth)+1
               ENDIF
            NooF = max0(int(REPP-(Telast-TFmin)/Cwidth)+1,0)
            NooW = max0(int(REPP-(Telast-TWmin)/Cwidth)+1,NooF)
               ! channels    1   .. NooF are moved around,
               ! channels NooF+1 .. NooW are thrown away
         ELSE
            NooW = NooF + NooD
            ENDIF

C  Move around :
         ! save first NooF time channels :
         DO i = 1,NooF
            Y2(i) = Y(i)
            D2(i) = D(i)
            ENDDO
         ! delete first NooW channels :
         DO i = 1+NooW, n
            X(i-NooW) =  i-REPP   ! distance from elastic peak, in channels
            D(i-NooW) =  D(i)
            Y(i-NooW) =  Y(i)
            ENDDO
         ! append the overlapping NooF channels at the end :
         DO i = 1, NooF
            X (n-NooW+i) = i-REPP + Period
            Y (n-NooW+i) = Y2(i)
            D (n-NooW+i) = D2(i)
            ENDDO
         nout = n - NooW + NooF
         ! eliminate neutrons that are too slow :
         DO WHILE (X(nout)*Cwidth.gt.TFmax) 
            nout = nout - 1
            ENDDO

C Set energies, and convert to S(2Th,w) :
         DO i = 1, nout
            IF (qOutE) THEN
               R1 = 1+Cwidth/Telast*X(i)
               E = Eelast*(1.d0/R1/R1-1)
               fac = .5*Telast/Eelast*R1*R1*R1/Cwidth*R1 !The last R1 for Ki/Kf
            ELSE ! channel number -> time-of-flight
               E = ( Telast + Cwidth*X(i) ) / FPath(K) / 1000.
                  ! units: (usec/m) -> (msec/m)
               fac = 1. ! no conversion, counts remain counts
               ENDIF
            X(i) = -iSEG * E      ! 7jul93. Vercors group has -(-1)
            D(i) = fac * D(i)
            Y(i) = fac * Y(i)
            ENDDO

         CALL OlfPutXYD (j, K, nout, X, Y, D, Fehler)
         IF (Fehler.ne.'&ff') RETURN

         ENDDO ! K

C  If channels were moved around :
      IF (NooF.GT.0) THEN
         Print *
         CALL Say2 ('  throw away '//cl6(NooW-NooF), ' high energy channels')
         CALL Say2 ('  move around '//cl6(NooF), ' channels')
         CALL Say2 ('  throw away '//cl6((n-nout)-(NooW-NooF)),
     *                   ' of them')
         CALL Say3 ('  thus retaining '//cl6(nout),
     *                   ' of '//cl6(n), ' original channels')
         ENDIF

      END ! EppCon

C  --------------------------------------------------------------------
      SUBROUTINE RRT_DetEff (j, nK, E0, C1, C2, Fehler)
C  --------------------------------------------------------------------
            ! Correct for the variation of detector efficiency with
            !   neutrons energy, according to the coefficients C1, C2
            !   which have to be preset in the input routine.
            ! The influence of the detector stainless steel walls
            !   which produce a cutoff around 4 Angstroms is neglected.

      IMPLICIT REAL*8  (a-h,o-p,r-z)
      INCLUDE      'i_dim.f'
      INCLUDE      'i_wrk.f'

      REAL*8        C1(*), C2(*), E, E0
      CHARACTER*(*) Fehler

      write (35,'(a3,6a10)') 'K', 'C1(K)', 'C2(K)', 'E0', 'X(1)', 'X(n)', 'eff0'

      DO K = 1, nK
         IF (C1(K).ge.0 .or. C2(K).ge.0) THEN
            write (35, '(i3,a,2(2x,g8.3))') K, C1(K), C2(K),
     *         ' no detector efficiency correction (parameters not set)'
         ELSE
            CALL OlfGetXYD (j, K, n, X, Y, D, Fehler)
            IF (Fehler.ne.'&ff') RETURN
            eff0 = dexp1(C1(K)/dsqrt(E0)) * (1-dexp1(C2(K)/dsqrt(E0)))
            write (35, '(i3,6(2x,g8.3))') K, C1(K), C2(K), E0, X(1), X(n), eff0
            DO i = 1, n
               E = E0 + X(i)
               IF (E.gt.1.d-8) THEN 
                  eff = dexp1(C1(K)/dsqrt(E)) * (1-dexp1(C2(K)/dsqrt(E)))
               ELSE 
                  eff = 1
                  ENDIF 
               Y(i) = Y(i)*eff0/eff
               D(i) = D(i)*eff0/eff
               ENDDO
            CALL OlfPutXYD (j, K, n, X, Y, D, Fehler)
            IF (Fehler.ne.'&ff') RETURN
         ENDIF 

         ENDDO ! K

      END! DetEff

C  --------------------------------------------------------------------
      SUBROUTINE RRT_Norm (j, nK, YepV, BadV, BadS, Fehler)
C  --------------------------------------------------------------------
            ! Correct for the variation of detector efficiency with
            !   neutrons energy, according to the coefficients C1, C2
            !   which have to be preset in the input routine.
            ! The influence of the detector stainless steel walls
            !   which produce a cutoff around 4 Angstroms is neglected.

      IMPLICIT REAL*8  (a-h,o-p,r-z)
      INCLUDE      'i_dim.f'
      INCLUDE      'i_wrk.f'

      CHARACTER*1   BadV(*), BadS(*)
      REAL*8        YepV(*)
      CHARACTER*(*) Fehler

      DO K = 1, nK
         CALL OlfGetXYD (j,  K, n, X, Y, D, Fehler)

         IF     (BadV(K).ne.' ') THEN
            BadS(K) = 'n'
         ELSEIF (YepV(K).le.0) THEN
            Fehler = 'RRT_Norm/ van-int=0 not recognized in _EPeak'
            RETURN
         ELSE
            DO i = 1, n
               Y(i) = Y(i) / YepV(K)
               D(i) = D(i) / YepV(K)
               ENDDO
            ENDIF

         CALL OlfPutXYD (j, K, n, X, Y, D, Fehler)
         IF (Fehler.ne.'&ff') RETURN

         ENDDO ! K

      END ! RRT_Norm

C  --------------------------------------------------------------------
      SUBROUTINE RRT_Add (j, nK, Angle, Fehler)
C  --------------------------------------------------------------------
         ! Add spectra if angles are equal
            ! mrz95 by Andreas Meyer for data treatment of SIL tof spectra
            ! JWu 3mrz95 generalized for IN6 (3 angles may be equal)

      IMPLICIT REAL*8  (a-h,o-p,r-z)
      INCLUDE      'i_dim.f'
      INCLUDE      'i_wrk.f'

      REAL*8        Angle(*)
      CHARACTER*(*) Fehler

      K = 1
 300  CONTINUE 
         KK = K+1 ! next K (if nothing happens..)
         CALL OlfGetXYD (j, K, n, X, Y, D, Fehler)
         IF (Angle(K).le.0) GOTO 309
         DO KK = K+1, nK 
            IF (dabs(Angle(K)-Angle(KK)) .gt. Angle(K)/1000) GOTO 301
            ENDDO 
 301     CONTINUE ! found out that AngleS(K)=..=AngleS(KK-1)
         IF (KK.le.K+1) GOTO 309 ! angles are *not* equal
         ! sum   
         DO i = 1, n
            D(i) = D(i)**2
            ENDDO
         DO KKK = K+1, KK-1
            CALL OlfGetXYD (j, KKK, n, X1, Y1, D1, Fehler)
            DO i = 1, n
               Y(i) = Y(i) + Y1(i)
               D(i) = D(i) + D1(i)**2
               ENDDO
            Angle(KKK) = 0 ! spectrum will be deleted later
            ENDDO ! KKK
         DO i = 1, n
            Y(i) =       Y(i)  / (KK-K)
            D(i) = dsqrt(D(i)) / (KK-K)
            ENDDO
         Write (35,'(i1,a,2i4,g9.3)') KK-K, ' spectra summed: ', 
     *      K, KK, Angle(K)
         CALL OlfPutXYD (j, K, n, X, Y, D, Fehler)
         IF (Fehler.ne.'&ff') RETURN
 309     CONTINUE 
         IF (KK.le.nK) THEN 
            K = KK
            GOTO 300 
            ENDIF

      END ! RRT_Add

C  --------------------------------------------------------------------
      SUBROUTINE RRT_Sort (j, nK, Angle, Temp, qOutE, Fehler)
C  --------------------------------------------------------------------
         ! sort energies, eliminate spectra with angle=0
         ! set coordinate names 

      IMPLICIT LOGICAL (q)
      IMPLICIT REAL*8  (a-h,o-p,r-z) 
      INCLUDE      'l_def.f'
      INCLUDE      'i_dim.f'
      INCLUDE      'i_wrk.f'

      CHARACTER*(*) Fehler
      DIMENSION     Angle(*)

      Write (35, '(a)') ' RRT_Sort/ begin'
      jout = 0
      CALL OlfHeadDup (j, .false., jout, nK, Kout, Fehler)
      IF (Fehler.ne.'&ff') RETURN

      IF (qOutE) THEN
         CALL OlfCnuP (jout, 'x', 'w', 'meV', Fehler)
         CALL OlfCnuP (jout, 'y', 'S(2th,w)', 'meV-1', Fehler)
      ELSE
         CALL OlfCnuP (jout, 'x', 'tof', 'msec/m', Fehler)
         CALL OlfCnuP (jout, 'y', 'I(2th,tof)', 'cts', Fehler)
         ENDIF
      CALL OlfCnuP (jout, 'z1', '2th', ' ', Fehler)
      IF (Fehler.ne.'&ff') RETURN

      Kout = 0
      DO K = 1, nK
         IF (Angle(K).ne.0.) THEN
            CALL OlfGetXYD (j, K, n, X, Y, D, Fehler)
            IF (Fehler.ne.'&ff') RETURN
            ! sort spectrum :
            DO i = 1, n
               X1(n+1-i) = X(i)
               Y1(n+1-i) = Y(i)
               D1(n+1-i) = D(i)
               ENDDO
            Kout = Kout + 1
            CALL OlfPutSpe (jout, Kout, 1, Angle(K), n, X1, Y1, D1, Fehler)
            IF (Fehler.ne.'&ff') RETURN
            ENDIF
         ENDDO

      CALL OlfClos (jout, Kout, Fehler)
      IF (Fehler.ne.'&ff') RETURN

      CALL MemFileDel (j, Fehler)
      IF (Fehler.ne.'&ff') RETURN
      Write (35, '(a)') ' RRT_Sort/ end'

      END ! RRT_Sort

C  ====================================================================
C  i80 / 7 :   read HFBS data
C  ====================================================================

      SUBROUTINE RRT_In_Hfbs (Fehler)
C     ----------------------------
        ! Andreas Meyer 12/99 
      ! Read converted HFBS (NIST) data and normalize to monitor (FC) 

      IMPLICIT REAL*8   (a-h,o-p,r-z)
      IMPLICIT LOGICAL  (q)

      INCLUDE 'i_dim.f'
      INCLUDE 'i_wrk.f'
      INCLUDE 'l_def.f'

      CHARACTER*(*)  Fehler
      CHARACTER*40   Filehfbs
      CHARACTER*30   aus, File, Title, LongTit

      DATA Title / ' ' /, File / ' ' /

      Fehler = '&ff' ! 

         Print *, ' read energy converted HFBS data files '
      
 105  CONTINUE               ! outer loop

      irun = iAsk ('Run number')

      IF (irun.lt.0) GOTO 151
      
      IF (irun.lt.1 .or. irun.gt.999999) THEN
         CALL Absturz ('RRT_In_Hfbs', 'Bad number of run : '//cl6(irun))
         ENDIF

      CALL OlfCreate (jout, Kout, File, Title, Fehler)
      IF (Fehler.ne.'&ff') RETURN
      CALL tOlfG (jout, 'fil', File, Fehler)
      CALL tOlfG (jout, 'tit', Title, Fehler)
      IF (Fehler.ne.'&ff') RETURN


      CALL OlfCnuP (jout, 'x', 'w', 'ueV', Fehler)
      IF (Fehler.ne.'&ff') RETURN 

      CALL OlfCnuP (jout, 'y', 'norm. Int.', 'Cnts', Fehler)
      IF (Fehler.ne.'&ff') RETURN 

      CALL OlfCnuP (jout, 'z1', 'q', 'A-1', Fehler)
      IF (Fehler.ne.'&ff') RETURN 


C  The following variables are fixed :

      nK = 17   ! # of spectra: detectors + monitor
      nC = 1161 ! for Dec99 Dibut

C  Open the input files :

      CALL Compose2 (Filehfbs, 'd'//cl6(irun), '.con')
 
      CALL OpenFile (11, Filehfbs, '&noext', 'a', Fehler)

      IF (Fehler.ne.'&ff')
     *   CALL Absturz ('RRT_In_Hfbs', 'Could not open file '//Filehfbs)

      Read (11, '(21x,a)')        LongTit   ! sample name and comment
      Read (11, '(x)')
      Read (11, '(x)')                          
      Read (11, '(x)')
      Read (11, '(x)')
      Read (11, '(21x,i3)') Ncycl            ! number of cycles
      Read (11, '(x)')
      Read (11, '(x)')                          
      Read (11, '(x)')
      Read (11, '(x)')
      Read (11, '(x)')
      Read (11, '(23x,d13.0)') rMon          ! Monitor
      Read (11, '(x)')
C      Read (11, '(x)')
      
      DO K = 1, Ncycl 
       Read (11, '(x)')
       ENDDO

         Print *, ' sample:    ',LongTit
         Print *, ' monitor = ',rMon
         Print *, ' Cycles: ',Ncycl
         Print *
C  Data
      IF (MC.lt.nC) THEN
         CALL Absturz ('RRT_In_Hfbs', 'Recompile with MC >= nC')
      ENDIF

      DO K = 1,nK
      Read (11, '(20x,d10.0)') zAng            ! Angle
         zq = 2.0 * dsind(zAng/2.0)            ! Approximation
         DO i = 1,nC
           Read(11,'(d16.0,d16.0)')  X(i),Y(i)
           D(i) = sqrt(Y(i))
           Y(i) = Y(i) / rMon
           D(i) = D(i) /rMon
         ENDDO

         CALL OlfPutSpe (jout, K, 1, zq, nC, X, Y, D, Fehler)
         IF (Fehler.ne.'&ff') RETURN
         ENDDO

      Close (11)

      GOTO 105

 151  CONTINUE

      END ! RRT_In_Hfbs
