
// edif.cpp: edit on-line files (text_edit, dir, make, plot, ...)

#include <stdlib.h>
#include <stdio.h>

#include <string>
#include <iomanip>
#include <algorithm>

#include "workspace.h"
#include "cmd.h"
#include "asi.h"
#include "gar.h"
#include "edif.h"
#include "range.h"
// #include "dualplot.h"

using namespace std;

namespace NEdif {
//    CParam* RFind(CEle* e, CCoord OneR);
}

void NEdif::register_commands()
{
    theCommands.add_one( Cmd( &dir, "dw" ) );
    theCommands.add_one( Cmd( &dirZ, "dz" ) );
}

void NEdif::dir(void)
{
    WorkFileIterator iter( ANY_SLICE, &theWorkspace );
    WorkFile *pw;
    uint npts;
    string txtpts;
    while( iter( &pw ) ) {

        npts = pw->has_data() ? ((DataFile*)pw)->nT() : ((CurvFile*)pw)->nPar();
        if (npts!=(uint)-1) 
            txtpts = strg(npts);
        else
            txtpts = "?";

        char tmp[4];
        if( pw->ZCo.size()>1 )
            sprintf(tmp,"&%ic",pw->ZCo.size()-1);
        else
            strcpy(tmp,"");

        printf("%3d%1s%c%c %-16s %5d %-12s %3s %6s %-12s %-12s\n",
               pw->id,
               pw->as_on_disk ? "=" : " ",
               pw->has_data() ? valueTypeLetter( ((DataFile*)pw)->channelType( 0 ) ) : ' ',
               pw->has_data() ? valueTypeLetter( ((DataFile*)pw)->channelType( 1 ) ) : ' ',
               pw->name.substr(0,16).c_str(),
               pw->nK(), 
               pw->nZ() ? pw->ZCo[0].str().substr(0,12).c_str() : "",
               tmp,
               txtpts.c_str(),
               pw->xco.str().substr(0,12).c_str(),
               pw->yco.str().substr(0,12).c_str()
            );
    }
}

void NEdif::dirZ(void)
{
    WorkFileIterator iter( ANY_SLICE, &theWorkspace );
    WorkFile *pw;
    while( iter( &pw) ) {
        cout << "file " << pw->id << ":\n";
        printf( "   " );
        for( uint iz=0; iz<pw->ZCo.size(); ++iz ){
            printf( " %12s", pw->ZCo[iz].str().c_str() );
        }
        printf("%8s %38s\n", "#pts.", "x");
        for( uint j=0; j<pw->nK(); ++j ){
            printf("%3u", j);
            for( uint iz=0; iz<pw->ZCo.size(); ++iz ){
                cout << setw(12) << pw->V[j]->z[iz];
            }
            if( pw->styp()==DATA )
                printf( "%8u %.38s", P_SPEC(pw->V[j])->size(),
                        "range wieder ausgebaut" ); //P_SPEC(pw->V[j])->T[1]->dbl_range().c_str() );
            printf("\n");
        }
    }
}


/*
void NEdif::EditZ(void)
{
    ListZ();
    printf("Use oz+, mzd, mzx to modify\n");
}
*/

/*
void NEdif::ListZ(void)
{
    NOlm::IterateEle eiter;
    CEle *e;
    uint k, nz, i;
    string act;
    vector<CCoord> ZCo;

    if (!(e=eiter())) return;

    // Get current coordinates:
    ZCo = e->P()->ZCo;
    nz = ZCo.size();
    if (nz) 
        for (i=0; i<nz; ++i)
            printf("  z%u: %s\n", i, ZCo[i].str().c_str());
    else
        printf("  there are no z coordinates\n");
       
    // Check other input files:
    while((e=eiter())) {
        k = eiter.SelNo();
        VTEST ((e->P()->ZCo.size()!=nz), \
               ("! file %u has different number of z coordinates\n", k));

        for (i=0; i<nz; ++i)
            VTEST ((e->P()->ZCo[i]!=ZCo[i]), \
                   ("! file %u has different z coordinate %u\n", k, i));
    }

}
*/

/*
void NEdif::EditR(void)
{
}
*/

/*	
CParam* NEdif::RFind(CEle* e, CCoord OneR)
{
    for (uint i=0; i<e->P()->RPar.size(); ++i)
        if (e->P()->RPar[i].Co == OneR) return &(e->P()->RPar[i]);
    return 0;
}
*/

/*
void NEdif::DirCoord(void)
{
    NOlm::IterateEle eiter;
    CEle *e;
    string out;
    char   aux[4];
    while((e=eiter())) {
        sprintf( aux, "%3d", eiter.SelNo() );
        out = string( aux );
        out += " x: " + e->P()->xco.str();
        out += "  y: " + e->P()->yco.str();
        for (uint i=0; i<e->P()->nZ(); i++) {
            out += "  z" + strg(i) + ": " + e->P()->ZCo[i].str() + "\n";
        }
        cout << out;
    }
}
*/

/*
void NEdif::EditCoord( string which )
{
    NOlm::IterateEle eiter;
    CEle *ein, eout;
    COlo *fin, *fout;
    CRef cref( which );
    if ( !cref.defined() ) {
        printf( "! invalid coordinate %s\n", which.c_str() );
        return;
    }
    CCoord new_co, old_co;
    new_co.Ask( (which+" coordinate").c_str() );
    if (!NOlm::overwrite) NOlm::SelNew();
    while( (ein=eiter()) ) {
        fin = ein->P();
        if( !(NOlm::overwrite) ){ 
            NOlm::OloDup( fin, &eout );
            fout = eout.P();
            *fout = *fin;
        } else {
            fout = fin;
        }
                              
        if        ( cref.var==CRef::_X ) {
            old_co = fin->xco;
            fout->xco = new_co;
        } else if ( cref.var==CRef::_Y ) {
            old_co = fin->yco;
            fout->yco = new_co;
        } else if ( cref.var==CRef::_Z ) {
            if( cref.num >= fin->nZ() ){
                printf( "! not all files in selection have %s, use oz+\n",
                        which.c_str() );
                return;
            }
            old_co = fin->ZCo[cref.num];
            fout->ZCo[cref.num] = new_co;
        }
        fout->lDoc.push_back( "coordinate "+string(which)+" renamed from "+
                              old_co.str()+" into "+new_co.str() );
        if(!(NOlm::overwrite)) NOlm::OloPushBack( &eout );
    }
}
*/

/*
void NEdif::EditDoc(void)
{
    NOlm::IterateD fiter;
    COld *f;
    uint i;
    while((f=fiter())) {
        cout << "file " << fiter.SelNo() << " has doc-lines:\n";
        for (i=0; i<f->lDoc.size(); i++)
            cout << f->lDoc[i] << "\n";
    }
}
*/

/*
void NEdif::Save( string fmt )
{
    COlo *f;
    string fnam;
    FILE *fd;
    int err;

    CEle *e;
    NOlm::IterateEle iter;
    while((e=iter())) {

        if ( !(f = e->P()) ) {
            printf("SEVERE / f ?\n");
            return;
        }
        string quest = "Save as (."+fmt+")";
        fnam = wask( quest.c_str(), f->name);
        if ( fnam==string("") ) return;
        if ( fmt=="a01" )
            f->name = fnam;
        fnam += "."+fmt;

        if ( (fd = fopen(fnam.c_str(), "r")) ) {
            if (!bask("Overwrite ?")) return;
            fclose(fd);
        }
        if ( !(fd = fopen(fnam.c_str(), "w")) ) {
            printf("! cannot write to file %s\n", fnam.c_str());
        }

        err = e->Save( fd, fmt );

        fclose(fd);

        if (err) return;
    }
}
*/


/*
void NEdif::ReadIn(void)
{
    COld olf;
    static CCoord xco, yco, zco;
	
    static string fnam;
    olf.name = wask("Save as", fnam);

    // *** set coordinates ***
    olf.xco = xco.Ask("x", ASK_DEF_GIVEN, xco);
    olf.yco = yco.Ask("y", ASK_DEF_GIVEN, yco);
    cout << "if there is only one scan, answer \"\\\" to the following:\n";
    zco.Ask("z", ASK_DEF_GIVEN, zco);
    if(zco.defined()) olf.ZCo.push_back(zco);

    // *** set input mode ***
    cout << "input modes:\n"
        "  (1) enter x pointwise\n"
        "  (2) set regular x values\n"
        "  (3) enter x-y pairs\n"
        "  (4) select x,y from multicolumn input\n";
    static int mInpX = -1;
    mInpX = iask_lim("Option", mInpX, CLimits(0,4));

    static int imcx, imcy; 
    int mInpY=-1; // ask
	
    switch(mInpX) {
    case 0: 
        return;
    case 1: case 2:
        break;
    case 3:
        imcx = 0;
        imcy = 1;
        mInpY = 0;
        break;
    case 4:
        cout << "ATTENTION: columns counting from 0\n";
        imcx = iask("x from column #", imcx);
        if (imcx<0) return;
        imcy = iask("y from column #", imcy);
        if (imcy<0) return;
        mInpY = 0;
        break;
    default:
        cout << "PROGRAM ERROR in Make: invalid switch(mInpX)\n";
        return;
    }
			
    if (mInpY<0) {
        cout << "input modes:\n"
            "  (1) enter y pointwise\n"
            "  (2) set regular y values\n"
            "  (3) select y from multicolumn input\n";
        static int mInpY_in = -1;
        mInpY_in = iask_lim("Option", mInpY_in, CLimits(0,3));
        if (!mInpY) return;
        mInpY = mInpY_in;
    }

    switch(mInpY) {
    case 0: case 1: case 2:
        break;
    case 3:
        cout << "ATTENTION: columns counting from 0\n";
        imcy = iask("y from column #", imcy);
        if (imcy<0) return;
        break;
    default:
        cout << "PROGRAM ERROR in Make: invalid switch(mInpY)\n";
        return;
    }

    static int mInpZ;
    if (zco.defined()) {
        cout << "at present, we allow only _one_ z value per scan\n";
        cout << "input modes:\n"
            "  (1) enter z per spectrum\n"
            "  (2) select z from y column\n";
        mInpZ = iask_lim("Option", mInpZ, CLimits(0,2));
        if (!mInpZ) return;
    } else {
        mInpZ = 0;
    }

    switch (mInpZ) {
    case 0: case 1:
        break;
    case 2:
        if (! (mInpX==4 || mInpY==3)) {
            cout << "invalid combination of options:"
                " y is not read from multicolumn format\n";
            return;
        }
        break;
    default:
        cout << "PROGRAM ERROR in Make: invalid switch(mInpZ)\n";
        return;
    }

    static int imcy_delta=0; 
    static uint imcy_last;
    cout << "DEBUG "<< mInpZ << "\n";
    if (mInpZ==2) {
        imcy_delta = iask("step in number of y column", imcy_delta);
        if (imcy_delta) 
            imcy_last = iask("last y column", imcy_last);
    } else {
        imcy_delta = 0;
    }

    CScan S;
    static CGrid xgrid, ygrid;
    uint n = 0;
    uint imcm;
    string quest, line;
    double val;
    vector<double> linv;

    // *** loop over scans ***
    int ns = 0;
    while(1) {

        S.Clear();

        switch(mInpZ) {
        case 1:
            line = sask("Enter z value");
            // cout << "DEBUG: line [" << line << "]\n";
            if (mystd::str2vec(line, &linv, 2) < 0) {
                cout << "-> invalid line [" << line << "]\n";
                goto store;
            }
            // printf ("DEBUG: size = %d\n", linv.size());	
            if (linv.size()<1) {
                cout << "found no z in line [" << line << "] => assuming end-of-input\n";
                goto store;
            }
            // printf ("DEBUG: val = %g\n", linv[0]);	
            val = linv[0];
            break;
        case 2:
            line = sask("Enter header line with z values");
            if (mystd::str2vec(line, &linv, imcy+1)) {
                cout << "-> bad input\n";
                goto store;
            }
            if (linv.size()<=imcy) {
                cout << "-> not enough entries in header\n";
                goto store;
            }
            val = linv[imcy];
            break;
        }
        if (mInpZ)
            S.z.push_back(val);

        switch(mInpX) {
        case 1:
            cout << "Enter x values (or empty line to exit):\n";
            while (1) {
                quest = "[" + strg(n) + "] ";
                line = sask(quest.c_str());
                if (sscanf(line.c_str(), "%lg", &val)!=1) break;
                S.x.push_back(val);
                n++;
            }
            break;
        case 2:
            S.x.Ask("x");
            n = S.x.size();
            break;
        case 3: case 4:
            cout<<"Enter data columns (or empty line to exit):\n";
            imcm = (imcx>imcy) ? imcx : imcy;
            while (1) {
                quest = "[" + strg(n) + "] ";
                line = sask(quest.c_str());
                if (line==string("")) break;
                if (mystd::str2vec(line, &linv, imcm+1)) {
                    printf(
                        " could not extract x value no. %u"
                        " from string [%s]\n", imcm, line.c_str());
                    continue;
                }
                if (linv.size()<=imcm) {
                    cout << "-> not enough entries in line\n";
                    continue;
                }
                S.push_back(linv[imcx], linv[imcy]);
                n++;
            }
            break;
        }
        if (n<=0) return;

        switch(mInpY) {
        case 0:
            break;
        case 1:
            cout << "Enter " << n << " y values:\n";
            S.y.resize(n);
            for (uint i=0; i<n; i++) {
                quest = "x[" + strg(i) + "] = " +
                    strg(S.x[i]) + " -> ";
                do {
                    line = sask(quest.c_str());
                } while (sscanf(line.c_str(), "%lg", &val)!=1);
                S.y[i] = val;
            }
            break;
        case 2:
            ygrid.Ask("y", n);
            ygrid.AsVec(&(S.y));
            break;
        case 3:
            cout << "Enter " << n << " lines with y values:\n";
            S.y.resize(n);
            for (uint i=0; i<n; i++) {
                quest = "x[" + strg(i) + "] =" +
                    strg(S.x[i]) + " -> ";
                while (1) {
                    line = sask(quest.c_str());
                    if (mystd::str2vec(line, 
                                       &linv, imcy+1)) {
                        printf(
                            " could not extract y value no. %u"
                            " from string [%s]\n", imcy, line.c_str());
                        continue;
                    }
                    if (linv.size()<=imcy) {
                        cout << "-> not enough entries"
                            " in line\n";
                        continue;
                    }
                    break;
                }
                S.y[i] = linv[imcy];
            }
            break;
        }
        olf.VS.push_back(S);
        ns++;
		
        if (!mInpZ) goto store;
        if (imcy_delta) {
            imcy += imcy_delta;
            if (imcy_delta>0 && imcy>imcy_last ||
                imcy_delta<0 && imcy<imcy_last) goto store;
        }
    }
	
    // *** store new on-line file ***
 store:
    // cout << "DEBUG: store\n";
    if (ns<=0) return;
    // cout << "DEBUG: sel\n";
    NOlm::SelNew();
    // cout << "DEBUG: add\n";
    NOlm::OloAdd(&olf);
    // cout << "DEBUG: main\n";
}
*/

/*
void NEdif::ReadTab(void)
{
    COld olf;
    NOlm::SelNew();
        
    string inFile = wask("Read tab from file");
    FILE *fd;
    if (!(fd = fopen(inFile.c_str(), "r"))) {
        printf ("! cannot open file %s\n", inFile.c_str());
        return;
    }

    mystd::fname_divide( inFile, 0, &(olf.name), 0 );
    olf.lDoc.push_back( "data from table "+inFile );

    olf.xco = CCoord("x", "");
    olf.yco = CCoord("y", "");

    string lin;
    vector<double> dat, z123;
    int ni = 0;
    int nj, j0=0;
    int nz = 0;
    CScan S;
    while( mystd::freadln(fd, &lin)>0 ){
        if( lin[0]==':' ){ // multi-z header
            if( mystd::str2vec(lin.substr(1), &dat, 0, false) ) break;
            if( nz==0 ){ // first such header (must be first line of file)
                nz = dat.size() + 1;
                for( uint iz=0; iz<nz; ++iz )
                    olf.ZCo.push_back( CCoord("z"+strg(iz), "") );
            } else {
                j0 += nj;
                ni = 0;
            }
            if( dat.size()+1 != nz ){
                printf("read2tab header in line %i has length %i instead"
                       " of %i-1", ni, dat.size(), nz );
                return;
            }
            z123 = dat;
            continue;
        }
        
        if( mystd::str2vec(lin, &dat, 0, false) ) break;

        if( ni==0 ){ // first data line
            nj = dat.size();
            if( nz==0 ){
                nz = (nj>0) ? 1 : 0;
                if ( nz>0 ) olf.ZCo.push_back( CCoord("z", "") );
            } 
            for ( int j=0; j<nj; ++j ) {
                S.Clear();
                if( nz!=0 ) {
                    S.z.resize( nz );
                    S.z[nz-1] = j ;
                    for( uint iz=0; iz<nz-1; ++iz )
                        S.z[iz] = z123[iz];
                }
                olf.VS.push_back(S);
            }
        } else if( dat.size() != nj ){
            printf("read2tab line %i has length %i instead"
                   " of %i", ni, dat.size(), nj );
            return;
        }
        for( int j=0; j<nj; ++j ){
            olf.VS[j0+j].push_back( (double)ni, dat[j] );
        }
        ++ni;
    }

    NOlm::SelNew();
    NOlm::OloAdd(&olf);
}
*/

/*
void NEdif::MakeGrid(void)
{
    COld olf;

    static int ni=1, nj=1;
    static string fnam;

    int niIn = iask("Number of points per scan", ni);
    if (niIn<1) return;
    ni = niIn;

    int njIn = iask("Number of scans", nj);
    if (njIn<1) return;
    nj = njIn;

    fnam = "grid"+strg(ni);
    if (nj>1) fnam += "x"+strg(nj);

    olf.name = wask("Save as", fnam);

    // *** set coordinates ***
    olf.xco = CCoord("x", "");
    olf.yco = CCoord("y", "");
    if (nj>1)
        olf.ZCo.push_back(CCoord("z", ""));

    CScan S;

    // *** loop over scans ***
    for (int j=0; j<nj; ++j) {

        S.Clear();

        S.x = CGrid( ni, 0., 1. );
        S.y.clear();
        S.y.resize( ni, 0. );

        if (nj>1) S.z.push_back( (double)j );

        olf.VS.push_back(S);
    }
	
    NOlm::SelNew();
    NOlm::OloAdd(&olf);
}
*/
