// This is range.cpp.tpl, #include'd by range.cpp

T_CLASS::T_CLASS(void)
{
	// initialize as infinite
	flag = 1;
	inf = 0;
	sup = 0;
	valid = 1;
}
	
T_CLASS::T_CLASS(T_TYPE infin, T_TYPE supin)
{
	flag = 0;
	inf = infin;
	sup = supin;
	test();
}	

T_CLASS::T_CLASS(int flagin)
{
	flag = flagin;
	inf = 0;
	sup = 0;
	valid = 1;
}
	
T_CLASS::T_CLASS(int flagin, T_TYPE infin, T_TYPE supin)
{
	flag = flagin;
	inf = infin;
	sup = supin;
	test();
}	

T_CLASS::T_CLASS(const O_CLASS& In) // copy between class<double> and class<int>
{
	flag = In.flag;
	inf  = (T_TYPE) In.inf;
	sup  = (T_TYPE) In.sup;
	test();
}

char RangeDecode(const char *resp, T_CLASS *rval, const CAskPars *AP);

T_CLASS::T_CLASS(const string& text)
{
	T_CLASS rval;
	char ok;
	ok = RangeDecode(text.c_str(), &rval, 0);
	if (ok!='v') {
		flag = 1;
		inf = 0;
		sup = 0;
		valid = 0;
	} else {
		*this = rval;
	}
}

T_CLASS::~T_CLASS(void)
{
}

#ifdef _CRANGE
void T_CLASS::Round(int logflag, double relmargin, double digits)
{
	if (!test()) {
		cout << "WARNING by Range::Round: illegal params: "
			 << *this << "\n";
		return;
	}
	if        (inf==sup) {
		*this = T_CLASS(inf-1., sup+1.);
	} else if (finite()) {
		double margin = relmargin * (sup - inf);
		double mydigits = digits;
		T_TYPE inf2, sup2;
		do {
//			cout << "DEB Round loop " << sup << " " << inf << " : "
//			     << margin << " " << mydigits << "\n";
			if (mydigits>=8.5) return; // do not round
			sup2 = mystd::round(sup+margin, mydigits);
			inf2 = mystd::round(inf-margin, mydigits);
			if(sup<0 && sup2>0) sup2 = 0;
			if(inf2<0 && inf>0) inf2 = 0;
			// margin *= 2;
			mydigits += 0.5;
		      // cout << *this << " rounded -> " << T_CLASS(inf2, sup2);
		} while (!((inf2<inf) && (sup<sup2)));
		*this = T_CLASS(inf2, sup2);
	}
}	
#endif

bool T_CLASS::infinite() const
{
	return (flag&1);
}

bool T_CLASS::empty() const
{
	return (flag&2 || inf>sup || ((flag&4 || flag&8) && inf==sup));
}

bool T_CLASS::finite() const
{
	return (!this->infinite() && !this->empty());
}

bool T_CLASS::contained(double val)  const
{
	if (this->infinite())
		return 1;
	else if (this->empty())
		return 0;
	else
		return ((inf<val || (!(flag&4) && inf==val)) &&
			(sup>val || (!(flag&8) && sup==val)));
}

#ifdef _CLIMITS
bool T_CLASS::contained(int val) const
{
	if (this->infinite())
		return 1;
	else if (this->empty())
		return 0;
	else
		return ((inf<val || (!(flag&4) && inf==val)) &&
			(sup>val || (!(flag&8) && sup==val)));
}
#endif

bool T_CLASS::overlap(const T_CLASS& T2) const
{
	if      (empty() || T2.empty())
		return 0;
	else if (infinite() || T2.infinite())
		return 1;
	else
		return (contained(T2.inf) || contained(T2.sup) ||
			T2.contained(inf) || T2.contained(sup));
}

double T_CLASS::relval(int logflag, double v) const
{
	if (flag&1 || flag&2) return 0;
	if (logflag) {
		if(inf<0 || v<0) return 0;
		return (T_TYPE) 
		    (log(v/(double)inf) / log((double)sup/(double)inf));
	} else {
		return (v-inf) / (sup-inf);
	}
}

T_CLASS T_CLASS::operator*(const T_CLASS& R)
{
	T_CLASS Z;
	if        (infinite() || R.infinite()) {
		Z.flag = 1;
	} else if (R.empty()) {
		Z = *this;
	} else if (empty()) {
		Z = R;
	} else {
		// cout << " input: " << *this << " * " << R;
		Z = T_CLASS(min(inf, R.inf), max(sup, R.sup));
		// Z.flag4 = ..
		// Z.flag8 = ..
		// cout << " output : " << Z << endl;
	}
	return Z;
}

T_CLASS T_CLASS::operator*=(const T_CLASS& R)
{
	T_CLASS Z = *this * R;
	return *this = Z;
}
	
T_CLASS T_CLASS::operator+=(const T_TYPE shift)
{
	if (finite()) {
		inf += shift;
		sup += shift;
	}
	return *this;
}
	
T_CLASS T_CLASS::operator-=(const T_TYPE shift)
{
	return (*this += -shift);
}

T_CLASS T_CLASS::operator+(const T_TYPE shift)
{
	T_CLASS Z = *this;;
	return (Z += shift);
}

T_CLASS T_CLASS::operator-(const T_TYPE shift)
{
	T_CLASS Z = *this;
	return (Z -= shift);
}

bool T_CLASS::test(void) 
{
	return valid = (inf<=sup || flag&1 || flag&2);
}

const char * T_CLASS::helptext =
        T_INFO ":\n"
        "   -> <x> <y> for a closed interval (x y)\n"
	"   -> -       for an empty range\n"
	"   -> *       for an infinite range\n";

char RangeDecode(const char *resp, T_CLASS *rval, const CAskPars *AP)
{
	T_TYPE r1, r2;
	if (resp[0]=='\0') {
		return 'd';
	} else if (!strcmp(resp, "*")) {
		*rval = T_CLASS(1);
		return 'v';
	} else if (!strcmp(resp, "-")) {
		*rval = T_CLASS(2);
		return 'v';
	}
	string s1, s2;
        mystd::string_extract_word(string(resp), &s1, &s2);
	try{ 
		r1 = mystd::ANY2TYP(s1);
	        r2 = mystd::ANY2TYP(s2);
		*rval = T_CLASS(r1, r2);
		if (rval->test()) throw BadData("");
		return 'v';
	}
	catch(MyException &e){
		return 'h';	
 	}	
}

T_CLASS T_CLASS::Ask(const char* quest)
{
	return Ask(quest, 0, T_CLASS());
}

T_CLASS T_CLASS::Ask(const char* quest, const T_CLASS& Def)
{
	return Ask(quest, 1, Def);
}

T_CLASS T_CLASS::Ask(const char* quest, const bool defflag, const T_CLASS& Def)
{
	*this =  myask(string(quest), defflag, Def, 
		       helptext, RangeDecode, 0, 0);
	return *this;
}

string T_CLASS::str(void) const
{
	if      (!valid)
		return "<invalid range>";
	else if (finite()) 
		return ((flag&4) ? ")" : "(")
			+ strg(inf) + " " + strg(sup)
			+ ((flag&8) ? "(" : ")");
	else if (infinite())
		return "*";
	else
		return "-";
}

ostream& operator<< (ostream &s, const T_CLASS& R)
{
	if      (!R.valid)
		return s << "<invalid range>";
	else if (R.finite()) 
		return s << ((R.flag&4) ? ")" : "(")
			 << R.inf << " " << R.sup 
			 << ((R.flag&8) ? "(" : ")");
	else if (R.infinite())
		return s << "*";
	else
		return s << "-";
}
